SystemOrganization addCategory: #'Cog-Morphing Bytecode Set'!
SystemOrganization addCategory: #'Cog-Scripts'!
SystemOrganization addCategory: #'Cog-Scripting'!
SystemOrganization addCategory: #'Cog-Benchmarks'!
SystemOrganization addCategory: #'Cog-Processors'!
SystemOrganization addCategory: #'Cog-ProcessorPlugins'!
SystemOrganization addCategory: #'Cog-Processors-Tests'!
SystemOrganization addCategory: #'Cog-Tests'!

AbstractLauncher subclass: #CommandLineLauncher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Scripting'!

!CommandLineLauncher class methodsFor: 'activation' stamp: 'eem 10/23/2008 13:41'!
activate
	"Register this launcher with the auto start class"

	self autoStarter addLauncherFirst: self! !

!CommandLineLauncher class methodsFor: 'accessing' stamp: 'eem 10/23/2008 13:56'!
extractParameters

	| pName value index globals |
	globals := Dictionary new.
	index := 2.
	[pName := Smalltalk getSystemAttribute: index.
	pName isEmptyOrNil] whileFalse:[
		index := index + 1.
		value := Smalltalk getSystemAttribute: index.
		value ifNil: [value := ''].
 		globals at: pName asLowercase put: value.
		index := index + 1].
	^globals! !

!CommandLineLauncher class methodsFor: 'class initialization' stamp: 'eem 10/23/2008 13:45'!
initialize
	self activate! !

!CommandLineLauncher methodsFor: 'running' stamp: 'eem 10/23/2008 14:05'!
parameters: startupParameters
	"AnstractLauncher class>>extractParameters extracts from the wrong index for Mac OS"
	super parameters: self class extractParameters! !

!CommandLineLauncher methodsFor: 'running' stamp: 'eem 10/23/2008 14:09'!
startUp
	"UnixProcess stdOut print: parameters; nl."
	(parameters includesKey: '-doit') ifTrue:
		[Compiler evaluate: (parameters at: '-doit').
		 Smalltalk quitPrimitive]! !

!MethodContext methodsFor: '*Cog-Tests-xrays' stamp: 'eem 6/16/2009 18:52'!
xray
	"Lift the veil from a context and answer an integer describing its interior state.
	 Used for e.g. VM tests so they can verify they're testing what they think they're testing.
	 0 implies a vanilla heap context.
	 Bit 0 = is or was married to a frame
	 Bit 1 = is still married to a frame
	 Bit 2 = frame is executing machine code
	 Bit 3 = has machine code pc (as opposed to nil or a bytecode pc)
	 Bit 4 = method is currently compiled to machine code"
	<primitive: 213>
	^0 "Can only fail if unimplemented; therefore simply answer 0"! !

!MethodContext methodsFor: '*Cog-Tests-xrays' stamp: 'eem 6/16/2009 18:44'!
xrayIsDivorced
	^(self xray bitAnd: 3) = 1! !

!MethodContext methodsFor: '*Cog-Tests-xrays' stamp: 'eem 6/16/2009 18:52'!
xrayIsExecutingMachineCode
	^self xray anyMask: 4! !

!MethodContext methodsFor: '*Cog-Tests-xrays' stamp: 'eem 6/16/2009 18:43'!
xrayIsMarried
	^self xray anyMask: 2! !

!MethodContext methodsFor: '*Cog-Tests-xrays' stamp: 'eem 6/16/2009 18:52'!
xrayLastExecutedMachineCode
	^self xray anyMask: 8! !

!MethodContext methodsFor: '*Cog-Tests-xrays' stamp: 'eem 6/16/2009 18:53'!
xrayMethodIsCompiledToMachineCode
	^self xray anyMask: 16! !

Alien variableByteSubclass: #CogProcessorAlien
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Processors'!

!CogProcessorAlien commentStamp: 'lw 8/23/2012 19:15' prior: 0!
I am the superclass for the Simulation CPU instance wrappers. I ensure that methods used in both/all of them need not be copied.!

CogProcessorAlien variableByteSubclass: #BochsIA32Alien
	instanceVariableNames: ''
	classVariableNames: 'OpcodeExceptionMap PostBuildStackDelta'
	poolDictionaries: ''
	category: 'Cog-Processors'!

!BochsIA32Alien commentStamp: '<historical>' prior: 0!
I am a wrapper around the Bochs C++ IA32 CPU emulator.  Note that currently I provide no access to the x87/mmx FPU state, only providing access to the SSE/xmm registers.

Here is the configure script for the configuration this code assumes.  Offsets of fields will change with different configurations so they must agree.

----8<---- conf.COG ----8<----
#!!/bin/sh

# this sets up the compile for Cog.  Disable as much inessential stuff
# as possible leaving only the cpu/fpu & memory interface

set echo
# CFLAGS="-pipe -O3 -fomit-frame-pointer -finline-functions -falign-loops=16 -falign-jumps=16 -falign-functions=16 -falign-labels=16 -falign-loops-max-skip=15 -falign-jumps-max-skip=15 -fprefetch-loop-arrays $CFLAGS"
CFLAGS="-m32 $CFLAGS"
CFLAGS="-Dlongjmp=_longjmp -Dsetjmp=_setjmp $CFLAGS"
CFLAGS="-pipe -O3 -fomit-frame-pointer -finline-functions $CFLAGS"
CFLAGS="-g $CFLAGS"
CPATH="/sw/include"
CPPFLAGS=""
CXXFLAGS="$CFLAGS"
LDFLAGS="-L/sw/lib"

export CFLAGS
export CPATH
export CPPFLAGS
export CXXFLAGS
export LDFLAGS

./configure --enable-Cog \
	--enable-cpu-level=6 \
	--enable-sse=2 \
	--enable-assert-checks \
	--with-nogui \
		--disable-x86-64 \
		--disable-pae \
		--disable-large-pages \
		--disable-global-pages \
		--disable-mtrr \
		--disable-sb16 \
		--disable-ne2000 \
		--disable-pci \
		--disable-acpi \
		--disable-apic \
		--disable-clgd54xx \
		--disable-usb \
		--disable-plugins \
	${CONFIGURE_ARGS}

# apic == Advanced programmable Interrupt Controller
# acpi == Advanced Configuration and Power Interface
# pci == Peripheral Component Interconnect local bus
# clgd54xx == Cirrus Logic GD54xx video card
----8<---- conf.COG ----8<----!

!BochsIA32Alien class methodsFor: 'instance creation' stamp: 'eem 11/13/2011 11:13'!
dataSize
	^18096! !

!BochsIA32Alien class methodsFor: 'debugger' stamp: 'eem 2/14/2011 09:19'!
defaultIntegerBaseInDebugger
	^16! !

!BochsIA32Alien class methodsFor: 'class initialization' stamp: 'eem 5/7/2010 11:52'!
initialize
	"BochsIA32Alien initialize"
	PostBuildStackDelta := 0.
	OpcodeExceptionMap := Array new: 256 withAll: #handleExecutionPrimitiveFailureAt:in:readOnlyBelow:.
	OpcodeExceptionMap
		at: 1 + self basicNew callOpcode			put: #handleCallFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew jmpOpcode			put: #handleJmpFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew retOpcode			put: #handleRetFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew movALObOpcode	put: #handleMovALObFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew movObALOpcode	put: #handleMovObALFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew movGvEvOpcode	put: #handleMovGvEvFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew movEvGvOpcode	put: #handleMovEvGvFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew movGbEbOpcode	put: #handleMovGbEbFailureAt:in:readOnlyBelow:;
		at: 1 + self basicNew movEbGbOpcode	put: #handleMovEbGbFailureAt:in:readOnlyBelow:! !

!BochsIA32Alien class methodsFor: 'instance creation' stamp: 'eem 12/30/2008 07:53'!
new
	^(self atAddress: self primitiveNewCPU) reset! !

!BochsIA32Alien class methodsFor: 'primitives' stamp: 'eem 12/11/2008 13:33'!
primitiveNewCPU
	"Answer the address of a new Bochs C++ class bx_cpu_c/BX_CPU_C x86 CPU emulator instance."
	<primitive: 'primitiveNewCPU' module: 'BochsIA32Plugin'>
	^self primitiveFailed! !

!BochsIA32Alien class methodsFor: 'accessing' stamp: 'eem 3/30/2009 11:36'!
setStackAlignmentDelta: 	stackAlignmentDelta
	self assert: stackAlignmentDelta isPowerOfTwo.
	PostBuildStackDelta :=  stackAlignmentDelta > 8
								ifTrue: [stackAlignmentDelta - 8]
								ifFalse: [0]! !

!BochsIA32Alien methodsFor: 'Cog API' stamp: 'eem 12/20/2008 10:55'!
abstractInstructionCompilerClass
	^CogIA32Compiler! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 11:59'!
al
	^self eax bitAnd: 16rFF! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 12:01'!
al: aByte
	self eax: ((self eax bitAnd: -16rFF) + aByte).
	^aByte! !

!BochsIA32Alien methodsFor: 'Cog API' stamp: 'eem 1/21/2009 15:49'!
bitsInWord
	^32! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 11:59'!
bl
	^self ebx bitAnd: 16rFF! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 12:01'!
bl: aByte
	self ebx: ((self ebx bitAnd: -16rFF) + aByte).
	^aByte! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 1/17/2009 15:52'!
cResultRegister
	^self eax! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 1/17/2009 15:52'!
cResultRegister: aValue
	^self eax: aValue! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 12/30/2008 09:14'!
callOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2, pA8"
	^16rE8! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 11:59'!
cl
	^self ecx bitAnd: 16rFF! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 12:01'!
cl: aByte
	self ecx: ((self ecx bitAnd: -16rFF) + aByte).
	^aByte! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr0
	^self unsignedLongLongAt: 1009! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr0: anUnsignedInteger
	^self unsignedLongLongAt: 1009 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr1
	^self unsignedLongLongAt: 1013! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr1: anUnsignedInteger
	^self unsignedLongLongAt: 1013 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr2
	^self unsignedLongLongAt: 1017! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr2: anUnsignedInteger
	^self unsignedLongLongAt: 1017 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr3
	^self unsignedLongLongAt: 1021! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr3: anUnsignedInteger
	^self unsignedLongLongAt: 1021 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr4
	^self unsignedLongLongAt: 1029! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/16/2009 17:52'!
cr4: anUnsignedInteger
	^self unsignedLongLongAt: 1029 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 11:59'!
dl
	^self edx bitAnd: 16rFF! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 5/7/2010 12:01'!
dl: aByte
	self edx: ((self edx bitAnd: -16rFF) + aByte).
	^aByte! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
eax
	^self unsignedLongAt: 469! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
eax: anUnsignedInteger
	^self unsignedLongAt: 469 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
ebp
	^self unsignedLongAt: 489! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
ebp: anUnsignedInteger
	^self unsignedLongAt: 489 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 18:3'!
ebx
	^self unsignedLongAt: 481! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 18:3'!
ebx: anUnsignedInteger
	^self unsignedLongAt: 481 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 18:3'!
ecx
	^self unsignedLongAt: 473! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 18:3'!
ecx: anUnsignedInteger
	^self unsignedLongAt: 473 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
edi
	^self unsignedLongAt: 497! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
edi: anUnsignedInteger
	^self unsignedLongAt: 497 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 18:3'!
edx
	^self unsignedLongAt: 477! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 18:3'!
edx: anUnsignedInteger
	^self unsignedLongAt: 477 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
eflags
	^self unsignedLongAt: 513! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 9/18/2009 10:31'!
eflags: anUnsignedInteger
	^self unsignedLongAt: 513 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
eip
	^self unsignedLongAt: 501! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
eip: anUnsignedInteger
	^self unsignedLongAt: 501 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 12/28/2008 16:20'!
endianness
	^#little! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
esi
	^self unsignedLongAt: 493! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
esi: anUnsignedInteger
	^self unsignedLongAt: 493 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
esp
	^self unsignedLongAt: 485! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/11/2008 17:55'!
esp: anUnsignedInteger
	^self unsignedLongAt: 485 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 1/15/2009 16:09'!
flushICacheFrom: startAddress "<Integer>" to: endAddress "<Integer>"
	self primitiveFlushICacheFrom: startAddress To: endAddress! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 1/9/2009 09:20'!
fp
	^self ebp! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 12/6/2010 18:17'!
fp: anAddress
	"Set whatever the processor considers its frame pointer to anAddress."
	self ebp: anAddress! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 2/13/2009 16:00'!
frameBuildDeltaBytes
	"Answer how many words are pushed when a frame is built.  This is for the Cogit's stack alignment checking code."
	^4 "4 bytes pushed for $ebp"! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 5/7/2010 11:11'!
handleCallFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a call into a ProcessorSimulationTrap signal."
	|  relativeJump |
	relativeJump := memoryArray longAt: pc + 2 bigEndian: false.
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 5
			address: (pc + 5 + relativeJump) signedIntToLong
			type: #call)
		signal! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 10/21/2010 09:22'!
handleExecutionPrimitiveFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Handle an execution primitive failure for an unhandled opcode."
	^self reportPrimitiveFailure! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 10/21/2010 10:04'!
handleExecutionPrimitiveFailureIn: memoryArray "<Bitmap|ByteArray>" minimumAddress: minimumAddress "<Integer>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Handle an execution primitive failure.  Convert out-of-range call and absolute
	 memory read into register instructions into ProcessorSimulationTrap signals."
	"self printIntegerRegistersOn: Transcript"
	"self printRegistersOn: Transcript"
	| pc opcode |
	((pc := self eip) between: minimumAddress and: memoryArray byteSize - 1) ifTrue:
		[opcode := memoryArray byteAt: pc + 1.
		^self
			perform: (OpcodeExceptionMap at: opcode + 1)
			with: pc
			with: memoryArray
			with: minimumWritableAddress].
	^self reportPrimitiveFailure! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 5/7/2010 11:11'!
handleJmpFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a jmp into a ProcessorSimulationTrap signal."
	|  relativeJump |
	relativeJump := memoryArray longAt: pc + 2 bigEndian: false.
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 5
			address: (pc + 5 + relativeJump) signedIntToLong
			type: #jump)
		signal! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 5/7/2010 11:36'!
handleMovALObFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a read into eax into a ProcessorSimulationTrap signal."
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 5
			address: (memoryArray unsignedLongAt: pc + 2 bigEndian: false)
			type: #read
			accessor: #eax:)
		signal! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 2/16/2011 14:44'!
handleMovEbGbFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a byte register write into a ProcessorSimulationTrap signal."
	| modrmByte |
	^(((modrmByte := memoryArray byteAt: pc + 2) bitAnd: 16rC0) = 16r80) "ModRegRegDisp32"
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 6
					address: ((self perform: (#(eax ecx edx ebx esp ebp esi edi) at: (modrmByte bitAnd: 7) + 1))
							+ (memoryArray unsignedLongAt: pc + 3 bigEndian: false)
								bitAnd: 16rFFFFFFFF)
					type: #write
					accessor: (#(al cl dl bl ah ch dh bh) at: ((modrmByte >> 3 bitAnd: 7) + 1)))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 2/16/2011 14:44'!
handleMovEvGvFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a register write into a ProcessorSimulationTrap signal."
	| modrmByte |
	^((modrmByte := memoryArray byteAt: pc + 2) bitAnd: 16rC7) = 16r5 "ModRegInd & disp32"
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 6
					address: (memoryArray unsignedLongAt: pc + 3 bigEndian: false)
					type: #write
					accessor: (#(eax ecx edx ebx esp ebp esi edi) at: ((modrmByte >> 3 bitAnd: 7) + 1)))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 2/16/2011 14:44'!
handleMovGbEbFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a byte register load into a ProcessorSimulationTrap signal."
	| modrmByte |
	^(((modrmByte := memoryArray byteAt: pc + 2) bitAnd: 16rC0) = 16r80) "ModRegRegDisp32"
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 6
					address: ((self perform: (#(eax ecx edx ebx esp ebp esi edi) at: (modrmByte bitAnd: 7) + 1))
							+ (memoryArray unsignedLongAt: pc + 3 bigEndian: false)
								bitAnd: 16rFFFFFFFF)
					type: #read
					accessor: (#(al: cl: dl: bl: ah: ch: dh: bh:) at: ((modrmByte >> 3 bitAnd: 7) + 1)))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 2/16/2011 14:45'!
handleMovGvEvFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a register load into a ProcessorSimulationTrap signal."
	| modrmByte |
	^(((modrmByte := memoryArray byteAt: pc + 2) bitAnd: 16rC7) = 16r5) "ModRegInd & disp32"
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 6
					address: (memoryArray unsignedLongAt: pc + 3 bigEndian: false)
					type: #read
					accessor: (#(eax: ecx: edx: ebx: esp: ebp: esi: edi:) at: ((modrmByte >> 3 bitAnd: 7) + 1)))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 5/7/2010 11:37'!
handleMovObALFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a write of eax into a ProcessorSimulationTrap signal."
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 5
			address: (memoryArray unsignedLongAt: pc + 2 bigEndian: false)
			type: #write
			accessor: #eax)
		signal! !

!BochsIA32Alien methodsFor: 'error handling' stamp: 'eem 5/7/2010 11:13'!
handleRetFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a ret into a ProcessorSimulationTrap signal."
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 1
			address: (memoryArray unsignedLongAt: self esp + 1)
			type: #return
			accessor: #eip:)
		signal! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 9/9/2009 21:27'!
integerRegisterState
	^{	self eax. self ebx. self ecx. self edx. self esp. self ebp. self esi. self edi.
		self eip. self eflags }! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 12/30/2008 09:15'!
jmpOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2, pA8"
	^16rE9! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 10/20/2010 17:12'!
leafRetpcIn: aMemory
	^aMemory unsignedLongAt: self esp + 1 bigEndian: false! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 9/9/2009 21:24'!
lockPrefix
	^16rF0! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 12/30/2008 09:14'!
movALObOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2, pA7"
	^16rA1! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 5/7/2010 11:51'!
movEbGbOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2"
	^16r88! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 1/1/2009 22:26'!
movEvGvOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2, pA8"
	^16r89! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 5/7/2010 11:50'!
movGbEbOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2"
	^16r8A! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 12/30/2008 09:14'!
movGvEvOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2, pA8"
	^16r8B! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 1/2/2009 12:45'!
movObALOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2, pA7"
	^16rA3! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 12/12/2008 15:52'!
nopOpcode
	^16r90! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 1/4/2009 13:08'!
pc
	"Return whatever the processor considers its program counter."
	^self eip! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 1/5/2009 14:20'!
pc: anAddress
	"Set whatever the processor considers its program counter to anAddress."
	self eip: anAddress! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 10/20/2010 17:14'!
popWordIn: aMemory 
	| sp word |
	word := aMemory unsignedLongAt: (sp := self esp) + 1 bigEndian: false.
	self esp: sp + 4.
	^word! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 3/30/2009 12:35'!
postCallArgumentsNumArgs: numArgs "<Integer>" in: memory "<ByteArray|Bitmap>"
	"Answer an argument vector of the requested size after a vanilla
	 ABI call.  On IA32 this typically means accessing stacked arguments
	 beyond the pushed return address and saved frame pointer.
	 For compatibility with Cog/Slang we answer unsigned values."
	^(9 to: numArgs * 4 + 5 by: 4) collect:
		[:i|
		memory unsignedLongAt: self ebp + i bigEndian: false]! !

!BochsIA32Alien methodsFor: 'primitives' stamp: 'eem 1/14/2009 15:26'!
primitiveDisassembleAt: address inMemory: memoryArray "<Bitmap|ByteArray>"
	"Answer an Array of the size and the disassembled code string for the instruction at the current instruction pointer in memory."
	<primitive: 'primitiveDisassembleAtInMemory' module: 'BochsIA32Plugin'>
	^self primitiveFailed! !

!BochsIA32Alien methodsFor: 'primitives' stamp: 'eem 12/12/2008 13:02'!
primitiveErrorAndLog
	"Answer an array of the current error code and log contents"
	<primitive: 'primitiveErrorAndLog' module: 'BochsIA32Plugin'>
	^self primitiveFailed! !

!BochsIA32Alien methodsFor: 'primitives' stamp: 'eem 1/15/2009 09:58'!
primitiveFlushICacheFrom: startAddress "<Integer>" To: endAddress "<Integer>"
	"Flush the icache in the requested range"
	<primitive: 'primitiveFlushICacheFromTo' module: 'BochsIA32Plugin'>
	^self primitiveFailed! !

!BochsIA32Alien methodsFor: 'primitives' stamp: 'eem 12/12/2008 13:29'!
primitiveResetCPU
	"Reset the receiver to registers all zero, and protected 32-bit mode."
	<primitive: 'primitiveResetCPU' module: 'BochsIA32Plugin'>
	^self reportPrimitiveFailure! !

!BochsIA32Alien methodsFor: 'primitives' stamp: 'eem 1/27/2009 15:40'!
primitiveRunInMemory: memoryArray "<Bitmap|ByteArray>" minimumAddress: minimumAddress "<Integer>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Run the receiver using the argument as the store.  Origin the argument at 0. i.e. the first byte of the
	 memoryArray is address 0.  Make addresses below minimumAddress illegal.  Convert out-of-range
	 call, jump and memory read/writes into register instructions into ProcessorSimulationTrap signals."
	<primitive: 'primitiveRunInMemoryMinimumAddressReadWrite' module: 'BochsIA32Plugin' error: ec>
	^ec == #'inappropriate operation'
		ifTrue: [self handleExecutionPrimitiveFailureIn: memoryArray
					minimumAddress: minimumAddress
					readOnlyBelow: minimumWritableAddress]
		ifFalse: [self reportPrimitiveFailure]

	"self printRegistersOn: Transcript"! !

!BochsIA32Alien methodsFor: 'primitives' stamp: 'eem 1/27/2009 15:40'!
primitiveSingleStepInMemory: memoryArray "<Bitmap|ByteArray>" minimumAddress: minimumAddress "<Integer>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Single-step the receiver using the argument as the store.  Origin the argument at 0. i.e. the first byte of the
	 memoryArray is address 0.  Make addresses below minimumAddress illegal.  Convert out-of-range
	 call, jump and memory read/writes into register instructions into ProcessorSimulationTrap signals."
	<primitive: 'primitiveSingleStepInMemoryMinimumAddressReadWrite' module: 'BochsIA32Plugin' error: ec>
	^ec == #'inappropriate operation'
		ifTrue: [self handleExecutionPrimitiveFailureIn: memoryArray
					minimumAddress: minimumAddress
					readOnlyBelow: minimumWritableAddress]
		ifFalse: [self reportPrimitiveFailure]! !

!BochsIA32Alien methodsFor: 'printing' stamp: 'eem 9/9/2009 21:27'!
printIntegerRegistersOn: aStream
	self printRegisterState: self integerRegisterState on: aStream.
	aStream flush! !

!BochsIA32Alien methodsFor: 'printing' stamp: 'eem 11/29/2010 18:08'!
printRegisterState: registerStateVector on: aStream
	| rsvs fields|
	aStream ensureCr.
	rsvs := registerStateVector readStream.
	fields := (registerStateVector size < 18
			   or: [(11 to: 18) allSatisfy: [:i| (registerStateVector at: i) isZero]])
				ifTrue:
					[#(	eax ebx ecx edx cr
						esp ebp esi edi cr
						eip eflags cr )]
				ifFalse:
					[#(	eax ebx ecx edx cr
						esp ebp esi edi cr
						eip eflags cr
						xmm0low xmm1low cr
						xmm2low xmm3low cr
						xmm4low xmm5low cr
						xmm6low xmm7low cr )].
	fields withIndexDo:
		[:sym :index| | val |
		sym = #cr
			ifTrue: [aStream cr]
			ifFalse:
				[(val := rsvs next) isNil ifTrue: [^self].
				(sym beginsWith: 'xmm')
					ifTrue:
						[aStream nextPutAll: sym; nextPut: $:; space.
						 val printOn: aStream base: 16 length: 16 padded: true.
						 aStream space; nextPut: $(.
						 "At the image level Float is apparently in big-endian format"
						 ((Float basicNew: 2)
						 	at: 2 put: (val bitAnd: 16rFFFFFFFF);
							at: 1 put: (val bitShift: -32);
							yourself)
								printOn: aStream.
						 aStream nextPut: $)]
					ifFalse:
						[aStream nextPutAll: sym; nextPut: $:; space.
						 val printOn: aStream base: 16 length: 8 padded: true.
						 #eflags == sym
							ifTrue:
								[aStream space.
								 'C-P-A-ZS---O' withIndexDo:
									[:flag :bitIndex|
									flag ~= $- ifTrue:
										[aStream nextPut: flag; nextPutAll: 'F='; print: (val bitAnd: 1 << (bitIndex - 1)) >> (bitIndex - 1); space]]]
							ifFalse:
								[val > 16 ifTrue:
									[aStream space; nextPut: $(.
									 val printOn: aStream base: 10 length: 1 padded: false.
									 aStream nextPut: $)]]].
						(fields at: index + 1) ~~ #cr ifTrue:
							[aStream tab]]]! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 1/22/2009 15:43'!
pushWord: aValue in: aMemory
	| sp |
	sp := (self esp: self esp - 4).
	aMemory longAt: sp + 1 put: aValue bigEndian: false! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 12/6/2010 09:29'!
registerAt: index 
	^self perform: (#(eax ecx edx ebx esp ebp esi edi) at: index + 1)! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 4/17/2009 13:48'!
registerState
	^{	self eax. self ebx. self ecx. self edx. self esp. self ebp. self esi. self edi.
		self eip. self eflags.
		self xmm0low. self xmm1low. self xmm2low. self xmm3low.
		self xmm4low. self xmm5low. self xmm6low. self xmm7low }! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 12/5/2010 11:40'!
registerStatePCIndex
	^9! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 4/16/2009 17:56'!
reset
	self primitiveResetCPU.
	"Enable SSE extensions by setting the OSFXSR (Operating System FXSAVE/FXRSTOR Support) bit"
	self cr4: (self cr4 bitOr: 1 << 9)! !

!BochsIA32Alien methodsFor: 'opcodes' stamp: 'eem 4/16/2009 13:42'!
retOpcode
	"[1] IA-32 Intel Architecture Software Developer's Manual Volume 2B: Instruction Set Reference, N-Z.
		table A2, pA7"
	^16rC3! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 10/20/2010 17:14'!
retpcIn: aMemory
	^aMemory unsignedLongAt: self ebp + 5 bigEndian: false! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 9/30/2009 13:16'!
saveEip
	^self unsignedLongAt: 1533! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 9/30/2009 13:16'!
saveEsp
	^self unsignedLongAt: 1537! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 1/4/2009 10:57'!
setFramePointer: framePointer stackPointer: stackPointer
	"Initialize the processor's frame and stack pointers"
	self ebp: framePointer.
	self esp: stackPointer! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 9/22/2009 13:24'!
setRegisterState: aRegisterStateArray
	"N.B. keep in sync with voidRegisterState"
	self eax: (aRegisterStateArray at: 1).
	self ebx: (aRegisterStateArray at: 2).
	self ecx: (aRegisterStateArray at: 3).
	self edx: (aRegisterStateArray at: 4).
	self esp: (aRegisterStateArray at: 5).
	self ebp: (aRegisterStateArray at: 6).
	self esi: (aRegisterStateArray at: 7).
	self edi: (aRegisterStateArray at: 8).
	self eip: (aRegisterStateArray at: 9).
	self eflags: (aRegisterStateArray at: 10).
	self xmm0low: (aRegisterStateArray at: 11).
	self xmm1low: (aRegisterStateArray at: 12).
	self xmm2low: (aRegisterStateArray at: 13).
	self xmm3low: (aRegisterStateArray at: 14).
	self xmm4low: (aRegisterStateArray at: 15).
	self xmm5low: (aRegisterStateArray at: 16).
	self xmm6low: (aRegisterStateArray at: 17).
	self xmm7low: (aRegisterStateArray at: 18)! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 5/6/2009 13:09'!
simulateCallOf: address nextpc: nextpc memory: aMemory
	"Simulate a frame-building call of address.  Build a frame since
	a) this is used for calls into the run-time which are unlikely to be leaf-calls, and
	b) stack alignment needs to be realistic for assert checking for platforms such as Mac OS X"
	self pushWord: nextpc in: aMemory.
	self pushWord: self ebp in: aMemory.
	self ebp: self esp.
	PostBuildStackDelta ~= 0 ifTrue:
		[self esp: self esp - PostBuildStackDelta].
	self eip: address! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 5/6/2009 13:10'!
simulateJumpCallOf: address memory: aMemory
	"Simulate a frame-building jump call of address (i.e. do not push the return pc
	 as this has already been done).  Build a frame since
	a) this is used for calls into the run-time which are unlikely to be leaf-calls, and
	b) stack alignment needs to be realistic for assert checking for platforms such as Mac OS X"
	self pushWord: self ebp in: aMemory.
	self ebp: self esp.
	PostBuildStackDelta ~= 0 ifTrue:
		[self esp: self esp - PostBuildStackDelta].
	self eip: address! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 3/30/2009 11:55'!
simulateLeafCallOf: address nextpc: nextpc memory: aMemory
	self pushWord: nextpc in: aMemory.
	self eip: address! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 3/30/2009 12:38'!
simulateLeafReturnIn: aMemory
	self eip: (self popWordIn: aMemory)! !

!BochsIA32Alien methodsFor: 'execution' stamp: 'eem 3/30/2009 11:27'!
simulateReturnIn: aMemory
	PostBuildStackDelta ~= 0 ifTrue:
		[self esp: self esp + PostBuildStackDelta].
	self ebp: (self popWordIn: aMemory).
	self eip: (self popWordIn: aMemory)! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 12/13/2010 09:26'!
smashCallerSavedRegistersWithValuesFrom: base by: step
	#(eax: ecx: edx:)
	   withIndexDo:
		[:accessor :index|
		self perform: accessor with: index - 1 * step + base]! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'lw 8/8/2012 11:51'!
smashRegisterAccessors
	^#(eax: ebx: ecx: edx: esi: edi:)! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 1/9/2009 09:20'!
sp
	^self esp! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 12/6/2010 18:18'!
sp: anAddress
	"Set whatever the processor considers its stack pointer to anAddress."
	self esp: anAddress! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 9/30/2009 13:16'!
stopReason
	^self unsignedByteAt: 1577! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 9/22/2009 13:23'!
voidRegisterState
	"N.B. keep in sync with setRegisterState:"
	self setRegisterState: (Array new: 18 withAll: 0)! !

!BochsIA32Alien methodsFor: 'accessing-abstract' stamp: 'eem 9/22/2009 16:31'!
withStackPointersInRegisterState: registerState do: aBinaryBlock
	^aBinaryBlock value: (registerState at: 5) value: (registerState at: 6)! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm0high
	^self unsignedLongLongAt: 1185! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm0high: anUnsignedInteger
	^self unsignedLongLongAt: 1185 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm0low
	^self unsignedLongLongAt: 1177! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm0low: anUnsignedInteger
	^self unsignedLongLongAt: 1177 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm1high
	^self unsignedLongLongAt: 1201! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm1high: anUnsignedInteger
	^self unsignedLongLongAt: 1201 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm1low
	^self unsignedLongLongAt: 1193! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm1low: anUnsignedInteger
	^self unsignedLongLongAt: 1193 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm2high
	^self unsignedLongLongAt: 1217! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm2high: anUnsignedInteger
	^self unsignedLongLongAt: 1217 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm2low
	^self unsignedLongLongAt: 1209! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm2low: anUnsignedInteger
	^self unsignedLongLongAt: 1209 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm3high
	^self unsignedLongLongAt: 1233! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm3high: anUnsignedInteger
	^self unsignedLongLongAt: 1233 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm3low
	^self unsignedLongLongAt: 1225! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm3low: anUnsignedInteger
	^self unsignedLongLongAt: 1225 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm4high
	^self unsignedLongLongAt: 1249! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm4high: anUnsignedInteger
	^self unsignedLongLongAt: 1249 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm4low
	^self unsignedLongLongAt: 1241! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm4low: anUnsignedInteger
	^self unsignedLongLongAt: 1241 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm5high
	^self unsignedLongLongAt: 1265! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm5high: anUnsignedInteger
	^self unsignedLongLongAt: 1265 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm5low
	^self unsignedLongLongAt: 1257! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm5low: anUnsignedInteger
	^self unsignedLongLongAt: 1257 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm6high
	^self unsignedLongLongAt: 1281! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm6high: anUnsignedInteger
	^self unsignedLongLongAt: 1281 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm6low
	^self unsignedLongLongAt: 1273! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm6low: anUnsignedInteger
	^self unsignedLongLongAt: 1273 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm7high
	^self unsignedLongLongAt: 1297! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm7high: anUnsignedInteger
	^self unsignedLongLongAt: 1297 put: anUnsignedInteger! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm7low
	^self unsignedLongLongAt: 1289! !

!BochsIA32Alien methodsFor: 'accessing' stamp: 'eem 4/15/2009 19:06'!
xmm7low: anUnsignedInteger
	^self unsignedLongLongAt: 1289 put: anUnsignedInteger! !

!CogProcessorAlien methodsFor: 'Cog API' stamp: 'lw 7/10/2012 16:26'!
abstractInstructionCompilerClass
	self subclassResponsibility! !

!CogProcessorAlien methodsFor: 'opcodes' stamp: 'lw 7/11/2012 14:36'!
callOpcode
	self subclassResponsibility.! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'eem 12/2/2010 10:24'!
decorateDisassembly: anInstructionString for: aSymbolManager "<Cogit>"
	| string i1 i2 v |
	string := anInstructionString.
	(i1 := string indexOfSubCollection: '%ds:0x') > 0 ifTrue:
		[i2 := i1 + 6.
		 ['0123456789abcdef' includes: (string at: i2)] whileTrue: [i2 := i2 + 1].
		 string := string
					copyReplaceFrom: i1 + 4
					to: i2 - 1
					with: (aSymbolManager lookupCHexString: (string copyFrom: i1 + 4 to: i2 - 1))].
	(i1 := string indexOfSubCollection: '%ss:0x') > 0 ifTrue:
		[i2 := i1 + 6.
		 ['0123456789abcdef' includes: (string at: i2)] whileTrue: [i2 := i2 + 1].
		 ((string at: i2) = $(
		 and: [(string at: i2 + 1) = $%]) ifTrue:
			[v := Integer readFrom: (ReadStream on: string from: i1 + 6 to: i2 - 1) base: 16.
			string := string
						copyReplaceFrom: i1
						to: i2 - 1
						with: ((v bitAnd: (1 bitShift: 31) - 1) - (v bitAnd: (1 bitShift: 31))) printString]].
	(i1 := string indexOfSubCollection: '$0x') > 0 ifTrue:
		[i2 := i1 + 3.
		 ['0123456789abcdef' includes: (string at: i2)] whileTrue: [i2 := i2 + 1].
		 string := string
					copyReplaceFrom: i1 + 1
					to: i2 - 1
					with: (aSymbolManager lookupCHexString: (string copyFrom: i1 + 1 to: i2 - 1))].
	((i1 := string indexOf: $() > 1
	 and: [(string at: i1 + 1) isDigit
	 and: [i1 < (i2 := string indexOf: $))]]) ifTrue:
		[string := string
					copyReplaceFrom: i1 + 1
					to: i2 - 1
					with: (aSymbolManager lookupCHexString: (string copyFrom: i1 + 1 to: i2 - 1))].
	^string! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'lw 8/24/2012 19:18'!
decorateDisassembly: anInstructionString for: aSymbolManager "<Cogit>" fromAddress: address
	^self decorateDisassembly: anInstructionString for: aSymbolManager! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'lw 8/24/2012 19:15'!
disassembleFrom: startAddress to: endAddress in: memory for: aSymbolManager "<Cogit>" labels: labelDictionary on: aStream
	| address |
	address := startAddress.
	[address <= endAddress] whileTrue:
		[[:size :string|
		(aSymbolManager labelForPC: address) ifNotNil:
			[:label| aStream nextPutAll: label; nextPut: $:; cr].
		(labelDictionary at: address ifAbsent: []) ifNotNil:
			[:label| aStream nextPutAll: label; nextPut: $:; cr].
		aStream nextPutAll: (self decorateDisassembly: string for: aSymbolManager fromAddress: address); cr; flush.
		address := address + size]
			valueWithArguments: (self
									primitiveDisassembleAt: address
									inMemory: memory)]! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'lw 8/9/2012 13:32'!
disassembleFrom: startAddress to: endAddress in: memory on: aStream
	| address |
	address := startAddress.
	[address < endAddress] whileTrue:
		[[:size :string|
		aStream nextPutAll: (address printStringBase: 16 length: 8 padded: true); nextPut: $:; tab; tab; nextPutAll: string; cr; flush.
		address := address + size]
			valueWithArguments: (self
									primitiveDisassembleAt: address
									inMemory: memory)]! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'eem 1/20/2009 12:39'!
disassembleInstructionAt: pc In: memory
	^(self primitiveDisassembleAt: pc inMemory: memory) last! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'eem 1/14/2009 15:27'!
disassembleInstructionAt: ip In: memory into: aBlock
	| lenAndDi |
	lenAndDi := self primitiveDisassembleAt: ip inMemory: memory.
	^aBlock value: lenAndDi last value: lenAndDi first! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'lw 7/11/2012 13:40'!
disassembleNextInstructionIn: memory
	^(self primitiveDisassembleAt: self pc inMemory: memory) last! !

!CogProcessorAlien methodsFor: 'disassembly' stamp: 'lw 8/9/2012 18:44'!
disassembleNextInstructionIn: memory for: aSymbolManager "<Cogit|nil>"
	| string |
	string := (self primitiveDisassembleAt: self pc inMemory: memory) last.
	^aSymbolManager
		ifNil: [string]
		ifNotNil: [self decorateDisassembly: string for: aSymbolManager]! !

!CogProcessorAlien methodsFor: 'accessing-abstract' stamp: 'lw 7/12/2012 22:16'!
endianness
	self subclassResponsibility! !

!CogProcessorAlien methodsFor: 'opcodes' stamp: 'lw 7/10/2012 16:23'!
nopOpcode
	self subclassResponsibility.! !

!CogProcessorAlien methodsFor: 'printing' stamp: 'lw 7/10/2012 16:22'!
printRegisterState: registerStateVector on: aStream

	self subclassResponsibility.! !

!CogProcessorAlien methodsFor: 'printing' stamp: 'eem 4/17/2009 13:59'!
printRegistersOn: aStream
	self printRegisterState: self registerState on: aStream.
	aStream flush! !

!CogProcessorAlien methodsFor: 'error handling' stamp: 'lw 7/11/2012 17:04'!
reportPrimitiveFailure
	| errorAndLog |
	errorAndLog := self primitiveErrorAndLog.
	self error: 'Error ', errorAndLog first printString, (errorAndLog last ifNil: [''] ifNotNil: [:log| ': ', log])! !

!CogProcessorAlien methodsFor: 'execution' stamp: 'eem 1/27/2009 15:20'!
runInMemory: aMemory
	| result |
	result := self primitiveRunInMemory: aMemory minimumAddress: 0 readOnlyBelow: 0.
	result ~~ self ifTrue:
		[self error: 'eek!!']! !

!CogProcessorAlien methodsFor: 'execution' stamp: 'eem 1/27/2009 15:14'!
runInMemory: aMemory minimumAddress: minimumAddress readOnlyBelow: minimumWritableAddress
	| result |
	result := self primitiveRunInMemory: aMemory
				minimumAddress: minimumAddress
				readOnlyBelow: minimumWritableAddress.
	result ~~ self ifTrue:
		[self error: 'eek!!']! !

!CogProcessorAlien methodsFor: 'execution' stamp: 'eem 10/21/2010 11:03'!
runInMemory: aMemory readExecuteOnlyBelow: minWriteMaxExecAddr
	| result |
	result := self primitiveRunInMemory: aMemory minimumAddress: 0 readOnlyBelow: minWriteMaxExecAddr.
	result ~~ self ifTrue:
		[self error: 'eek!!']! !

!CogProcessorAlien methodsFor: 'execution' stamp: 'eem 10/21/2010 10:58'!
singleStepIn: aMemory
	| result |
	result := self primitiveSingleStepInMemory: aMemory minimumAddress: 0 readOnlyBelow: aMemory size.
	result ~~ self ifTrue:
		[self error: 'eek!!']! !

!CogProcessorAlien methodsFor: 'execution' stamp: 'eem 1/27/2009 15:17'!
singleStepIn: aMemory minimumAddress: minimumAddress readOnlyBelow: minimumWritableAddress
	| result |
	result := self primitiveSingleStepInMemory: aMemory
				minimumAddress: minimumAddress
				readOnlyBelow: minimumWritableAddress.
	result ~~ self ifTrue:
		[self error: 'eek!!']! !

!CogProcessorAlien methodsFor: 'execution' stamp: 'eem 10/21/2010 11:05'!
singleStepIn: aMemory readExecuteOnlyBelow: minWriteMaxExecAddr
	| result |
	result := self primitiveSingleStepInMemory: aMemory minimumAddress: 0 readOnlyBelow: minWriteMaxExecAddr.
	result ~~ self ifTrue:
		[self error: 'eek!!']! !

!CogProcessorAlien methodsFor: 'accessing-abstract' stamp: 'lw 8/8/2012 11:50'!
smashRegisterAccessors
	self subclassResponsibility! !

!CogProcessorAlien methodsFor: 'accessing-abstract' stamp: 'lw 8/8/2012 11:50'!
smashRegistersWithValuesFrom: base by: step
	self smashRegisterAccessors
	   withIndexDo:
		[:accessor :index|
		self perform: accessor with: index - 1 * step + base]! !

CogProcessorAlien variableByteSubclass: #GdbARMAlien
	instanceVariableNames: ''
	classVariableNames: 'PostBuildStackDelta'
	poolDictionaries: ''
	category: 'Cog-Processors'!

!GdbARMAlien commentStamp: 'lw 8/23/2012 19:17' prior: 0!
I am a wrapper around the ARMulator CPU instance and emulator routines and I give access to disassembling using libopcodes. My C-part must be compiled with -DMODET, because otherwise my offesets are wrong by one field.!

!GdbARMAlien class methodsFor: 'instance creation' stamp: 'lw 7/11/2012 10:38'!
dataSize

	^1536! !

!GdbARMAlien class methodsFor: 'debugger' stamp: 'lw 6/25/2012 15:32'!
defaultIntegerBaseInDebugger
	^16! !

!GdbARMAlien class methodsFor: 'class initialization' stamp: 'lw 7/12/2012 22:34'!
initialize
	"GdbARMAlien initialize"
	super initialize.
	PostBuildStackDelta := 0.! !

!GdbARMAlien class methodsFor: 'instance creation' stamp: 'lw 6/25/2012 15:31'!
new
	^(self atAddress: self primitiveNewCPU) reset! !

!GdbARMAlien class methodsFor: 'primitives' stamp: 'lw 6/25/2012 15:35'!
primitiveNewCPU
	"Answer the address of a new ARMulator C type ARMul_State instance."
	<primitive: 'primitiveNewCPU' module: 'GdbARMPlugin'>
	^self primitiveFailed! !

!GdbARMAlien class methodsFor: 'accessing' stamp: 'lw 7/17/2012 22:12'!
setStackAlignmentDelta: 	stackAlignmentDelta
	"copy of BochsIA32Alien>>#setStackAlignmentDelta:, because I don't know yet what it's effect is."
	self assert: stackAlignmentDelta isPowerOfTwo.
	PostBuildStackDelta :=  stackAlignmentDelta > 8
								ifTrue: [stackAlignmentDelta - 8]
								ifFalse: [0]! !

!GdbARMAlien methodsFor: 'Cog API' stamp: 'lw 7/2/2012 11:50'!
abstractInstructionCompilerClass
	^CogARMCompiler! !

!GdbARMAlien methodsFor: 'Cog API' stamp: 'lw 7/31/2012 14:44'!
bitsInWord
	^32! !

!GdbARMAlien methodsFor: 'opcodes' stamp: 'lw 8/9/2012 20:59'!
branchAndLinkOpcodeWithOffset: aNumber
	
	| offset |
	offset := (aNumber - 8) asInteger.
	(offset between: -33554432 and: 33554428) ifFalse: [self error: 'The offset is to far. ARM does not support such far jumps.'].
	^ 16reb000000 bitOr: (offset >> 2 bitAnd: 16r00FFFFFF)
! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/11/2012 10:38'!
cflag
	^self unsignedLongAt: 577! !

!GdbARMAlien methodsFor: 'disassembly' stamp: 'lw 8/24/2012 19:45'!
decorateDisassembly: anInstructionString for: aSymbolManager "<Cogit>" fromAddress: address
	"Prepend the address to the decoration string."
	| addressPrefix |
	addressPrefix := ((address storeStringBase: 16 length: 9 padded: true) allButFirst: 3), ' ', String tab.
	anInstructionString = 'mov	r1, r1' 
		ifTrue: [^super decorateDisassembly: addressPrefix, 'nop' for: aSymbolManager].
	^super decorateDisassembly: addressPrefix, anInstructionString for: aSymbolManager! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/11/2012 11:09'!
eflags
	
	^ (((self nflag << 5 bitOr: self zflag << 4) 
			bitOr: self cflag << 3) 
				bitOr: self vflag << 2)
					bitOr: self ifflags! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 7/12/2012 22:16'!
endianness
	^#little! !

!GdbARMAlien methodsFor: 'execution' stamp: 'lw 7/31/2012 11:52'!
flushICacheFrom: startAddress "<Integer>" to: endAddress "<Integer>"
	self primitiveFlushICacheFrom: startAddress To: endAddress! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 7/12/2012 22:57'!
fp
	"According to the use in SVr4"
	^self r11! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 7/16/2012 09:42'!
fp: aNumber
	"According to SVr4, the frame pointer is stored in r11"
	^self r11: aNumber! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 21:00'!
handleCallFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a call into a ProcessorSimulationTrap signal."
	|  relativeJump callAddress |
	((memoryArray byteAt: pc + 4) bitAnd: 16rF) = 16rB "BL opcode"
		ifTrue: ["short jump via BL, therefore we have a 24bit signed integer offset"
			relativeJump := (memoryArray unsignedLongAt: pc + 1 bigEndian: false) bitAnd: 16r00FFFFFF.
			relativeJump := (relativeJump bitAt: 24) = 1 
									ifTrue: [((relativeJump bitOr: 16r3F000000) << 2) signedIntFromLong]
									ifFalse: [relativeJump << 2].
			callAddress := (pc + 8 + relativeJump)]
		
		ifFalse: ["long jump using RISCTempReg"
			"The memoryArray starts indexing from 1, whereas the pc is based on 0-indexing, therefore all access-offsets are one greater than expected"
			callAddress := (memoryArray byteAt: pc + 4) 
								+ ((memoryArray byteAt: pc - 4) << 24) 
								+ ((memoryArray byteAt: pc - 8) << 16) 
								+ ((memoryArray byteAt: pc - 12) << 8)].
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 4
			address: callAddress signedIntToLong
			type: #call)
		signal! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 10:41'!
handleExecutionPrimitiveFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Handle an execution primitive failure for an unhandled opcode."
	^self reportPrimitiveFailure! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 20:53'!
handleExecutionPrimitiveFailureIn: memoryArray "<Bitmap|ByteArray>" minimumAddress: minimumAddress "<Integer>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Handle an execution primitive failure.  Convert out-of-range call and absolute
	 memory read into register instructions into ProcessorSimulationTrap signals."
	"self printRegistersOn: Transcript"
	| pc |
	((pc := self pc) between: minimumAddress and: memoryArray byteSize - 1) ifTrue:
		[(self instructionTypeAt: pc in: memoryArray)
			caseOf: {
				[#Call] 			-> [^self handleCallFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#Jump] 		-> [^self handleJmpFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#MoveAwR]	-> [^self handleMoveAwRFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#MoveMbrR]	-> [^self handleMoveMbrRFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#MoveRAw]	-> [^self handleMoveRAwFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#MoveRMbr]	-> [^self handleMoveRMbrFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#Ret]			-> [^self handleRetFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].} 
			otherwise: [^self handleExecutionPrimitiveFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress]].
	((pc := self lr - 4) between: minimumAddress and: memoryArray byteSize - 1) ifTrue:
		[(self instructionTypeAt: pc in: memoryArray)
			caseOf: {
				[#Call] 		-> [^self handleCallFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#Jump] 	-> [^self handleJmpFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].
				[#Ret]		-> [^self handleRetFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress].} 
			otherwise: [^self handleExecutionPrimitiveFailureAt: pc in: memoryArray readOnlyBelow: minimumWritableAddress]].
	^self reportPrimitiveFailure! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 14:02'!
handleJmpFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a jmp into a ProcessorSimulationTrap signal."
	|  relativeJump |
	self halt.
	relativeJump := memoryArray longAt: pc + 2 bigEndian: false.
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 5
			address: (pc + 5 + relativeJump) signedIntToLong
			type: #jump)
		signal! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 20:49'!
handleMoveAwRFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a register load from an Address into a ProcessorSimulationTrap signal."
	((memoryArray byteAt: pc + 4) = 16rE5  "test for E593 as the most significant two bytes"
		and: [(memoryArray byteAt: pc + 3) = (16r90 + (CogARMCompiler classPool at: #RISCTempReg))])
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 4 
					address: (memoryArray byteAt: pc + 1) "address, combined from four instructions"
								+ ((memoryArray byteAt: pc - 3) << 24) 
								+ ((memoryArray byteAt: pc - 7) << 16) 
								+ ((memoryArray byteAt: pc - 11) << 8)
					type: #read
					accessor: (self registerStateSetters at: ((memoryArray byteAt: pc + 2) >> 4) + 1))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 20:48'!
handleMoveMbrRFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a byte register load into a ProcessorSimulationTrap signal."
	"MoveMbrR"
	| modrmByte |
	self halt.
	^(((modrmByte := memoryArray byteAt: pc + 2) bitAnd: 16rC0) = 16r80) "ModRegRegDisp32"
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 6
					address: ((self perform: (#(eax ecx edx ebx esp ebp esi edi) at: (modrmByte bitAnd: 7) + 1))
							+ (memoryArray unsignedLongAt: pc + 3 bigEndian: false)
								bitAnd: 16rFFFFFFFF)
					type: #read
					accessor: (#(al: cl: dl: bl: ah: ch: dh: bh:) at: ((modrmByte >> 3 bitAnd: 7) + 1)))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 20:48'!
handleMoveRAwFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a register write into Memory at a predefined address into a ProcessorSimulationTrap signal."
	^((memoryArray byteAt: pc + 4) = 16rE5 
		and: [(memoryArray byteAt: pc + 3) = (16r80 + (CogARMCompiler classPool at: #RISCTempReg))])
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 4
					address: (memoryArray byteAt: pc + 1) 
								+ ((memoryArray byteAt: pc - 3) << 24) 
								+ ((memoryArray byteAt: pc - 7) << 16) 
								+ ((memoryArray byteAt: pc - 11) << 8)
					type: #write
					accessor: (self registerStateNames at: ((memoryArray byteAt: pc + 2) >> 4) + 1))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 20:48'!
handleMoveRMbrFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
		"Convert an execution primitive failure for a byte register write into a ProcessorSimulationTrap signal."
	"MoveRMbr"
	| modrmByte |
	self halt.
	^(((modrmByte := memoryArray byteAt: pc + 2) bitAnd: 16rC0) = 16r80) "ModRegRegDisp32"
		ifTrue:
			[(ProcessorSimulationTrap
					pc: pc
					nextpc: pc + 6
					address: ((self perform: (#(eax ecx edx ebx esp ebp esi edi) at: (modrmByte bitAnd: 7) + 1))
							+ (memoryArray unsignedLongAt: pc + 3 bigEndian: false)
								bitAnd: 16rFFFFFFFF)
					type: #write
					accessor: (#(al cl dl bl ah ch dh bh) at: ((modrmByte >> 3 bitAnd: 7) + 1)))
				signal]
		ifFalse:
			[self reportPrimitiveFailure]! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 17:54'!
handleRetFailureAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Convert an execution primitive failure for a ret into a ProcessorSimulationTrap signal."
	self halt.
	^(ProcessorSimulationTrap
			pc: pc
			nextpc: pc + 4
			address: (memoryArray unsignedLongAt: self sp + 1)
			type: #return
			accessor: #pc:)
		signal! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/11/2012 10:38'!
ifflags
	^self unsignedLongAt: 585! !

!GdbARMAlien methodsFor: 'error handling' stamp: 'lw 8/9/2012 19:58'!
instructionTypeAt: pc "<Integer>" in: memoryArray "<Bitmap|ByteArray>"
	"Identify which type of instruction is at pc in memoryArray. For the time beeing, only those instructions needed for PrimitiveFailure are identified."
	| RISCTempReg lastInstruction typeOperandStatus |
	RISCTempReg := CogARMCompiler classPool at: #RISCTempReg.
	lastInstruction := memoryArray unsignedLongAt: pc + 1 bigEndian: false.
	"self disassembleFrom: pc - 16 to: pc + 3 in: memoryArray on: Transcript cr"
	
	"Ret"
	lastInstruction = 16rE8BD8000 ifTrue: [#Ret].

	"Call"
	(((memoryArray byteAt: pc + 4) bitAnd: 16rF) = 16rB "BL opcode"
		or: [(memoryArray unsignedLongAt: pc - 3 bigEndian: false) = 16rE1A0E00F]) ifTrue: [^#Call].
	
	"Jump"
	((memoryArray byteAt: pc + 3) >> 4 = 16rA "B opcode, for short jumps"
		or: [(lastInstruction >> 12 bitAnd: 16r0FFFF) = 
				(16r0280F + (RISCTempReg << 4)) 	"ADD? PC, RISCTempReg, #anything, for JumpLong-Instructions"])
		ifTrue: [^#Jump].
		
	typeOperandStatus := lastInstruction >> 20 bitAnd: 16rFF.
	"MoveRMbr"
	(typeOperandStatus = 16r54 or: [typeOperandStatus = 16r5C] or: [typeOperandStatus = 16r7C])
		ifTrue: [^#MoveRMbr].
		
	"MoveRAw"
	(typeOperandStatus = 16r58 and: [(lastInstruction >> 16 bitAnd: 16rF) = RISCTempReg])
		ifTrue: [^#MoveRAw].
		
	"MoveMbrR"
	(typeOperandStatus = 16r55 or: [typeOperandStatus = 16r5D] or: [typeOperandStatus = 16r7D])
		ifTrue: [^#MoveMbrR].
		
	"MoveAwR"
	((typeOperandStatus = 16r59) and: [(lastInstruction >> 16 bitAnd: 16rF) = RISCTempReg])
		ifTrue: [^#MoveAwR].
	
	
	^#UnidentifiedInstruction! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 7/10/2012 14:20'!
integerRegisterState
	^{	self r0. self r1. self r2. self r3. self r4. self r5. self r6. self r7. self r8. 
		self r9. self r10. self r11. self r12. self sp. self lr. self pc}! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
lr
	^self unsignedLongAt: 69! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
lr: anUnsignedInteger

	^self unsignedLongAt: 69 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/11/2012 10:38'!
nflag
	^self unsignedLongAt: 569! !

!GdbARMAlien methodsFor: 'opcodes' stamp: 'lw 8/8/2012 13:55'!
nopOpcode
	"mov r0, r0"
	^ 16rE1A00000! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
pc
	^self unsignedLongAt: 73! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
pc: anUnsignedInteger

	^self unsignedLongAt: 73 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'execution' stamp: 'lw 8/20/2012 12:45'!
postCallArgumentsNumArgs: numArgs "<Integer>" in: memory "<ByteArray|Bitmap>"
	"Answer an argument vector of the requested size after a vanilla
	 ABI call. For ARM the Procedure Calling Specification can be found in IHI0042D_aapcs.pdf.
	On ARM this typically means accessing r0 through r3 and fetching additional arguments from the stack, acording to pages 20f. aapcs.
	We assume that all arguments are single word arguments, which can not be supplied on co-processor-registers.
	 For compatibility with Cog/Slang we answer unsigned values."
	^(1 to: numArgs) collect: [:i |
		i < 5 
			ifTrue: [self perform: (self registerStateNames at: i)]
			"ARM uses a full descending stack. Directly after calling a procedure, nothing but the arguments are pushed."
			ifFalse: [memory unsignedLongAt: self sp + (i-5)*4 bigEndian: false]].! !

!GdbARMAlien methodsFor: 'primitives' stamp: 'lw 8/24/2012 19:10'!
primitiveDisassembleAt: address inMemory: memoryArray "<Bitmap|ByteArray>"
	"Answer an Array of the size and the disassembled code string for the instruction at the current instruction pointer in memory."
	<primitive: 'primitiveDisassembleAtInMemory' module: 'GdbARMPlugin'>
	^self primitiveFailed! !

!GdbARMAlien methodsFor: 'primitives' stamp: 'lw 7/9/2012 20:27'!
primitiveErrorAndLog
	"Answer an array of the current error code and log contents"
	<primitive: 'primitiveErrorAndLog' module: 'GdbARMPlugin'>
	^self primitiveFailed! !

!GdbARMAlien methodsFor: 'primitives' stamp: 'lw 7/31/2012 11:53'!
primitiveFlushICacheFrom: startAddress "<Integer>" To: endAddress "<Integer>"
	"Flush the icache in the requested range"
	<primitive: 'primitiveFlushICacheFromTo' module: 'GdbARMPlugin'>
	^self primitiveFailed! !

!GdbARMAlien methodsFor: 'primitives' stamp: 'lw 7/2/2012 11:16'!
primitiveResetCPU
	"Reset the receiver to registers all zero, and protected 32-bit mode."
	<primitive: 'primitiveResetCPU' module: 'GdbARMPlugin'>
	^self reportPrimitiveFailure! !

!GdbARMAlien methodsFor: 'primitives' stamp: 'lw 7/2/2012 14:35'!
primitiveRunInMemory: memoryArray "<Bitmap|ByteArray>" minimumAddress: minimumAddress "<Integer>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Run the receiver using the argument as the store.  Origin the argument at 0. i.e. the first byte of the
	 memoryArray is address 0.  Make addresses below minimumAddress illegal.  Convert out-of-range
	 call, jump and memory read/writes into register instructions into ProcessorSimulationTrap signals."
	<primitive: 'primitiveRunInMemoryMinimumAddressReadWrite' module: 'GdbARMPlugin' error: ec>
	^ec == #'inappropriate operation'
		ifTrue: [self handleExecutionPrimitiveFailureIn: memoryArray
					minimumAddress: minimumAddress
					readOnlyBelow: minimumWritableAddress]
		ifFalse: [self reportPrimitiveFailure]

	"self printRegistersOn: Transcript"! !

!GdbARMAlien methodsFor: 'primitives' stamp: 'lw 7/9/2012 20:27'!
primitiveSingleStepInMemory: memoryArray "<Bitmap|ByteArray>" minimumAddress: minimumAddress "<Integer>" readOnlyBelow: minimumWritableAddress "<Integer>"
	"Single-step the receiver using the argument as the store.  Origin the argument at 0. i.e. the first byte of the
	 memoryArray is address 0.  Make addresses below minimumAddress illegal.  Convert out-of-range
	 call, jump and memory read/writes into register instructions into ProcessorSimulationTrap signals."
	<primitive: 'primitiveSingleStepInMemoryMinimumAddressReadWrite' module: 'GdbARMPlugin' error: ec>
	^ec == #'inappropriate operation'
		ifTrue: [self handleExecutionPrimitiveFailureIn: memoryArray
					minimumAddress: minimumAddress
					readOnlyBelow: minimumWritableAddress]
		ifFalse: [self reportPrimitiveFailure]! !

!GdbARMAlien methodsFor: 'printing' stamp: 'lw 8/8/2012 14:04'!
printOn: aStream
	aStream nextPutAll: 'an ARMAlien('.
	self registerState allButLast with: self registerStateNames allButLast do: [ :state :i |
		aStream 
			<< i << ': ' 
			<< (state abs > 15 ifTrue: [state hex] ifFalse: [state asString]) 
			<< ', '].
	aStream << 'NZCVIF: ' 
		<< (self registerState last printStringBase: 2 length: 6 padded: true) << ')'.! !

!GdbARMAlien methodsFor: 'printing' stamp: 'lw 7/11/2012 10:46'!
printRegisterState: registerStateVector on: aStream
	| rsvs fields|
	aStream ensureCr.
	rsvs := registerStateVector readStream.
	fields := #(	r0 r1 r2 r3 r4 r5 r6 cr r7 r8 r9 r10 r11 r12 cr sp lr pc eflags cr).
	fields withIndexDo:
		[:sym :index| | val |
		sym = #cr
			ifTrue: [aStream cr]
			ifFalse:
				[(val := rsvs next) isNil ifTrue: [^self].
				aStream nextPutAll: sym; nextPut: $:; space.
				val printOn: aStream base: 16 length: 8 padded: true.
				#eflags == sym
					ifTrue:
						[aStream space.
						 "'FIVCZN'"'--VCZN' withIndexDo:
							[:flag :bitIndex|
							flag ~= $- ifTrue:
								[aStream nextPut: flag; nextPutAll: 'F='; print: (val bitAnd: 1 << (bitIndex - 1)) >> (bitIndex - 1); space]]]
					ifFalse:
						[val > 16 ifTrue:
							[aStream space; nextPut: $(.
							 val printOn: aStream base: 10 length: 1 padded: false.
							 aStream nextPut: $)]].
				(fields at: index + 1) ~~ #cr ifTrue:
					[aStream tab]]]! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r0
	^self unsignedLongAt: 13! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r0: anUnsignedInteger

	^self unsignedLongAt: 13 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r1
	^self unsignedLongAt: 17! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r10
	^self unsignedLongAt: 53! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r10: anUnsignedInteger

	^self unsignedLongAt: 53 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r11
	^self unsignedLongAt: 57! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r11: anUnsignedInteger

	^self unsignedLongAt: 57 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r12
	^self unsignedLongAt: 61! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r12: anUnsignedInteger

	^self unsignedLongAt: 61 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r1: anUnsignedInteger

	^self unsignedLongAt: 17 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r2
	^self unsignedLongAt: 21! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r2: anUnsignedInteger

	^self unsignedLongAt: 21 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r3
	^self unsignedLongAt: 25! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r3: anUnsignedInteger

	^self unsignedLongAt: 25 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r4
	^self unsignedLongAt: 29! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r4: anUnsignedInteger

	^self unsignedLongAt: 29 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r5
	^self unsignedLongAt: 33! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r5: anUnsignedInteger

	^self unsignedLongAt: 33 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r6
	^self unsignedLongAt: 37! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r6: anUnsignedInteger

	^self unsignedLongAt: 37 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r7
	^self unsignedLongAt: 41! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r7: anUnsignedInteger

	^self unsignedLongAt: 41 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r8
	^self unsignedLongAt: 45! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r8: anUnsignedInteger

	^self unsignedLongAt: 45 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r9
	^self unsignedLongAt: 49! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
r9: anUnsignedInteger

	^self unsignedLongAt: 49 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 7/31/2012 15:12'!
registerState
	^{	self r0. self r1. self r2. self r3. self r4. self r5. self r6. self r7.
		self r8. self r9. self r10. self r11. self r12. self sp. self lr. self pc. self eflags }! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 8/9/2012 19:23'!
registerStateNames
	^#(	r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 r10 fp r12 sp lr pc eflags)! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 8/9/2012 19:23'!
registerStateSetters
	^#(	r0: r1: r2: r3: r4: r5: r6: r7: r8: r9: r10: fp: r12: sp: lr: pc:)! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/2/2012 11:15'!
reset
	self primitiveResetCPU! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 7/12/2012 22:38'!
setFramePointer: framePointer stackPointer: stackPointer
	"Initialize the processor's frame and stack pointers"
	self fp: framePointer.
	self sp: stackPointer! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/11/2012 10:38'!
sflag
	^self unsignedLongAt: 589! !

!GdbARMAlien methodsFor: 'execution simulation' stamp: 'lw 8/9/2012 20:44'!
simulateCallOf: address nextpc: nextpc memory: aMemory
	"Simulate a frame-building call of address.  Build a frame since
	a) this is used for calls into the run-time which are unlikely to be leaf-calls, and
	b) stack alignment needs to be realistic for assert checking for platforms such as Mac OS X"
	self lr: nextpc.
	self pc: address! !

!GdbARMAlien methodsFor: 'execution' stamp: 'lw 8/8/2012 11:56'!
simulateLeafCallOf: address nextpc: nextpc memory: aMemory
	self lr: nextpc.
	self pc: address! !

!GdbARMAlien methodsFor: 'accessing-abstract' stamp: 'lw 8/8/2012 11:52'!
smashRegisterAccessors
	^#(r0: r1: r2: r3: r4: r5: r6: r7: r8:)! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
sp
	^self unsignedLongAt: 65! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/10/2012 17:25'!
sp: anUnsignedInteger

	^self unsignedLongAt: 65 put: anUnsignedInteger! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/11/2012 10:38'!
vflag
	^self unsignedLongAt: 581! !

!GdbARMAlien methodsFor: 'accessing' stamp: 'lw 7/11/2012 10:38'!
zflag
	^self unsignedLongAt: 573! !

Object subclass: #BytecodeEncoderPutschEditor
	instanceVariableNames: 'xlator xlation kwdxlation ranges index output codeIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Morphing Bytecode Set'!

!BytecodeEncoderPutschEditor commentStamp: '<historical>' prior: 0!
I am a MethodNode that uses the size and emit methods that use the BytecodeEncoder hierarchy's bytecode generation facilities.  This means my nodes no longer need encode specifics about the bytecode set.

To compile edited versions of the size* and emit* methods use
	BytecodeEncoderPutschEditor new edit

In 3.8:
To get the source of the new version of MethodNode>>generate: for BytecodeAgnosticMethodNode use
BytecodeEncoderPutschEditor new
	editCode: (MethodNode sourceCodeAt: #generate:) asString
	inClass: BytecodeAgnosticMethodNode
	withSelector: #generate:

In 3.9:
To get the source of the new version of MethodNode>>generateWith:using: for BytecodeAgnosticMethodNode use
BytecodeEncoderPutschEditor new
	editCode: (BytecodeAgnosticMethodNode sourceCodeAt: #generateWith:using:) asString
	inClass: BytecodeAgnosticMethodNode
	withSelector: #generateWith:using:!

!BytecodeEncoderPutschEditor methodsFor: 'code editing' stamp: 'eem 5/5/2008 19:24'!
edit
	| sn |
	sn := SystemNavigation default.
	xlation keysAndValuesDo:
		[:s :t|
		(sn allImplementorsOf: s localTo: ParseNode) do:
			[:md|
			md actualClass
				compile: (self editCode: md sourceString inClass: md actualClass withSelector: md methodSymbol)
				classified: 'code generation (new scheme)']]! !

!BytecodeEncoderPutschEditor methodsFor: 'code editing' stamp: 'eem 5/6/2008 10:55'!
editCode: code inClass: class withSelector: selector
	| codeString |
	ranges := SHParserST80 new rangesIn: code classOrMetaClass: class workspace: nil  environment: nil.
	index := 1.
	codeIndex := 1.
	output := String new writeStream.
	codeString := code asString.
	self process: codeString while: [:ign| true].
	"(StringHolder new textContents:
		(TextDiffBuilder buildDisplayPatchFrom: code to: output contents))
			openLabel: class name , '>>', selector."
	^output contents! !

!BytecodeEncoderPutschEditor methodsFor: 'initialize-release' stamp: 'eem 5/6/2008 10:53'!
initialize
	xlator := [:s|
			((((s allButFirst: 4) beginsWith: 'For') or: [((s allButFirst: 4) beginsWith: 'Except') or: [s fifth = $:]])
				ifTrue: [(s first: 4), 'Code', (s allButFirst: 4)]
				ifFalse: [(s first: 4), 'CodeFor', (s allButFirst: 4)]) copyReplaceAll: ':on:' with: ':encoder:'].
	xlation := Dictionary new.
	(((ParseNode withAllSubclasses removeAll: Encoder withAllSubclasses; yourself)
		inject: Set new
		into: [:s :c| s addAll: c selectors. s])
			select:
				[:s| ((s beginsWith: 'emit') or: [s beginsWith: 'size'])
					and: [(s includesSubstring: 'Code' caseSensitive: true) not]]) do:
					[:s|xlation at: s put: (xlator value: s) asSymbol].
	kwdxlation := Dictionary new.
	xlation keysAndValuesDo: [:k :v| kwdxlation at: k keywords first put: v keywords first]! !

!BytecodeEncoderPutschEditor methodsFor: 'code editing' stamp: 'eem 5/6/2008 11:25'!
process: code while: aBlock
	| range mapOnToEncoder kwd |
	mapOnToEncoder := false.
	[index <= ranges size] whileTrue:
		[range := ranges at: index.
		  (aBlock value: range) ifFalse:
			[^self].
		 index := index + 1.
		 [codeIndex < range start] whileTrue:
			[output nextPut: (code at: codeIndex).
			 codeIndex := codeIndex + 1].
		range type == #assignment
			ifTrue: [output nextPutAll: ':='.
					codeIndex := range end + 1]
			ifFalse:
				[(#(keyword patternKeyword) includes: range type) ifTrue:
					[kwd := code copyFrom: range start to: range end.
					(mapOnToEncoder and: [kwd = 'on:'])
						ifTrue:
							[output nextPutAll: 'encoder: encoder'.
							 mapOnToEncoder := false.
							 codeIndex := (ranges at: index) end + 1.
							 index := index + 1]
						ifFalse:
							[(kwdxlation includesKey: kwd) ifTrue:
								[(kwd beginsWith: 'emit') ifTrue:
									[mapOnToEncoder := true].
								 output nextPutAll: (kwdxlation at: kwd).
								 codeIndex := range end + 1]]]
					ifFalse:
						[[codeIndex <= range end] whileTrue:
							[output nextPut: (code at: codeIndex).
							 codeIndex := codeIndex + 1].
						"kill whitespace after up-arrow in ^ expr"
						(range type == #return and: [index <= ranges size]) ifTrue:
							[codeIndex := (ranges at: index) start]]]]! !

Object subclass: #ClassHierarchyDuplicator
	instanceVariableNames: 'xlator classes prefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Morphing Bytecode Set'!

!ClassHierarchyDuplicator methodsFor: 'code editing' stamp: 'eem 5/16/2008 15:50'!
edit
	(ChangeSet superclassOrder: classes asArray) do:
		[:class| | newClass |
		newClass := class subclassDefinerClass evaluate: (self editClassDefinitionOf: class).
		{ class. class class }
			with: { newClass. newClass class }
			do: [:b :nb|
				b selectors do:
					[:s|
					nb
						compile: (self editCode: (b sourceCodeAt: s) inClass: b withSelector: s)
						classified: (class organization categoryOfElement: s)]]]

	"self new edit"! !

!ClassHierarchyDuplicator methodsFor: 'code editing' stamp: 'eem 5/16/2008 15:43'!
editClassDefinitionOf: aClass
	| classDef categoryString |
	categoryString := 'category: '''.
	classDef := self editCode: aClass definition inClass: nil withSelector: nil.
	classDef := classDef
					copyReplaceAll: categoryString
					with: categoryString, prefix.
	^classDef
		copyReplaceAll: aClass classVariablesString
		with: (String streamContents: [:s| aClass classVariablesString subStrings do: [:ea| s nextPutAll: prefix; nextPutAll: ea; space]])

	"self new editClassDefinitionOf: MessageNode"! !

!ClassHierarchyDuplicator methodsFor: 'code editing' stamp: 'eem 5/16/2008 15:59'!
editCode: sourceText inClass: class withSelector: selector
	| ranges index codeIndex output codeString range |
	ranges := SHParserST80 new
				rangesIn: (codeString := sourceText asString)
				classOrMetaClass: class
				workspace: nil
				environment: nil.
	index := codeIndex := 1.
	output := (String new: codeString size) writeStream.
	[index <= ranges size] whileTrue:
		[range := ranges at: index.
		 index := index + 1.
		 [codeIndex < range start] whileTrue:
			[output nextPut: (codeString at: codeIndex).
			 codeIndex := codeIndex + 1].
		range type == #assignment
			ifTrue: [output nextPutAll: ':='.
					codeIndex := range end + 1]
			ifFalse:
				[(#(globalVar classVar) includes: range type)
					ifTrue:
						[output nextPutAll: (xlator value: (codeString copyFrom: range start to: range end)).
						 codeIndex := range end + 1]
					ifFalse:
						[#symbol == range type
							ifTrue:
								[output
									nextPut: (codeString at: range start); "#"
									nextPutAll: (xlator value: (codeString copyFrom: range start + 1 to: range end)).
								 codeIndex := range end + 1]
							ifFalse:
								[[codeIndex <= range end] whileTrue:
									[output nextPut: (codeString at: codeIndex).
									 codeIndex := codeIndex + 1].
								"kill whitespace after up-arrow in ^ expr"
								(range type == #return and: [index <= ranges size]) ifTrue:
									[codeIndex := (ranges at: index) start]]]]].
	^output contents

	"SHParserST80 new
		rangesIn: (ClassHierarchyDuplicator sourceCodeAt: #editCode:inClass:withSelector:) asString
		classOrMetaClass: ClassHierarchyDuplicator
		workspace: nil
		environment: nil"

	"SHParserST80 new
		rangesIn: ClassHierarchyDuplicator definition asString
		classOrMetaClass: nil
		workspace: nil
		environment: nil"

	"ClassHierarchyDuplicator new editCode: LeafNode definition inClass: nil withSelector: nil"
	"ClassHierarchyDuplicator new editCode: (MessageNode sourceCodeAt: #emitToDo:on:value:) inClass: MethodNode withSelector: #emitToDo:on:value:"
	"ClassHierarchyDuplicator new editCode: (MessageNode class sourceCodeAt: #initialize) inClass: MessageNode class withSelector: #initialize"! !

!ClassHierarchyDuplicator methodsFor: 'initialize-release' stamp: 'eem 5/16/2008 15:53'!
initialize
	| globalNames |
	prefix := 'XXX'.
	classes := Set new.
	((SystemOrganization categories reject: [:c| c beginsWith: prefix]) select:
		[:c| ('*Compiler*' match: c) and: [#'Compiler-Morphing Bytecode Set' ~= c]]) do:
			[:cat|
			(SystemOrganization listAtCategoryNamed: cat) do:
				[:cn|
				classes add: (Smalltalk at: cn)]].
	globalNames := classes collect: [:ea| ea name asString].
	classes do: [:c| c classPool keys do: [:sym| globalNames add: sym asString]].
	xlator := [:s| (globalNames includes: s) ifTrue: [prefix, s] ifFalse: [s]]

	"self new edit"! !

Object subclass: #ClosureLabelsPrintEditor
	instanceVariableNames: 'xlator xlation kwdxlation ranges index output codeIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Morphing Bytecode Set'!

!ClosureLabelsPrintEditor methodsFor: 'code editing' stamp: 'eem 6/2/2008 11:47'!
edit: generateViewBar
	"self new edit: true"
	| sn |
	sn := SystemNavigation default.
	xlation keysAndValuesDo:
		[:s :t|
		(sn allImplementorsOf: s localTo: ParseNode) do:
			[:md| | newCode |
			newCode := self editCode: md sourceString inClass: md actualClass withSelector: md methodSymbol.
			generateViewBar
				ifTrue: [md actualClass compile: newCode classified: 'printing']
				ifFalse:
					[(StringHolder new textContents:
						(CodeDiffBuilder buildDisplayPatchFrom: md sourceString to: newCode))
							openLabel: md printString]]]! !

!ClosureLabelsPrintEditor methodsFor: 'code editing' stamp: 'eem 6/2/2008 11:39'!
editCode: code inClass: class withSelector: selector
	| codeString |
	ranges := SHParserST80 new rangesIn: code classOrMetaClass: class workspace: nil  environment: nil.
	index := 1.
	codeIndex := 1.
	output := String new writeStream.
	codeString := code asString.
	self process: codeString.
	^output contents! !

!ClosureLabelsPrintEditor methodsFor: 'initialize-release' stamp: 'eem 6/2/2008 11:38'!
initialize
	xlator := [:s| s copyReplaceAll: 'print' with: 'printWithClosureAnalysis'].
	xlation := Dictionary new.
	(((ParseNode withAllSubclasses removeAll: Encoder withAllSubclasses; yourself)
		inject: Set new
		into: [:s :c| s addAll: c selectors. s]) select:
			[:s| s beginsWith: 'print']) do:
				[:s|xlation at: s put: (xlator value: s) asSymbol].
	kwdxlation := Dictionary new.
	xlation keysAndValuesDo: [:k :v| kwdxlation at: k keywords first put: v keywords first]! !

!ClosureLabelsPrintEditor methodsFor: 'code editing' stamp: 'eem 6/2/2008 11:43'!
process: code
	| range kwd |
	[index <= ranges size] whileTrue:
		[range := ranges at: index.
		 index := index + 1.
		 [codeIndex < range start] whileTrue:
			[output nextPut: (code at: codeIndex).
			 codeIndex := codeIndex + 1].
		range type == #assignment
			ifTrue: [output nextPutAll: ':='.
					codeIndex := range end + 1]
			ifFalse:
				[(#(keyword patternKeyword) includes: range type) ifTrue:
					[kwd := code copyFrom: range start to: range end.
					(kwdxlation includesKey: kwd) ifTrue:
						[output nextPutAll: (kwdxlation at: kwd).
						 codeIndex := range end + 1]]
					ifFalse:
						[[codeIndex <= range end] whileTrue:
							[output nextPut: (code at: codeIndex).
							 codeIndex := codeIndex + 1].
						"kill whitespace after up-arrow in ^ expr"
						(range type == #return and: [index <= ranges size]) ifTrue:
							[codeIndex := (ranges at: index) start]]]]! !

Object subclass: #CogScripts
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Scripts'!

!CogScripts class methodsFor: 'closure scripts' stamp: 'eem 7/17/2008 18:56'!
bootstrapClosures
	"CogScripts bootstrapClosures"
	| rep |
	Transcript clear.
	rep := false
			ifTrue: [MCCacheRepository default]
			ifFalse:
				[MCHttpRepository
					location: 'http://dev.qwaq.com/ss/Oinq'
					user: 'qwaq'
					password: ''].
	"This changes load order in Monticello such that additions come in before modifications."
	(rep loadVersionFromFileNamed: 'Monticello-eem.302.mcz') load.
	"This adds some prereqs the compiler uses that are loaded in later packages:
	 Fix the ClassBuilder so redefining CompiledMethod can add and remove class variables.
	 Add Object/Array>>isArray.
	 Add new interface for accessing inst vars & fields on initializing the compiler (Encoder)."
	self bootstrapClosuresCompilerPreloadCode readStream fileIn.
	"This temporarily stops Monticello from unloading code on load and warning about overwriting changes.
	 Since changes span multiple packages need all additions in before any deletions occur.
	 Can't warn about anything until the new debugger api is installed."
	ChangeSet
		newChangesFromStream: self bootstrapClosuresNeuterMonticelloCode readStream
		named: 'neuterMonticello'.
	Smalltalk at: #DoNotUnload put: true.
	1 to: 2 do:
		 [:i|
		  #(	'Compiler-eem.30.mcz'
			'Files-eem.21.mcz'
			'Exceptions-eem.14.mcz'
			'Collections-eem.55.mcz'
			'Tools-eem.45.mcz'
			'Kernel-eem.82.mcz'
			'System-eem.53.mcz'
			'Brad-eem.51.mcz'
			'Morphic-eem.38.mcz'
			'Tweak-Compiler-eem.36.mcz'
			'Tweak-Hacks-eem.30.mcz'
			'Tweak-Basic-eem.151.mcz'
			'Tweak-Core-Proto-eem.56.mcz') do:
				[:pn|
				Transcript clear; nextPutAll: pn; space; nextPut: $(; print: i; nextPut: $); endEntry.
				(rep loadVersionFromFileNamed: pn) load].
		 Smalltalk at: #DoNotUnload put: false].
	"Now remove the temporary hacks to Monticello"
	(ChangeSet named: 'neuterMonticello') changedMessageList do:
		[:mr| | changeRecords |
		changeRecords := mr actualClass changeRecordsAt: mr methodSymbol.
		changeRecords second fileIn].
	"Install BlockClosure in the specialObjectsArray"
	Smalltalk recreateSpecialObjectsArray.
	"Throw the switch to compile to closures"
	self bootstrapClosuresClosureCompilerSwitchCode readStream fileIn.
	"Recompile the system except the one method we can't yet deal with in GeniePlugin (1 too many literals)"
	(Smalltalk forgetDoIts allClasses reject: [:c| c name == #GeniePlugin]) do:
		[:c|
		{ c. c class } do:
			[:b|
			Transcript cr; print: b; endEntry.
			b selectors asSortedCollection do:
				[:s|
				b recompile: s from: b]]].
	UsefulScripts postRecompileCleanup.
	self inform: 'Save and quit and then run UsefulScripts postRecompileCleanup.\Rinse and repeat' withCRs! !

!CogScripts class methodsFor: 'closure scripts' stamp: 'eem 6/16/2008 17:29'!
bootstrapClosuresClosureCompilerSwitchCode
	^'
!!Parser methodsFor: ''public access'' stamp: ''eem 5/15/2008 15:44''!!
encoder
	encoder isNil ifTrue:
		[encoder := EncoderForV3PlusClosures new].
	^encoder!! !!'! !

!CogScripts class methodsFor: 'closure scripts' stamp: 'eem 6/13/2008 14:52'!
bootstrapClosuresCompilerPreloadCode
	^'
!!ClassBuilder methodsFor: ''class format'' stamp: ''eem 6/13/2008 10:03''!!
computeFormat: type instSize: newInstSize forSuper: newSuper ccIndex: ccIndex
	"Compute the new format for making oldClass a subclass of newSuper.
	Return the format or nil if there is any problem."
	| instSize isVar isWords isPointers isWeak |
	type == #compiledMethod
		ifTrue:[^CompiledMethod format].
	instSize := newInstSize + (newSuper ifNil:[0] ifNotNil:[newSuper instSize]).
	instSize > 254 ifTrue:[
		self error: ''Class has too many instance variables ('', instSize printString,'')''.
		^nil].
	type == #normal ifTrue:[isVar := isWeak := false. isWords := isPointers := true].
	type == #bytes ifTrue:[isVar := true. isWords := isPointers := isWeak := false].
	type == #words ifTrue:[isVar := isWords := true. isPointers := isWeak := false].
	type == #variable ifTrue:[isVar := isPointers := isWords := true. isWeak := false].
	type == #weak ifTrue:[isVar := isWeak := isWords := isPointers := true].
	(isPointers not and:[instSize > 0]) ifTrue:[
		self error:''A non-pointer class cannot have instance variables''.
		^nil].
	^(self format: instSize 
		variable: isVar 
		words: isWords 
		pointers: isPointers 
		weak: isWeak) + (ccIndex bitShift: 11).!! !!

!!ClassBuilder methodsFor: ''public'' stamp: ''eem 6/13/2008 10:00''!!
superclass: aClass
	variableByteSubclass: t instanceVariableNames: f 
	classVariableNames: d poolDictionaries: s category: cat
	"This is the standard initialization message for creating a new class as a 
	subclass of an existing class in which the subclass is to 
	have indexable byte-sized nonpointer variables."
	| oldClassOrNil actualType |
	(aClass instSize > 0)
		ifTrue: [^self error: ''cannot make a byte subclass of a class with named fields''].
	(aClass isVariable and: [aClass isWords])
		ifTrue: [^self error: ''cannot make a byte subclass of a class with word fields''].
	(aClass isVariable and: [aClass isPointers])
		ifTrue: [^self error: ''cannot make a byte subclass of a class with pointer fields''].
	oldClassOrNil := aClass environment at: t ifAbsent:[nil].
	actualType := (oldClassOrNil notNil
				   and: [oldClassOrNil typeOfClass == #compiledMethod])
					ifTrue: [#compiledMethod]
					ifFalse: [#bytes].
	^self 
		name: t
		inEnvironment: aClass environment
		subclassOf: aClass
		type: actualType
		instanceVariableNames: f
		classVariableNames: d
		poolDictionaries: s
		category: cat!! !!

!!Array methodsFor: ''testing'' stamp: ''eem 5/8/2008 11:13''!!
isArray
	^true!! !!

!!Object methodsFor: ''testing'' stamp: ''eem 5/8/2008 11:13''!!
isArray
	^false!! !!

!!Behavior methodsFor: ''compiling'' stamp: ''eem 5/13/2008 09:50''!!
instVarNamesAndOffsetsDo: aBinaryBlock
	"This is part of the interface between the compiler and a class''s instance or field names.
	 The class should enumerate aBinaryBlock with the instance variable name strings and
	 their integer offsets.  The order is important. Names evaluated later will override the
	 same names occurring earlier."

	"Nothing to do here; ClassDescription introduces named instance variables"
	^self!! !!

!!ClassDescription methodsFor: ''compiling'' stamp: ''eem 5/13/2008 09:48''!!
instVarNamesAndOffsetsDo: aBinaryBlock
	"This is part of the interface between the compiler and a class''s instance or field names.
	 The class should enumerate aBinaryBlock with the instance variable name strings and
	 their integer offsets.  The order is important. Names evaluated later will override the
	 same names occurring earlier."

	| superInstSize |
	(superInstSize := superclass notNil ifTrue: [superclass instSize] ifFalse: [0]) > 0 ifTrue:
		[superclass instVarNamesAndOffsetsDo: aBinaryBlock].
	1 to: self instSize - superInstSize do:
		[:i| aBinaryBlock value: (instanceVariables at: i) value: i + superInstSize]!! !!


!!Behavior methodsFor: ''compiling'' stamp: ''eem 5/13/2008 09:50''!!
variablesAndOffsetsDo: aBinaryBlock
	"This is the interface between the compiler and a class''s instance or field names.  The
	 class should enumerate aBinaryBlock with the field definitions (with nil offsets) followed
	 by the instance variable name strings and their integer offsets.  The order is important.
	 Names evaluated later will override the same names occurring earlier."

	"Only need to do instance variables here.  CProtoObject introduces field definitions."
	self instVarNamesAndOffsetsDo: aBinaryBlock!! !!

!!CProtoObject class methodsFor: ''compiling'' stamp: ''eem 5/13/2008 09:53''!!
variablesAndOffsetsDo: aBinaryBlock
	"This is the interface between the compiler and a class''s instance or field names.  The
	 class should enumerate aBinaryBlock with the field definitions (with nil offsets) followed
	 by the instance variable name strings and their integer offsets.  The order is important.
	 Names evaluated later will override the same names occurring earlier."

	self allFieldsReverseDo: [:field| aBinaryBlock value: field value: nil].
	self instVarNamesAndOffsetsDo: aBinaryBlock!! !!'! !

!CogScripts class methodsFor: 'closure scripts' stamp: 'eem 6/13/2008 14:56'!
bootstrapClosuresNeuterMonticelloCode
	^'
!!MCVersionLoader methodsFor: ''checking'' stamp: ''eem 6/12/2008 17:30''!!
checkForModifications
	| modifications |
	modifications := versions select: [:ea | ea package workingCopy modified].
	true ifTrue: [^true].
	modifications isEmpty ifFalse: [self warnAboutLosingChangesTo: modifications].!! !!

!!MCClassDefinition methodsFor: ''installing'' stamp: ''eem 6/12/2008 17:53''!!
unload
	(Smalltalk at: #DoNotUnload ifAbsent: [false]) ifTrue: [^self].
	Smalltalk removeClassNamed: name!! !!

!!MCMethodDefinition methodsFor: ''installing'' stamp: ''eem 6/12/2008 17:53''!!
unload
	| previousVersion |
	(Smalltalk at: #DoNotUnload ifAbsent: [false]) ifTrue: [^self].
	self isOverrideMethod ifTrue: [previousVersion := self scanForPreviousVersion].
	previousVersion
		ifNil: [self actualClass ifNotNilDo: [:class | class removeSelector: selector]]
		ifNotNil: [previousVersion fileIn] !! !!

!!MCOrganizationDefinition methodsFor: ''as yet unclassified'' stamp: ''eem 6/12/2008 18:06''!!
postloadOver: oldDefinition
	[SystemOrganization categories:
		(self
			reorderCategories: SystemOrganization categories
			original: (oldDefinition ifNil: [#()] ifNotNil: [oldDefinition categories]))]
		on: Error
		do: [:ex| Transcript cr; print: ex messageText; endEntry]!! !!

!!MCPackageLoader methodsFor: ''private'' stamp: ''eem 6/12/2008 21:32''!!
tryToLoad: aDefinition
	[aDefinition loadOver: (self obsoletionFor: aDefinition)]
		on: Error
		do: [:ex| self halt. errorDefinitions add: aDefinition]!! !!'! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 6/11/2010 13:21'!
browseExclusiveMethodImportsForPrimitiveClass: primClass
	"Browse methods used by the class holding VM primitives and unused elsewhere."
	"self browseExclusiveMethodImportsForPrimitiveClass: StackInterpreterSPrimitives"
	Cursor execute showWhile:
		["The algorithm computes a fixed point of the methods used
		   by the parcel which are unused outside it.  The algorithm is
				compute messages sent within parcel
				repeat until at a fixed point:
					compute intersection of this with methods implemented outside parcel
					compute messages sent outside parcel not within the intersection
					remove these from intersection"

		| parcelMessages "<IdentitySet of: Symbol>
								messages sent within this parcel"
		  externalMethods "<Set of: MethodDescription>
								methods implemented outside this parcel"
		  danglingMethods "<Set of: MethodDescription>
								methods importeded only by this parcel"
		  previousDangling "<Set of: MethodDescription>
								intermediate value of danglingMethods (for determining if fixed point is reached)"
		  danglingMessages "<Set of: Symbol> selectors of methods in danglingMethods"
		  noLongerDangling "<Set of: Symbol> selectors to be moved from dangling" |

		parcelMessages := IdentitySet new.
		primClass methodsDo:
			[:meth| parcelMessages addAll: meth messages].
		"compute starting points for danglingMethods and externalMethods"
		danglingMethods := Set new: 50.
		externalMethods := Set new: 20000.
		(PackageInfo named: 'VMMaker') classes do:
			[:aBehavior| | extends |
			(aBehavior = primClass or: [aBehavior inheritsFrom: primClass]) ifFalse:
				[aBehavior selectorsAndMethodsDo:
					[:sel :meth|
					(primClass includesSelector: sel) ifFalse:
						[((parcelMessages includes: sel)
							ifTrue: [danglingMethods]
							ifFalse: [externalMethods])
								add: (MethodReference class: aBehavior selector: sel)]]]].
		["Now iterate to the fixed-point.  Any method in dangling with a selector sent
		  in external must be moved into external until dangling reaches its fixed point."
		previousDangling := danglingMethods copy.
		danglingMessages := danglingMethods collect: [:md| md selector].
		noLongerDangling := Set new: danglingMethods size.
		externalMethods do:
			[:md|
			md compiledMethod messages do:
				[:l|
				(danglingMessages includes: l) ifTrue:
					[noLongerDangling add: l]]].
		externalMethods := danglingMethods select: [:md| noLongerDangling includes: md selector].
		danglingMethods removeAll: externalMethods.
		danglingMethods size ~= previousDangling size] whileTrue.
		SystemNavigation default
			browseMessageList: danglingMethods asSortedCollection
			name: 'Methods Used Only by ', primClass name, ' but not in ', primClass name]! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 6/8/2010 16:46'!
changedMethodsForObjectMemorySends
	"Answer the methods in StackInterpreter and subclasses that change if sends to self for methods implemented
	 in ObjectMemory, NewObjectMemory (& NewObjectMemorySE ?) become sends to objectMemory."
	"CogScripts changedMethodsForObjectMemorySends"
	| selectors rules model environment sortedChanges |
	selectors := { ObjectMemory. NewObjectMemory. "NewObjectMemorySE" }
					inject: Set new
					into: [:sels :class| sels addAll: class selectors; yourself].
	rules := RBParseTreeRewriter new.
	rules
		replace: 'self `@method: ``@args'
		with: 'objectMemory `@method: ``@args'
		when: [:node| selectors includes: node selector].
	#(	'nilObj' 'trueObj' 'falseObj')
		do: [:instVar|
			rules
				replace: instVar, ' := ``@args' with: 'objectMemory ', instVar, 'ect: ``@args';
				replace: instVar with: 'objectMemory ', instVar, 'ect'].
	self readWriteVars do: [:instVar|
			rules
				replace: instVar, ' := ``@args' with: 'objectMemory ', instVar, ': ``@args';
				replace: instVar with: 'objectMemory ', instVar].
	self readOnlyVars do: [:instVar|
			rules replace: instVar with: 'objectMemory ', instVar].
	model := RBNamespace new.
	environment := BrowserEnvironment new forClasses:
						{ StackInterpreter. CoInterpreter. CoInterpreterMT.
						   StackInterpreterSimulator. CogVMSimulator }.
	environment classesAndSelectorsDo:
		[ :class :selector | | tree |
		"(class == StackInterpreter and: [selector == #isContextHeader:]) ifTrue: [self halt]."
		"(class == StackInterpreter and: [selector == #isIndexable:]) ifTrue: [self halt]."
		"(class == StackInterpreter and: [selector == #printContextCallStackOf:]) ifTrue: [self halt]."
		tree := class parseTreeFor: selector.
		(rules executeTree: tree) ifTrue:
			[model compile: rules tree newSource in: class classified: (class whichCategoryIncludesSelector: selector)]].
	false ifTrue: [model changes inspect].
	false ifTrue: "shortest change:"
		[(model changes changes inject: model changes changes first into: [:c1 :c2| c1 printString size < c2 printString size ifTrue: [c1] ifFalse: [c2]]) inspect].

	sortedChanges := model changes changes asSortedCollection:
						[:c1 :c2|
						c1 changeClass == c2 changeClass
							ifTrue: [c1 selector <= c2 selector]
							ifFalse: [c2 changeClass inheritsFrom: c1 changeClass]].
	true ifTrue:
		[MessageSetTextDifferencer
			openMessageList: (sortedChanges collect: [:mr| { MethodReference class: mr changeClass selector: mr selector. TextReference new text: mr source class: mr class selector: mr selector }])
			name: 'self foo <-> objectMemory foo et al' 
			autoSelect: nil].

	^sortedChanges! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 9/30/2010 13:20'!
createObjMemSims
	"self createObjMemSims"
	{ NewObjectMemory. NewCoObjectMemory } do:
		[:c| | sc oc nc |
		sc := Compiler evaluate: ((Class templateForSubclassOf: c category: c category, 'Simulation') copyReplaceAll: 'NameOfSubclass' with: c name, 'Simulator').
		false ifTrue:
			[oc := NewObjectMemorySimulator.
			oc selectors do:
				[:s| [sc compile: (oc sourceCodeAt: s) asString
						classified: (oc whichCategoryIncludesSelector: s)
						withStamp: (oc >> s) timeStamp
						notifying: nil] on: Error do: [:ex|]]].
		nc := Compiler evaluate: ((Class templateForSubclassOf: sc category: sc category) copyReplaceAll: 'NameOfSubclass' with: sc name, 'LSB').
		oc := nc selectors size > 0 ifTrue: [nc] ifFalse: [NewObjectMemorySimulatorLSB].
		oc selectors do:
			[:s| | aoc |
			aoc := (CogVMSimulatorLSB includesSelector: s) ifTrue: [CogVMSimulatorLSB] ifFalse: [NewObjectMemorySimulatorLSB].
			nc compile: (aoc sourceCodeAt: s) asString
					classified: (aoc whichCategoryIncludesSelector: s)
					withStamp: (aoc >> s) timeStamp
					notifying: nil].
		nc := Compiler evaluate: ((Class templateForSubclassOf: sc category: sc category) copyReplaceAll: 'NameOfSubclass' with: sc name, 'MSB').
		oc := NewObjectMemorySimulatorMSB.
		oc selectors do:
			[:s| nc compile: (oc sourceCodeAt: s) asString
					classified: (oc whichCategoryIncludesSelector: s)
					withStamp: (oc >> s) timeStamp
					notifying: nil]]! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 6/8/2010 16:54'!
createSVMTree
	"Create the parallel StackInterpreterS, CoInterpreterS tree in which
	 objectMemory is an inst var rather than ObjectMemory et al being a superclass"
	"CogScripts createSVMTree"

	| changes map |
	changes := Cursor execute showWhile: [self changedMethodsForObjectMemorySends].

	map := Cursor execute showWhile: [self createStackInterpreterSHierarchy].

	(ChangeSet superclassOrder: (StackInterpreter withAllSubclasses select: [:c| map includesKey: c]) asArray) do:
		[:sourceClass|
		sourceClass selectors do:
			[:sel| | destClass source stamp |
			destClass := map
							at: (((sel beginsWith: 'primitive')
								and: [sel last ~~ $:
								and: [sel ~~ #primitiveFail]])
									ifTrue: [{sourceClass. #primitives}]
									ifFalse: [sourceClass])
							ifAbsent: [map at: sourceClass].
			(changes detect: [:c| c changeClass == sourceClass and: [c selector = sel]] ifNone: [])
				ifNotNil:
					[:change|
					source := change source.
					stamp := Utilities changeStamp copyReplaceAll: Utilities authorInitials with: Utilities authorInitials, ' (objmem refactor)']
				ifNil:
					[source := sourceClass sourceCodeAt: sel.
					stamp := (sourceClass >> sel) timeStamp].
			[destClass
					compile: source
					classified: (sourceClass whichCategoryIncludesSelector: sel)
					withStamp: stamp
					notifying: nil]
				on: SyntaxErrorNotification
				do: [:ex| | newBrowser |
					newBrowser := Browser new setClass: destClass selector: nil.
					newBrowser selectMessageCategoryNamed: (sourceClass whichCategoryIncludesSelector: sel).
					Browser
						openBrowserView: (newBrowser openMessageCatEditString: source)
						label: 'category "', (sourceClass whichCategoryIncludesSelector: sel), '" in ', destClass name]]].

	self readWriteVars, self readOnlyVars do:
		[:sym|
		(NewObjectMemory whichClassIncludesSelector: sym) ifNil:
			[(NewObjectMemory whichClassDefinesInstVar: sym asString)
				compile: sym, (String with: Character cr with: Character tab with: $^), sym
				classified: #accessing]].
	self readWriteVars do:
		[:sym| | setter | setter := (sym, ':') asSymbol.
		(NewObjectMemory whichClassIncludesSelector: setter) ifNil:
			[(NewObjectMemory whichClassDefinesInstVar: sym asString)
				compile: setter, ' aValue', (String with: Character cr with: Character tab with: $^), sym, ' := aValue'
				classified: #accessing]].! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 6/2/2010 18:15'!
createStackInterpreterSHierarchy
	"Create the parallel StackInterpreterS, CoInterpreterS tree (without methods).
	 Answer a Dictionary maping source class to dest class with {source. #primitives} -> dest
	 for the added primitives classes."

	| map |
	(Smalltalk classNamed: #StackInterpreterS) ifNotNil:
		[:sis|
		(Object confirm: 'StackInterpreterS exists, nuke?') ifTrue:
			[(ChangeSet superclassOrder: sis withAllSubclasses asArray) reverseDo:
				[:sissc| sissc removeFromSystemUnlogged]]].

	map := Dictionary new.
	(ChangeSet superclassOrder: (StackInterpreter withAllSubclasses
									remove: SchizophrenicClosureFormatStackInterpreter;
									yourself) asArray) do:
		[:sisc| | def |
		def := sisc definition.
		def := sisc == StackInterpreter
				ifTrue: [((def copyReplaceAll: sisc superclass name, ' ' with: ObjectMemory superclass name, ' ')
							copyReplaceAll: 'instanceVariableNames: ''' with: 'instanceVariableNames: ''objectMemory ')
							copyReplaceAll: 'poolDictionaries: ''' with: 'poolDictionaries: ''', (ObjectMemory poolDictionaryNames fold: [:a :b| a, ' ', b]), ' ']
				ifFalse: [def copyReplaceAll: sisc superclass name, ' ' with: sisc superclass name, 'S '].
		def := def copyReplaceAll: sisc name printString with: sisc name printString, 'S'.
		map at: sisc put: (Compiler evaluate: def)].

	map at: {StackInterpreter. #primitives}
		put: (Compiler
				evaluate: 'StackInterpreterS subclass: #StackInterpreterSPrimitives
							instanceVariableNames: ''''
							classVariableNames: ''''
							poolDictionaries: ''''
							category: ''VMMaker-Interpreter''');
		at: {CoInterpreter. #primitives}
		put: (Compiler
				evaluate: 'CoInterpreterS subclass: #CoInterpreterSPrimitives
						instanceVariableNames: ''''
						classVariableNames: ''''
						poolDictionaries: ''''
						category: ''VMMaker-Interpreter''');
		at: {StackInterpreter. #objmem}
		put: (Compiler
				evaluate: 'NewObjectMemory subclass: #NewObjectMemoryS
						instanceVariableNames: ''coInterpreter''
							classVariableNames: ''''
							poolDictionaries: ''''
							category: ''VMMaker-Interpreter''');
		at: {CoInterpreter. #objmem}
		put: (Compiler
				evaluate: 'NewObjectMemoryS subclass: #NewCoObjectMemoryS
						instanceVariableNames: ''''
						classVariableNames: ''''
						poolDictionaries: ''''
						category: ''VMMaker-Interpreter''').

	"reparent subclasses underneath StackInterpreterSPrimitives & CoInterpreterSPrimitives"
	#(StackInterpreterS CoInterpreterS) do:
		[:cn|
		((Smalltalk classNamed: cn) subclasses reject: [:c| c name endsWith: 'Primitives']) do:
			[:sisc| | def |
			def := sisc definition.
			def := def copyReplaceAll: cn, ' ' with: cn, 'Primitives '.
			Compiler evaluate: def]].
	^map! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 6/9/2010 11:55'!
doClassSide
	"self doClassSide"
	| classes rules model env sortedChanges |
	classes := (PackageInfo named: 'VMMaker') classes.
	classes := classes select: [:c| classes anySatisfy: [:d| d name last = $S and: [d name allButLast = c name]]].

	rules := RBParseTreeRewriter new.
	model := RBNamespace new.
	env := BrowserEnvironment new forClasses: classes.
	classes do: [:c| rules replace: c name with: c name, 'S'].
	env classesAndSelectorsDo:
		[:class :selector| | tree |
		class isMeta ifTrue:
			[tree := class parseTreeFor: selector.
			rules executeTree: tree.
			model compile: rules tree newSource in: class classified: (class whichCategoryIncludesSelector: selector)]].
	false ifTrue: [model changes inspect].
	false ifTrue: "shortest change:"
		[(model changes changes inject: model changes changes first into: [:c1 :c2| c1 printString size < c2 printString size ifTrue: [c1] ifFalse: [c2]]) inspect].

	sortedChanges := model changes changes asSortedCollection:
						[:c1 :c2|
						c1 changeClass == c2 changeClass
							ifTrue: [c1 selector <= c2 selector]
							ifFalse: [c2 changeClass inheritsFrom: c1 changeClass]].
	true ifTrue:
		[MessageSetTextDifferencer
			openMessageList: (sortedChanges collect: [:mr| { MethodReference class: mr changeClass selector: mr selector. TextReference new text: mr source class: mr class selector: mr selector }])
			name: 'class side' 
			autoSelect: nil].

	sortedChanges do:
		[:mr|
		(Smalltalk classNamed: (mr changeClass theNonMetaClass name, 'S') asSymbol) class
			compile: mr source
			classified: (mr changeClass whichCategoryIncludesSelector: mr selector)
			withStamp: (mr source asString = (mr changeClass sourceCodeAt: mr selector) asString
							ifTrue: [(mr changeClass >> mr selector) timeStamp]
							ifFalse: [Utilities changeStamp copyReplaceAll: Utilities authorInitials with: Utilities authorInitials, ' (objmem refactor)'])
			notifying: nil]! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 6/8/2010 17:01'!
readOnlyVars
	^#(#checkForLeaks #fullGCLock #gcStartUsecs #memoryLimit #scavengeThreshold #youngStart
		#statCompMoveCount #statFullGCUsecs #statFullGCs #statGCEndUsecs #statGrowMemory #statIGCDeltaUsecs
		#statIncrGCUsecs #statIncrGCs #statMarkCount #statMkFwdCount #statRootTableCount #statRootTableOverflows
		#statShrinkMemory #statSpecialMarkCount #statSurvivorCount #statSweepCount #statTenures)! !

!CogScripts class methodsFor: 'separate vm scripts' stamp: 'eem 6/8/2010 17:06'!
readWriteVars
	^#(#specialObjectsOop
		#edenBytes #endOfMemory #forceTenureFlag #freeStart #needGCFlag
		#gcBiasToGrow #gcBiasToGrowGCLimit #gcBiasToGrowThreshold
		#growHeadroom #lastHash #lowSpaceThreshold #memory
		#remapBuffer #remapBufferCount #rootTable #rootTableCount
		#signalLowSpace #shrinkThreshold tenuringThreshold)! !

!CogScripts methodsFor: 'look on class side' stamp: 'eem 6/13/2008 14:32'!
seeClassSide
	"All the code is on the class side"! !

Object subclass: #CrashReportsMailer
	instanceVariableNames: 'mailDirectory destination startDate endDate rs causes soloTime meetingTime'
	classVariableNames: ''
	poolDictionaries: 'ChronologyConstants'
	category: 'Cog-Scripts'!

!CrashReportsMailer commentStamp: '<historical>' prior: 0!
This class generates an almost complete crash report that still needs editing for categories and then posts it after editing.!

!CrashReportsMailer class methodsFor: 'instance creation' stamp: 'eem 6/25/2010 14:16'!
mailDirectory: aFileDirectoryOrString destination: anEmailAddress week: startDate
	^self new
		mailDirectory: aFileDirectoryOrString
		destination: anEmailAddress
		startDate: (TimeStamp fromSeconds: startDate asDate asSeconds)
		endDate:  (TimeStamp fromSeconds: (startDate asDate addDays: 7) asSeconds)

	"(self mailDirectory: '/Users/eliot/Library/Mail/IMAP-emiranda@ussc9-mail01.qwaq.com/INBOX.imapmbox/Messages'
		destination: 'eng@teleplace.com'
		week: '05/15/2010') generateReport"! !

!CrashReportsMailer class methodsFor: 'instance creation' stamp: 'eem 6/28/2010 14:05'!
mailDirectory: aFileDirectoryOrString destination: anEmailAddress week: startDateString soloTime: soloDurationOrString meetingTime: meetingDurationOrString
	| startDate |
	startDate := ((startDateString includes: $-)
					ifTrue: [(#(2 3 1) collect: [:i| (startDateString subStrings: '-') at: i]) fold: [:a :b| a,'/',b]]
					ifFalse: [startDateString]) asDate.
	^self new
		mailDirectory: aFileDirectoryOrString
		destination: anEmailAddress
		startDate: (TimeStamp fromSeconds: startDate asSeconds)
		endDate:  (TimeStamp fromSeconds: (startDate addDays: 7) asSeconds - 1)
		soloTime: soloDurationOrString asDuration
		meetingTime: meetingDurationOrString asDuration

	"(self mailDirectory: '/Users/eliot/Library/Mail/IMAP-emiranda@ussc9-mail01.qwaq.com/INBOX.imapmbox/Messages'
		destination: 'eng@teleplace.com'
		week: '05/15/2010'
		soloTime: '20:19:38:55'
		meetingTime: '7:23:44:45') generateReport"! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/25/2010 20:41'!
addReportFor: lines
	rs	resetContents;
		tab;
		nextPutAll: ((lines detect: [:l| l beginsWith: 'Subject:']) ifNotNil:
							[:l| l allButFirst: (l indexOfSubCollection: ' Teleplace Bug')]);
		crtab: 2;
		nextPutAll: (lines detect: [:l| l beginsWith: 'Date:']);
		cr.
	((lines anySatisfy: [:l| (l beginsWith: 'OS Version') and: [l includesSubString: 'Mac OS X']])
		ifTrue: [self reportForMacOS: lines]
		ifFalse: [(lines anySatisfy: [:l| l beginsWith: 'Module:'])
					ifTrue: [self reportForOldWin32: lines]
					ifFalse: [self reportForNewWin32: lines]]) ifNotNil:
			[:cause|
			(causes at: cause ifAbsentPut: [OrderedCollection new])
				addLast: rs contents]! !

!CrashReportsMailer methodsFor: 'parsing' stamp: 'eem 6/28/2010 12:07'!
breakIntoLines: aString
	^self pruneBogusEmptyLines:
		(Array streamContents:
			[:os| | is ss char |
			is := aString readStream.
			ss := ReadWriteStream on: String new.
			[is atEnd] whileFalse:
				[(char := is next) = Character lf
					ifTrue:
						[os nextPut: ss contents.
						 ss resetContents]
					ifFalse: [ss nextPut: char]].
			ss position > 0 ifTrue:
				[os nextPut: ss contents]])! !

!CrashReportsMailer methodsFor: 'parsing' stamp: 'eem 8/9/2010 13:42'!
candidateMessagesDo: aBlock
	"Evaluate aBlock with the file names of every file newer than the start date minus some slop"
	| start |
	start := (startDate minusDays: 7) asSeconds.
	mailDirectory entries do:
		[:dirEntry|
		((dirEntry name endsWith: '.emlx')
		 and: [dirEntry creationTime > start]) ifTrue:
			[aBlock value: dirEntry name]]! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/25/2010 14:15'!
collectReportSummaries
	self candidateMessagesDo:
		[:fn|
		self ifCrashReport: fn do:
			[:lines|
			self addReportFor: lines]]! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 8/9/2010 11:36'!
generateFinalReport
	| count numMac ws |
	count := causes inject: 0 into: [:sum :collection| sum + collection size].
	rs reset.
	rs print: count; nextPutAll: ' VM Crashes. ? OpenGL. ? Audio plugin. ? Video plugin. ? Python plugin.  ? core VM. ? unknown.'; cr.
	numMac := (causes keys select: [:k| k beginsWith: 'Mac:']) size.
	rs nextPutAll: 'win: '; print: count- numMac; cr.
	rs nextPutAll: 'mac: '; print: numMac; cr; cr.
	self reportCrashRates: count.
	causes keys asSortedCollection do:
		[:k|
		(causes at: k) do:
			[:report| rs nextPutAll: report].
		rs nextPutAll: k; cr; cr].
	(ws := Workspace new contents: rs contents)
		acceptAction: [:s| (UIManager default confirm: 'Sure you want to send?') ifTrue:
							[self mailReport: s.
							 ws acceptAction:
								[(UIManager default confirm: 'You''ve already sent this. Sure you want to send again?') ifTrue:
										[self mailReport: s]]]];
		openLabel: 'Customer VM Crashes ', startDate date printString, ' to ', endDate date printString;
		shouldStyle: false! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/25/2010 14:15'!
generateReport
	causes := Dictionary new.
	self collectReportSummaries.
	self generateFinalReport! !

!CrashReportsMailer methodsFor: 'parsing' stamp: 'eem 8/9/2010 11:33'!
ifCrashReport: fileName do: aBlock
	"Answer the evaluation of aBlock with the lines for fileName if it is a crash report, or nil if not."
	| file lines subject |
	file := mailDirectory oldFileNamed: fileName.
	lines := [file contentsOfEntireFile]
				on: Error
				do: [:ex|
					file close.
					Transcript nextPutAll: fileName; nextPutAll: ': '; nextPutAll: ex messageText; flush.
					^nil].
	lines := self breakIntoLines: lines.
	subject := lines detect: [:l| l beginsWith: 'Subject:'] ifNone: [^nil].
	(subject includesSubString: 'Teleplace Bug Report: ') ifFalse: [^nil].
	#('@qwaq.com' '@teleplace.com' '@chonkaa.com' 'craig@netjam.org' 'skysound@mac.com') do:
		[:localEmail| (subject includesSubstring: localEmail caseSensitive: false) ifTrue: [^nil]].
	(lines anySatisfy:
		[:l|
		((l beginsWith: 'OS Version') and: [l includesSubString: 'Mac OS X'])
		or: [l beginsWith: 'Operating System:']]) ifFalse: [^nil].
	^aBlock value: lines! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/25/2010 20:03'!
macOSDateFor: dateString 
	| tokens |
	tokens := dateString subStrings: ' -.+'.
	^TimeStamp
		date: (Date year: tokens first asInteger month: tokens second asInteger day: tokens third asInteger)
		time: tokens fourth asTime! !

!CrashReportsMailer methodsFor: 'initialize-release' stamp: 'eem 6/25/2010 20:38'!
mailDirectory: aFileDirectoryOrString destination: anEmailAddress startDate: startTimestamp endDate: endTimestamp
	mailDirectory := aFileDirectoryOrString isString
						ifTrue: [FileDirectory on: aFileDirectoryOrString]
						ifFalse: [aFileDirectoryOrString].
	destination := anEmailAddress.
	startDate := startTimestamp.
	endDate := endTimestamp.
	rs := ReadWriteStream on: (String new: 256)! !

!CrashReportsMailer methodsFor: 'initialize-release' stamp: 'eem 6/28/2010 09:46'!
mailDirectory: aFileDirectoryOrString destination: anEmailAddress startDate: startTimestamp endDate: endTimestamp soloTime: soloDuration meetingTime: meetingDuration
	mailDirectory := aFileDirectoryOrString isString
						ifTrue: [FileDirectory on: aFileDirectoryOrString]
						ifFalse: [aFileDirectoryOrString].
	destination := anEmailAddress.
	startDate := startTimestamp.
	endDate := endTimestamp.
	soloTime := soloDuration.
	meetingTime := meetingDuration.
	rs := ReadWriteStream on: (String new: 256)! !

!CrashReportsMailer methodsFor: 'mailing' stamp: 'eem 6/28/2010 12:46'!
mailReport: aText
	| msg |
	msg := MailMessage new
			setField: 'from' toString: 'eliot@teleplace.com';
			setField: 'to' toString: destination;
			setField: 'subject' toString:	'Customer VM Crashes ',
										(startDate date printFormat: #(3 2 1 $/ 1 2)),
										' - ',
										((endDate - 1 seconds) date printFormat: #(3 2 1 $/ 1 2));
			body: (MIMEDocument contentType: 'text/plain' content: aText asString).
	SMTPClient
		deliverMailFrom: msg from
		to: (Array with: msg to)
		text: msg text
		usingServer: MailComposition new smtpServer! !

!CrashReportsMailer methodsFor: 'parsing' stamp: 'eem 6/28/2010 11:31'!
pruneBogusEmptyLines: lines
	"Eliminate duplicate empty lines from broken CR-LF to LF-LF conversions."
	| firstEmpty lastEmpty |
	(firstEmpty := lines findFirst: [:l| l isEmpty]) = 0 ifTrue:
		[^lines].
	lastEmpty := firstEmpty.
	[(lines at: lastEmpty + 2) isEmpty] whileTrue:
		[lastEmpty := lastEmpty + 2].
	^(lines copyFrom: 1 to: firstEmpty - 1),
	   ((firstEmpty + 1 to: lastEmpty - 1 by: 2) collect: [:i| lines at: i]),
	   (lines copyFrom: lastEmpty + 1 to: lines size)! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/28/2010 14:15'!
reportCrashRates: count
	{ soloTime. meetingTime }
		with: #('solo hours: ' 'meeting hours: ')
		do: [:time :label|
			rs	nextPutAll: label; print: (time asSeconds / SecondsInHour) rounded;
				space; nextPut: $(; print: time; nextPut: $); cr].
	rs cr.
	{ soloTime. meetingTime }
		with: #('solo' 'meeting')
		do: [:time :label|
			{ SecondsInHour. SecondsInDay }
				with: #('hour' 'day')
				do: [:period :periodLabel| | rate |
					rate := count / (time asSeconds / period).
					rs	print: (rate roundTo: (rate >= 1 ifTrue: [0.1] ifFalse: [0.001]));
						nextPutAll: ' crashes per '; nextPutAll: label; space; nextPutAll: periodLabel; cr]].
	rs cr; cr! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/28/2010 12:18'!
reportForMacOS: lines
	| thread dateAndTime cStackTop cstIdx |

	thread := Integer readFrom: ((lines detect: [:l| l beginsWith: 'Crashed Thread: ']) allButFirst: 15) readStream skipSeparators.
	thread := 'Thread ', thread printString, ' Crashed'.
	lines withIndexDo:
		[:l :i|
		(l beginsWith: 'Date/Time:') ifTrue:
			[dateAndTime := l allButFirst: 11].
		((l beginsWith: thread)
		and: [i < lines size]) ifTrue:
			[cStackTop := lines at: (cstIdx := i + 1)]].
	[cstIdx <= lines size
	 and: [(lines at: cstIdx) notEmpty]] whileTrue:
		[(((lines at: cstIdx) includesSubString: ' _sigtramp ')
		 and: [((lines at: cstIdx + 1) includesSubString: ' ??? ')
		 and: [((lines at: cstIdx + 1) includesSubString: '0xffffffff 0 + 4294967295')]]) ifTrue:
			[cStackTop := lines at: cstIdx + 2.
			 cstIdx := lines size].
		 cstIdx := cstIdx + 1].
	dateAndTime := self macOSDateFor: dateAndTime.
	(dateAndTime between: startDate and: endDate) ifFalse:
		[^nil].
	^'Mac:', (cStackTop allButFirst: (cStackTop indexOf: Character space))! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 7/20/2010 15:22'!
reportForNewWin32: lines
	| dateAndTime reason stStackTop cStackTop |

	lines withIndexDo:
		[:l :i|
		(l beginsWith: 'Session Summary:') ifTrue:
			[^nil]. "Not a VM bug report, ignore it."
		(dateAndTime isNil
		and: [(l beginsWith: '--------------------')
		and: [i < lines size]]) ifTrue:
			[dateAndTime := lines at: i + 1].
		((l beginsWith: 'Smalltalk stack dump:')
		and: [i < lines size]) ifTrue:
			[stStackTop := lines at: i + 1].
		((l beginsWith: 'Stack backtrace:')
		and: [i < lines size]) ifTrue:
			[cStackTop := lines at: i + 1]].
	dateAndTime isNil ifTrue:
		[^nil].
	dateAndTime := self win32DateFor: dateAndTime.
	(dateAndTime between: startDate and: endDate) ifFalse:
		[^nil].
	reason := (lines detect: [:l| l beginsWith: 'Reason: '] ifNone: []) ifNotNil: [:rl| rl allButFirst: 8].
	"a stack entry looks like
		 0x9c2490 I NetNameResolver class>localHostAddress 271185968: a(n) NetNameResolver class
		t293325136 s BlockClosure>on:do:"
	stStackTop ifNotNil:
		[stStackTop := (stStackTop subStrings: ' ') allButFirst: 2.
		 stStackTop := stStackTop first: (stStackTop findFirst: [:t| t first isDigit]) - 1.
		 stStackTop := stStackTop fold: [:a :b| a, ' ', b]].
	^String streamContents:
		[:s|
		reason ifNotNil: [s nextPutAll: reason; cr].
	 	stStackTop ifNotNil: [s nextPutAll: stStackTop; cr].
		cStackTop ifNotNil: [s nextPutAll: cStackTop]]! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/28/2010 11:54'!
reportForOldWin32: lines
	| dateAndTime module idx stackTop |

	lines withIndexDo:
		[:l :i|
		(dateAndTime isNil
		and: [(l beginsWith: '--------------------')
		and: [i < lines size]]) ifTrue:
			[dateAndTime := lines at: i + 1].
		((l beginsWith: 'Stack dump:')
		and: [i < lines size]) ifTrue:
			 "Occasionally there is a bogus blank line between the label and the first entry"
			[(stackTop := (lines at: i + 1) isEmpty) ifTrue:
				[stackTop := lines at: i + 2]]].
	dateAndTime := self win32DateFor: dateAndTime.
	(dateAndTime between: startDate and: endDate) ifFalse:
		[^nil].
	module := (lines detect: [:l| l beginsWith: 'Module: ']) allButFirst: 8.
	(idx := module indexOfSubCollection: '\Teleplace') > 0 ifTrue:
		[module := module copyFrom: idx to: module size].
	(module first isLetter and: [module second = $:]) ifTrue:
		[module := module allButFirst: 2].
	"a stack entry looks like
		 0x9c2490 I NetNameResolver class>localHostAddress 271185968: a(n) NetNameResolver class
		293325136 s BlockClosure>on:do:
	 but if there is no stack there may be any old crap after Stack dump: so squash errors."
	[stackTop := (stackTop subStrings: ' ') allButFirst: 2.
	 stackTop := stackTop first: (stackTop findFirst: [:t| t first isDigit]) - 1.
	 stackTop := stackTop fold: [:a :b| a, ' ', b]]
		on: Error
		do: [:ex| stackTop := '?'].
	^stackTop, ' in ', module! !

!CrashReportsMailer methodsFor: 'reporting' stamp: 'eem 6/25/2010 14:10'!
win32DateFor: dateAndTimeString 
	"transform 'Thu Jun 24 14:11:18 2010' to 'Jun 24 2010 14:11:18' to allow parsing"
	| tokens |
	tokens := dateAndTimeString subStrings: ' '.
	^({ tokens second. tokens third. tokens fifth. tokens fourth } fold: [:a :b| a, ' ', b]) asTimeStamp! !

Object subclass: #MultiProcessor
	instanceVariableNames: 'mutex processor guardedProcessorProtocol unguardedProcessorProtocol owner registerState coInterpreter threadIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Processors'!

!MultiProcessor class methodsFor: 'instance creation' stamp: 'eem 10/19/2010 14:52'!
for: aProcessorAlien coInterpreter: coInterpreter
	^self new
		processor: aProcessorAlien;
		coInterpreter: coInterpreter;
		yourself! !

!MultiProcessor methodsFor: 'initialize-release' stamp: 'eem 10/19/2010 14:32'!
coInterpreter: aCoInterpreter
	coInterpreter := aCoInterpreter! !

!MultiProcessor methodsFor: 'message forwarding' stamp: 'eem 10/20/2010 17:28'!
doesNotUnderstand: aMessage
	"Forward a message to the actual processor, managing a thread-switch if necessary.
	 Catch ProcessorSimulationTraps and raise them outside of the critical section to
	 avoid deadlock when reentering the VM from a trap and switching threads in the run-time."
	| selector result trap |
	selector := aMessage selector.
	(guardedProcessorProtocol includes: selector) ifFalse:
		[^(unguardedProcessorProtocol includes: selector)
			ifTrue: [processor perform: selector withArguments: aMessage arguments]
			ifFalse: [super doesNotUnderstand: aMessage]].
	result := [mutex critical:
				[owner ~~ mutex owningProcess ifTrue:
					[owner ifNotNil:
						[registerState at: owner put: processor registerState].
					 (registerState at: (owner := mutex owningProcess) ifAbsent: nil)
						ifNil: [coInterpreter initializeProcessorForThreadIndex: (threadIndex := threadIndex + 1)]
						ifNotNil: [:newState| processor setRegisterState: newState]].
				 processor perform: selector withArguments: aMessage arguments]]
					on: ProcessorSimulationTrap, Error
					do: [:ex| trap := ex].
	^trap ifNil: [result] ifNotNil: [trap signal]! !

!MultiProcessor methodsFor: 'initialize-release' stamp: 'eem 10/20/2010 12:42'!
initialize
	registerState := WeakIdentityKeyDictionary new.
	mutex := Mutex new.
	threadIndex := 0.
	guardedProcessorProtocol := unguardedProcessorProtocol := Set new! !

!MultiProcessor methodsFor: 'initialize-release' stamp: 'eem 10/20/2010 12:43'!
processor: aProcessor
	processor := aProcessor.
	guardedProcessorProtocol := aProcessor class selectors asSet.
	unguardedProcessorProtocol := #(#'Cog API' #opcodes #disassembly #printing)
										inject: Set new
										into: [ :protocol :category|
											protocol
												addAll: (aProcessor class organization listAtCategoryNamed: category);
												yourself].
	guardedProcessorProtocol removeAll: unguardedProcessorProtocol! !

Object subclass: #ShootoutBody
	instanceVariableNames: 'x y z vx vy vz mass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutBody class methodsFor: 'constants' stamp: ''!
daysPerYear
	^365.24! !

!ShootoutBody class methodsFor: 'constants' stamp: ''!
jupiter
	^self new
		x: 4.84143144246472090
		y: -1.16032004402742839
		z: -1.03622044471123109e-1
		vx: 1.66007664274403694e-3 * self daysPerYear
		vy: 7.69901118419740425e-3 * self daysPerYear
		vz: -6.90460016972063023e-5 * self daysPerYear
		mass: 9.54791938424326609e-4 * self solarMass! !

!ShootoutBody class methodsFor: 'constants' stamp: ''!
neptune
	^self new
		x: 1.53796971148509165e1
		y: -2.59193146099879641e1
		z: 1.79258772950371181e-1
		vx: 2.68067772490389322e-3 * self daysPerYear
		vy: 1.62824170038242295e-3 * self daysPerYear
		vz: -9.51592254519715870e-5 * self daysPerYear
		mass: 5.15138902046611451e-5 * self solarMass! !

!ShootoutBody class methodsFor: 'constants' stamp: ''!
pi
	^3.141592653589793! !

!ShootoutBody class methodsFor: 'constants' stamp: ''!
saturn
	^self new
		x: 8.34336671824457987
		y: 4.12479856412430479
		z: -4.03523417114321381e-1
		vx: -2.76742510726862411e-3 * self daysPerYear
		vy: 4.99852801234917238e-3 * self daysPerYear
		vz: 2.30417297573763929e-5 * self daysPerYear
		mass: 2.85885980666130812e-4 * self solarMass! !

!ShootoutBody class methodsFor: 'constants' stamp: 'eem 7/28/2008 12:42'!
solarMass
	^4.0 * self pi * self pi! !

!ShootoutBody class methodsFor: 'constants' stamp: ''!
sun
	^self new
		x: 0.0
		y: 0.0
		z: 0.0
		vx: 0.0
		vy: 0.0
		vz: 0.0
		mass: self solarMass! !

!ShootoutBody class methodsFor: 'constants' stamp: ''!
uranus
	^self new
		x: 1.28943695621391310e1
		y: -1.51111514016986312e1
		z: -2.23307578892655734e-1
		vx: 2.96460137564761618e-3 * self daysPerYear
		vy: 2.37847173959480950e-3 * self daysPerYear
		vz: -2.96589568540237556e-5 * self daysPerYear
		mass: 4.36624404335156298e-5 * self solarMass! !

!ShootoutBody methodsFor: 'nbody' stamp: ''!
addMomentumTo: anArray
	anArray at: 1 put: (anArray at: 1) + (vx * mass).
	anArray at: 2 put: (anArray at: 2) + (vy * mass).
	anArray at: 3 put: (anArray at: 3) + (vz * mass).
	^anArray! !

!ShootoutBody methodsFor: 'nbody' stamp: ''!
and: aBody velocityAfter: dt
	| dx dy dz distance mag |
	dx := x - aBody x.
	dy := y - aBody y.
	dz := z - aBody z.

	distance := ((dx*dx) + (dy*dy) + (dz*dz)) sqrt.
	mag := dt / (distance * distance * distance).

	self decreaseVelocity: dx y: dy z: dz m: aBody mass * mag.
	aBody increaseVelocity: dx y: dy z: dz m: mass * mag! !

!ShootoutBody methodsFor: 'nbody' stamp: ''!
decreaseVelocity: dx y: dy z: dz m: m
	vx := vx - (dx * m).
	vy := vy - (dy * m).
	vz := vz - (dz * m)! !

!ShootoutBody methodsFor: 'nbody' stamp: ''!
increaseVelocity: dx y: dy z: dz m: m
	vx := vx + (dx * m).
	vy := vy + (dy * m).
	vz := vz + (dz * m)! !

!ShootoutBody methodsFor: 'nbody' stamp: 'eem 7/28/2008 12:44'!
kineticEnergy
	^0.5 * mass * ((vx * vx) + (vy * vy) + (vz * vz))! !

!ShootoutBody methodsFor: 'accessing' stamp: ''!
mass
	^mass! !

!ShootoutBody methodsFor: 'nbody' stamp: ''!
offsetMomentum: anArray
	| m |
	m := self class solarMass.
	vx := (anArray at: 1) negated / m.
	vy := (anArray at: 2) negated / m.
	vz := (anArray at: 3) negated / m! !

!ShootoutBody methodsFor: 'nbody' stamp: ''!
positionAfter: dt
	x := x + (dt * vx).
	y := y + (dt * vy).
	z := z + (dt * vz)! !

!ShootoutBody methodsFor: 'nbody' stamp: ''!
potentialEnergy: aBody
	| dx dy dz distance |
	dx := x - aBody x.
	dy := y - aBody y.
	dz := z - aBody z.

	distance := ((dx*dx) + (dy*dy) + (dz*dz)) sqrt.
	^mass * aBody mass / distance! !

!ShootoutBody methodsFor: 'accessing' stamp: ''!
x
	^x! !

!ShootoutBody methodsFor: 'accessing' stamp: ''!
x: d1 y: d2 z: d3 vx: d4 vy: d5 vz: d6 mass: d7
	x := d1.
	y := d2.
	z := d3.
	vx := d4.
	vy := d5.
	vz := d6.
	mass := d7! !

!ShootoutBody methodsFor: 'accessing' stamp: ''!
y
	^y! !

!ShootoutBody methodsFor: 'accessing' stamp: ''!
z
	^z! !

Object subclass: #ShootoutChameneosColour
	instanceVariableNames: 'color'
	classVariableNames: 'Blue Red Yellow'
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutChameneosColour class methodsFor: 'accessing' stamp: ''!
blue
   ^Blue! !

!ShootoutChameneosColour class methodsFor: 'accessing' stamp: ''!
blue: anObject
   Blue := anObject! !

!ShootoutChameneosColour class methodsFor: 'initialize-release' stamp: ''!
createBlue
   "comment stating purpose of message"

   ^super new color: #blue! !

!ShootoutChameneosColour class methodsFor: 'initialize-release' stamp: ''!
createRed
   "comment stating purpose of message"

   ^super new color: #red! !

!ShootoutChameneosColour class methodsFor: 'initialize-release' stamp: ''!
createYellow
   "comment stating purpose of message"

   ^super new color: #yellow! !

!ShootoutChameneosColour class methodsFor: 'printing' stamp: ''!
generateReportOfColoursOn: readOut
   | colours |
   colours := Array
            with: Blue
            with: Red
            with: Yellow.
   colours do:
         [:aColour |
         colours do:
               [:anotherColour |
               aColour printOn: readOut.
               readOut nextPutAll: ' + '.
               anotherColour printOn: readOut.
               readOut nextPutAll: ' -> '.
               (aColour complementaryColourFor: anotherColour) printOn: readOut.
               readOut nl]].
   ^readOut! !

!ShootoutChameneosColour class methodsFor: 'initialize-release' stamp: ''!
initialize
   "self initialize"

   Red := self createRed.
   Blue := self createBlue.
   Yellow := self createYellow! !

!ShootoutChameneosColour class methodsFor: 'accessing' stamp: ''!
red
   ^Red! !

!ShootoutChameneosColour class methodsFor: 'accessing' stamp: ''!
red: anObject
   Red := anObject! !

!ShootoutChameneosColour class methodsFor: 'accessing' stamp: ''!
yellow
   ^Yellow! !

!ShootoutChameneosColour class methodsFor: 'accessing' stamp: ''!
yellow: anObject
   Yellow := anObject! !

!ShootoutChameneosColour methodsFor: 'accessing' stamp: ''!
color
   ^color! !

!ShootoutChameneosColour methodsFor: 'accessing' stamp: ''!
color: aColor
   color := aColor! !

!ShootoutChameneosColour methodsFor: 'as yet unclassified' stamp: ''!
complementaryColourFor: aChameneosColour
   "determine the complementary colour defined as..."

   self == aChameneosColour ifTrue: [^self].
   self isBlue
      ifTrue:
         [aChameneosColour isRed
            ifTrue: [^self class yellow]
            ifFalse: [^self class red]].
   self isRed
      ifTrue:
         [aChameneosColour isBlue
            ifTrue: [^self class yellow]
            ifFalse: [^self class blue]].
   aChameneosColour isBlue
      ifTrue: [^self class red]
      ifFalse: [^self class blue]! !

!ShootoutChameneosColour methodsFor: 'testing' stamp: ''!
hasSameColorAs: aChameneos
   ^self color == aChameneos color! !

!ShootoutChameneosColour methodsFor: 'testing' stamp: ''!
isBlue
   ^self == self class blue! !

!ShootoutChameneosColour methodsFor: 'testing' stamp: ''!
isRed
   ^self == self class red! !

!ShootoutChameneosColour methodsFor: 'testing' stamp: ''!
isYellow
   ^self == self class yellow! !

!ShootoutChameneosColour methodsFor: 'printing' stamp: ''!
printOn: aStream
   aStream nextPutAll: self color! !

Object subclass: #ShootoutCreature
	instanceVariableNames: 'creatureName colour selfMet creaturesMet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutCreature class methodsFor: 'initialize-release' stamp: 'eem 8/1/2008 16:15'!
withName: aName colour: aColour
   ^(ShootoutCreature new initialize)
      name: aName;
      colour: aColour! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
colour
   ^colour! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
colour: anObject
   colour := anObject! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
creaturesMet
   ^creaturesMet! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
creaturesMet: anObject
   creaturesMet := anObject! !

!ShootoutCreature methodsFor: 'initialize-release' stamp: ''!
initialize
   selfMet := 0.
   creaturesMet := 0! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
name
   ^creatureName! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
name: anObject
   creatureName := anObject! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
selfMet
   ^selfMet! !

!ShootoutCreature methodsFor: 'accessing' stamp: ''!
selfMet: anObject
   ^selfMet := anObject! !

!ShootoutCreature methodsFor: 'controlling' stamp: ''!
visitMall: mall

   [| partner |
   partner := mall visitWith: self.
   partner ifNotNil:
         [colour := colour complementaryColourFor: partner colour.
         self == partner ifTrue: [selfMet := selfMet + 1].
         creaturesMet := creaturesMet + 1].
   partner isNil]
         whileFalse! !

Object subclass: #ShootoutMall
	instanceVariableNames: 'guard maxRendezvous open process queue cache pairCache'
	classVariableNames: 'Units'
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutMall class methodsFor: 'initialize-release' stamp: ''!
createAllowing: maxRendezvous
   "Private"

   ^self basicNew initialize maxRendezvous: maxRendezvous! !

!ShootoutMall class methodsFor: 'initialize-release' stamp: 'eem 8/1/2008 16:15'!
createCreaturesWith: aCollectionOfColours
   "Private"

   | aName |
   aName := 0.
   ^aCollectionOfColours collect:
         [:aColour |
         aName := aName + 1.
         ShootoutCreature withName: aName colour: aColour]! !

!ShootoutMall class methodsFor: 'printing' stamp: ''!
generateReportFor: creatures printOn: stream
   | sum |
   sum := creatures inject: 0 into: [:accum :each | accum + each creaturesMet].
   creatures do:
         [:aCreature |
         aCreature creaturesMet printOn: stream.
         stream
            space;
            nextPutAll: (self units at: aCreature selfMet + 1);
            nl].
   stream space.
   sum printString
      do: [:el | stream nextPutAll: (self units at: el digitValue + 1)]
      separatedBy: [stream space].
   ^stream! !

!ShootoutMall class methodsFor: 'printing' stamp: ''!
generateReportForColours: colours printOn: stream
   stream space.
   colours do: [:colour | colour printOn: stream] separatedBy: [stream space].
   ^stream! !

!ShootoutMall class methodsFor: 'initialize-release' stamp: ''!
initialize
   "self initialize"

   Units := #('zero' 'one' 'two' 'three' 'four' 'five' 'six' 'seven' 'eight' 'nine')! !

!ShootoutMall class methodsFor: 'initialize-release' stamp: ''!
new
   ^self shouldNotImplement! !

!ShootoutMall class methodsFor: 'private' stamp: ''!
openMall: aMall forCreatures: creatures usingGuard: sema
   | processes |
   processes := creatures
            collect: [:aCreature |
               [aCreature visitMall: aMall.
               sema signal] newProcess].
   processes do:
         [:proc |
         proc priority: Processor userBackgroundPriority.
         proc resume]! !

!ShootoutMall class methodsFor: 'initialize-release' stamp: ''!
openMallWith: aCollectionOfColours forNumberOfMeets: aNumber
   | mall creatures guard |
   mall := self createAllowing: aNumber.
   mall run.
   creatures := self createCreaturesWith: aCollectionOfColours.
   guard := Semaphore new.
   self
      openMall: mall
      forCreatures: creatures
      usingGuard: guard.
   self
      waitForClosingOfMall: mall
      withCreatures: creatures
      usingGuard: guard.
   ^creatures! !

!ShootoutMall class methodsFor: 'public' stamp: 'eem 8/1/2008 16:16'!
runBenchMark: number on: anOutputStream
   "self runBenchMark: 60000 on: Transcript."

   | firstTestColours secondTestColours blue red yellow creatures |
   blue := ShootoutChameneosColour blue.
   red := ShootoutChameneosColour red.
   yellow := ShootoutChameneosColour yellow.
   firstTestColours := Array
            with: blue
            with: red
            with: yellow.
   secondTestColours := (OrderedCollection new)
            add: blue;
            add: red;
            add: yellow;
            add: red;
            add: yellow;
            add: blue;
            add: red;
            add: yellow;
            add: red;
            add: blue;
            yourself.
   (ShootoutChameneosColour generateReportOfColoursOn: anOutputStream) nl.
   (self generateReportForColours: firstTestColours printOn: anOutputStream)
      nl.
   creatures := ShootoutMall openMallWith: firstTestColours forNumberOfMeets: number.
   (self generateReportFor: creatures printOn: anOutputStream)
      nl;
      nl.
   (self generateReportForColours: secondTestColours printOn: anOutputStream)
      nl.
   creatures := ShootoutMall openMallWith: secondTestColours forNumberOfMeets: number.
   (self generateReportFor: creatures printOn: anOutputStream)
      nl;
      nl! !

!ShootoutMall class methodsFor: 'accessing' stamp: ''!
units
   ^Units! !

!ShootoutMall class methodsFor: 'private' stamp: ''!
waitForClosingOfMall: aMall withCreatures: creatures usingGuard: guard
   creatures size timesRepeat: [guard wait].
   aMall close! !

!ShootoutMall methodsFor: 'controlling' stamp: ''!
close
   open := false! !

!ShootoutMall methodsFor: 'initialize-release' stamp: 'eem 8/1/2008 16:14'!
initialize
   guard := Semaphore forMutualExclusion.
   queue := SharedQueue new.
   cache := OrderedCollection new.
   1 to: 10 do: [:x | cache add: ShootoutPair new]! !

!ShootoutMall methodsFor: 'accessing' stamp: ''!
maxRendezvous: max
   maxRendezvous := max! !

!ShootoutMall methodsFor: 'private' stamp: ''!
obtainPair
   ^cache removeFirst! !

!ShootoutMall methodsFor: 'private' stamp: ''!
processVisitors
   [open] whileTrue:
         [1 to: maxRendezvous
            do:
               [:x |
               | first second |
               first := queue next.
               second := queue next.
               self setPartnersOn: first and: second.
               first signal.
               second signal].
         [queue isEmpty] whileFalse: [queue next signal]].
   process terminate.
   process := nil! !

!ShootoutMall methodsFor: 'private' stamp: ''!
releasePair: pair
   pair release.
   cache addFirst: pair! !

!ShootoutMall methodsFor: 'initialize-release' stamp: ''!
run
   open := true.
   process ifNil:
         [process := [self processVisitors] newProcess.
         process priority: Processor userBackgroundPriority -1 ].
   process resume! !

!ShootoutMall methodsFor: 'private' stamp: ''!
setPartnersOn: first and: second
   first partner: second me.
   second partner: first me.
! !

!ShootoutMall methodsFor: 'private' stamp: ''!
shutdown
   [queue isEmpty] whileFalse: [queue next signal].
   process terminate.
   process := nil! !

!ShootoutMall methodsFor: 'controlling' stamp: ''!
visitWith: aChameneos
   | pair partner |
   pair := self obtainPair.
   pair me: aChameneos.
   queue nextPut: pair.
   pair wait.
   partner := pair partner.
   self releasePair: pair.
   ^partner! !

Object subclass: #ShootoutNBodySystem
	instanceVariableNames: 'bodies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutNBodySystem methodsFor: 'nbody' stamp: ''!
after: dt
	1 to: bodies size do: [:i|
		i+1 to: bodies size do: [:j|
			(bodies at: i) and: (bodies at: j) velocityAfter: dt].
	].
	bodies do: [:each| each positionAfter: dt]! !

!ShootoutNBodySystem methodsFor: 'nbody' stamp: ''!
energy
	| e |
	e := 0.0.
	1 to: bodies size do: [:i|
		e := e + (bodies at: i) kineticEnergy.

		i+1 to: bodies size do: [:j|
			e := e - ((bodies at: i) potentialEnergy: (bodies at: j))].
	].
	^e! !

!ShootoutNBodySystem methodsFor: 'initialize-release' stamp: 'eem 8/1/2008 16:08'!
initialize
	bodies := (OrderedCollection new
		add: ShootoutBody sun; add: ShootoutBody jupiter; add: ShootoutBody saturn;
		add: ShootoutBody uranus; add: ShootoutBody neptune; yourself) asArray.

	bodies first offsetMomentum:
		(bodies inject: (Array with: 0.0 with: 0.0 with: 0.0)
			into: [:m :each | each addMomentumTo: m])! !

Object subclass: #ShootoutPair
	instanceVariableNames: 'partner me sema'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutPair class methodsFor: 'instance creation' stamp: ''!
new
   "Answer a newly created and initialized instance."
   ^super new initialize.! !

!ShootoutPair class methodsFor: 'instance creation' stamp: ''!
with: me
   "Answer a newly created and initialized instance."
self halt.
   ^super new initialize me: me! !

!ShootoutPair methodsFor: 'initialize-release' stamp: ''!
initialize
   "Initialize a newly created instance. This method must answer the receiver."

   partner := nil.
   me := nil.
   sema := Semaphore new.
   ^self! !

!ShootoutPair methodsFor: 'accessing' stamp: ''!
me
   ^me! !

!ShootoutPair methodsFor: 'accessing' stamp: ''!
me: anObject
   me := anObject! !

!ShootoutPair methodsFor: 'accessing' stamp: ''!
partner
   ^partner! !

!ShootoutPair methodsFor: 'accessing' stamp: ''!
partner: anObject
   partner := anObject! !

!ShootoutPair methodsFor: 'initialize-release' stamp: ''!
release
partner:=nil.! !

!ShootoutPair methodsFor: 'initialize-release' stamp: ''!
signal
   sema signal! !

!ShootoutPair methodsFor: 'initialize-release' stamp: ''!
wait
   sema wait! !

Object subclass: #ShootoutTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutTests class methodsFor: 'platform' stamp: 'eem 11/17/2008 12:37'!
arg
	3 to: 5 do:
		[:i|
		(SmalltalkImage current getSystemAttribute: i) ifNotNil:
			[:aString|
			aString asInteger ifNotNil:
				[:arg| ^arg]]].
	^nil! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: ''!
binarytrees
	self binarytrees: self arg to: self stdout.
	^''! !

!ShootoutTests class methodsFor: 'benchmarking' stamp: 'eem 8/1/2008 16:10'!
binarytrees: n to: output
	| minDepth maxDepth stretchDepth check longLivedTree iterations |
	minDepth := 4.
	maxDepth := minDepth + 2 max: n.
	stretchDepth := maxDepth + 1.

	check := (ShootoutTreeNode bottomUpTree: 0 depth: stretchDepth) itemCheck.
	output
		nextPutAll: 'stretch tree of depth '; print: stretchDepth; tab;
		nextPutAll: ' check: '; print: check; nl.

	longLivedTree := ShootoutTreeNode bottomUpTree: 0 depth: maxDepth.
	minDepth to: maxDepth by: 2 do: [:depth|
		iterations := 1 bitShift: maxDepth - depth + minDepth.

		check := 0.
		1 to: iterations do: [:i|
			check := check + (ShootoutTreeNode bottomUpTree: i depth: depth) itemCheck.
			check := check + (ShootoutTreeNode bottomUpTree: -1*i depth: depth) itemCheck
			].
		output
			print:  (2*iterations); tab;
			nextPutAll: ' trees of depth '; print: depth; tab;
			nextPutAll: ' check: '; print: check; nl
		].

	output
		nextPutAll: 'long lived tree of depth '; print: maxDepth; tab;
		nextPutAll: ' check: '; print: longLivedTree itemCheck; nl! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 8/1/2008 16:26'!
chameneosredux2
	self chameneosredux: self arg to: self stdout.
	^''! !

!ShootoutTests class methodsFor: 'benchmarking' stamp: 'eem 8/1/2008 16:26'!
chameneosredux: arg to: aStream
   ShootoutMall runBenchMark: arg on: aStream! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 5/4/2010 11:46'!
collectReferenceTimes
	"Run the benchmarks 3 times and take their average, e.g. suitable
	 for filling in values for referenceTimesForClosureInterpreter"

	"ShootoutTests collectReferenceTimes"
	| n refs |
	Transcript clear.
	n := 3.
	refs := (1 to: n) collect: [:i| ShootoutTests runAllToInternalStream].
	^{	refs.
		(1 to: refs first size) collect:
			[:i|
			((refs inject: 0 into: [:sum :ref| (ref at: i) + sum]) / n) rounded] }! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: ''!
nbody
	self nbody: self arg to: self stdout! !

!ShootoutTests class methodsFor: 'benchmarking' stamp: 'eem 8/1/2008 16:09'!
nbody: count to: output
	| bodies |
	bodies := ShootoutNBodySystem new initialize.

	output print: bodies energy digits: 9; cr.
	count timesRepeat: [bodies after: 0.01].
	output print: bodies energy digits: 9; cr.
	^''! !

!ShootoutTests class methodsFor: 'profiling' stamp: 'eem 10/16/2010 19:33'!
profileAll
	"self profileAll"
	| stream |
	stream := DummyStream new.
	self nbody: 200000 "20000000" to: stream.
	self binarytrees: 15 to: stream.
	self chameneosredux: 260000 to: stream.
	self threadring: 10000000 to: stream! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 5/4/2010 11:44'!
referenceTimesForClosureInterpreter
	 "ClosureVM (QF 1.2.23 + Closure Bytecodes) on Eliot's 2010 vintage 2.66GHz Intel Core i7 MacBook Pro"
	^Dictionary new
		at: #nbody put: 4543;
		at: #binarytrees put: 6944;
		at: #chameneosredux put: 5799;
		at: #threadring put: 5623;
		yourself

	 "ClosureVM (QF 1.2.23 + Closure Bytecodes) on Eliot's 2006 vintage 2.16GHz Intel Core Duo MacBook Pro"
	"^Dictionary new
		at: #nbody put: 7660;
		at: #binarytrees put: 14417;
		at: #chameneosredux put: 8478;
		at: #threadring put: 8718;
		yourself"! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/24/2010 19:57'!
referenceTimesForSqueakVM
	 "Squeak VM (4.2.2beta1 + Closure Bytecodes) on Eliot's 2010 vintage 2.66GHz Intel Core i7 MacBook Pro"
	^Dictionary new
		at: #nbody put: 4917;
		at: #binarytrees put: 8593;
		at: #chameneosredux put: 5405;
		at: #threadring put: 3789;
		yourself! !

!ShootoutTests class methodsFor: 'reporting' stamp: 'eem 10/24/2008 16:44'!
report: name time: millisecs reference: reference on: aStream
	aStream
		cr;
		nextPutAll: name; cr;
		nextPutAll: ' took '; print: millisecs / 1000.0; nextPutAll: ' seconds'; cr; flush;
		nextPutAll: 'ratio: '; print: ((millisecs / reference) roundTo: 0.001);
		nextPutAll: '   % change: '; print: ((millisecs - reference * 100 / reference) roundTo: 0.01); nextPut: $%;
		cr; flush! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/24/2010 20:01'!
runAllToDummyStream
	"Transcript clear.
	 self runAllToDummyStream"
	^self runAllToDummyStreamVs: self referenceTimesForClosureInterpreter! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/24/2010 20:00'!
runAllToDummyStreamVs: referenceTimes
	"Transcript clear.
	 self runAllToDummyStreamVs: self referenceTimesForClosureInterpreter"
	"Transcript clear.
	 self runAllToDummyStreamVs: self referenceTimesForSqueakVM"
	| stream times ratios geometricMean |
	stream := DummyStream new.
	times := Array new writeStream.
	ratios := Array new writeStream.
	{ [self nbody: 200000 "20000000" to: stream].
	   [self binarytrees: 15 to: stream].
	   [self chameneosredux: 260000 to: stream].
	   [self threadring: 10000000 to: stream] } do:
		[:block | | benchmark reference t |
		benchmark := (self selectorForSimpleBlock: block) copyUpTo: $:.
		reference := referenceTimes at: benchmark asSymbol.
		Smalltalk garbageCollect.
		times nextPut: (t := Time millisecondsToRun: block).
		ratios nextPut: t asFloat / reference.
		self report: block decompile printString time: t reference: reference on: Transcript].
	geometricMean := (ratios contents inject: 1 into: [:m :n| m * n]) raisedTo: 1 / ratios position.
	Transcript
		nextPutAll: 'geometric mean '; print: (geometricMean roundTo: 0.001);
		nextPutAll: '   average speedup '; print: ((geometricMean - 1 * 100) roundTo: 0.01); nextPut: $%; cr; cr; flush.
	^times contents! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/24/2010 20:01'!
runAllToInternalStream
	"Transcript clear.
	 self runAllToInternalStream"
	^self runAllToInternalStreamVs: self referenceTimesForClosureInterpreter! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/24/2010 19:59'!
runAllToInternalStreamVs: referenceTimes
	"Transcript clear.
	 self runAllToInternalStreamVs: self referenceTimesForClosureInterpreter"
	"Transcript clear.
	 self runAllToInternalStreamVs: self referenceTimesForSqueakVM"
	| stream times ratios geometricMean |
	stream := (ByteString new: 10000) writeStream.
	times := Array new writeStream.
	ratios := Array new writeStream.
	{ [self nbody: 200000 "20000000" to: stream].
	   [self binarytrees: 15 to: stream].
	   [self chameneosredux: 260000 to: stream].
	   [self threadring: 10000000 to: stream] } do:
		[:block | | benchmark reference t |
		benchmark := (self selectorForSimpleBlock: block) copyUpTo: $:.
		reference := referenceTimes at: benchmark asSymbol.
		Smalltalk garbageCollect.
		times nextPut: (t := Time millisecondsToRun: block).
		ratios nextPut: t asFloat / reference.
		self report: block decompile printString time: t reference: reference on: Transcript].
	geometricMean := (ratios contents inject: 1 into: [:m :n| m * n]) raisedTo: 1 / ratios position.
	Transcript
		nextPutAll: 'geometric mean '; print: (geometricMean roundTo: 0.001);
		nextPutAll: '   average speedup '; print: ((geometricMean - 1 * 100) roundTo: 0.01); nextPut: $%; cr; cr; flush.
	^times contents! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/24/2010 20:01'!
runAllToTranscript
	"Transcript clear.
	 self runAllToTranscript"
	^self runAllToTranscriptVs: self referenceTimesForClosureInterpreter! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/24/2010 20:00'!
runAllToTranscriptVs: referenceTimes
	"Transcript clear.
	 self runAllToTranscriptVs: self referenceTimesForClosureInterpreter"
	"Transcript clear.
	 self runAllToTranscriptVs: self referenceTimesForSqueakVM"
	| times ratios geometricMean |
	times := Array new writeStream.
	ratios := Array new writeStream.
	{ [self nbody: 200000 "20000000" to: Transcript].
	   [self binarytrees: 15 to: Transcript].
	   [self chameneosredux: 260000 to: Transcript].
	   [self threadring: 10000000 to: Transcript] } do:
		[:block | | benchmark reference t |
		benchmark := (self selectorForSimpleBlock: block) copyUpTo: $:.
		reference := referenceTimes at: benchmark asSymbol.
		Smalltalk garbageCollect.
		times nextPut: (t := Time millisecondsToRun: block).
		ratios nextPut: t asFloat / reference.
		self report: block decompile printString time: t reference: reference on: Transcript].
	geometricMean := (ratios contents inject: 1 into: [:m :n| m * n]) raisedTo: 1 / ratios position.
	Transcript
		nextPutAll: 'geometric mean '; print: (geometricMean roundTo: 0.001);
		nextPutAll: '   average speedup '; print: ((geometricMean - 1 * 100) roundTo: 0.01); nextPut: $%; cr; cr; flush.
	^times contents! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 10/30/2008 10:43'!
selectorForSimpleBlock: aBlock
	| is |
	is := InstructionStream on: aBlock method.
	is pc: aBlock startpc.
	is scanFor:
		[:x| | selectorOrScanner |
		(selectorOrScanner := is selectorToSendOrSelf) ~~ is ifTrue:
			[^selectorOrScanner].
		false].
	^nil! !

!ShootoutTests class methodsFor: 'platform' stamp: 'eem 12/22/2010 14:59'!
stdin
   ^StandardFileStream stdIn! !

!ShootoutTests class methodsFor: 'platform' stamp: 'eem 12/22/2010 14:59'!
stdout
   ^StandardFileStream stdOut! !

!ShootoutTests class methodsFor: 'benchmarking' stamp: 'eem 6/9/2009 12:25'!
threadRing: aSemaphore output: output
   | first last |
   503 to: 1 by: -1 do: [:i|
      first := ShootoutThread named: i next: first done: aSemaphore output: output.
      last isNil ifTrue: [ last := first ].
   ].
   last nextThread: first.
   ^first ! !

!ShootoutTests class methodsFor: 'benchmark scripts' stamp: 'eem 6/9/2009 12:30'!
threadring
  self threadring: self arg to: self stdout.
   ^''! !

!ShootoutTests class methodsFor: 'benchmarking' stamp: 'eem 6/9/2009 12:29'!
threadring: arg to: output
   | done |
   (self threadRing: (done := Semaphore new) output: output) takeToken: arg.
   done wait! !

Object subclass: #ShootoutThread
	instanceVariableNames: 'name nextThread token semaphore done output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutThread class methodsFor: 'instance creation' stamp: 'eem 6/9/2009 12:28'!
named: anInteger next: aThread done: aSemaphore output: aStream
   ^self new name: anInteger; nextThread: aThread; done: aSemaphore; output: aStream; fork ! !

!ShootoutThread class methodsFor: 'instance creation' stamp: ''!
new
   ^self basicNew semaphore: Semaphore new ! !

!ShootoutThread methodsFor: 'accessing' stamp: ''!
done: aSemaphore
   done := aSemaphore ! !

!ShootoutThread methodsFor: 'accessing' stamp: ''!
fork
   [ self run ] fork ! !

!ShootoutThread methodsFor: 'accessing' stamp: ''!
name: anInteger
   name := anInteger ! !

!ShootoutThread methodsFor: 'accessing' stamp: ''!
nextThread: aThread
   nextThread := aThread ! !

!ShootoutThread methodsFor: 'accessing' stamp: 'eem 6/9/2009 12:26'!
output: anObject
	"Set the value of output"

	output := anObject! !

!ShootoutThread methodsFor: 'accessing' stamp: 'eem 6/9/2009 12:33'!
run
   [ self tokenNotDone ] whileTrue: [ nextThread takeToken: token - 1 ].
   output print: name.
   output name = 'stdout'
	ifTrue: [output nl]
	ifFalse: [output cr; flush].
   done signal ! !

!ShootoutThread methodsFor: 'accessing' stamp: ''!
semaphore: aSemaphore
   semaphore := aSemaphore ! !

!ShootoutThread methodsFor: 'accessing' stamp: ''!
takeToken: x
   token := x.
   semaphore signal ! !

!ShootoutThread methodsFor: 'accessing' stamp: ''!
tokenNotDone
   semaphore wait.
   ^token > 0 ! !

Object subclass: #ShootoutTreeNode
	instanceVariableNames: 'left right item'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Benchmarks'!

!ShootoutTreeNode class methodsFor: 'instance creation' stamp: ''!
bottomUpTree: anItem depth: anInteger
	^(anInteger > 0)
		ifTrue: [
			self
				left: (self bottomUpTree: 2*anItem - 1 depth: anInteger - 1)
				right: (self bottomUpTree: 2*anItem depth: anInteger - 1) 
				item: anItem
			]
		ifFalse: [self left: nil right: nil item: anItem]! !

!ShootoutTreeNode class methodsFor: 'instance creation' stamp: ''!
left: leftChild right: rightChild item: anItem
	^(super new) left: leftChild right: rightChild item: anItem! !

!ShootoutTreeNode methodsFor: 'accessing' stamp: ''!
itemCheck
	^left isNil
		ifTrue: [item] ifFalse: [item + (left itemCheck - right itemCheck)]! !

!ShootoutTreeNode methodsFor: 'initialize-release' stamp: ''!
left: leftChild right: rightChild item: anItem
	left := leftChild.
	right := rightChild.
	item := anItem! !

!DummyStream methodsFor: '*Cog-Benchmarks-platform' stamp: 'eem 10/21/2008 13:18'!
nl
	"do nothing"! !

!DummyStream methodsFor: '*Cog-Benchmarks-platform' stamp: 'eem 10/21/2008 13:20'!
print: number digits: decimalPlaces
	"do nothing"! !

!DummyStream methodsFor: '*Cog-Benchmarks-platform' stamp: 'eem 10/21/2008 13:20'!
print: number paddedTo: width
	"do nothing"! !

Error subclass: #ProcessorSimulationTrap
	instanceVariableNames: 'pc nextpc address type registerAccessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Processors'!

!ProcessorSimulationTrap commentStamp: 'lw 8/23/2012 19:25' prior: 0!
A ProcessorSimulationTrap is an Error raised by CogProcessorAliens which allow the VMSimulator to fix the problem and resume execution.
I know where the problem occured (pc) and which instruction is next (nextpc), which field of the processor need be read/written and what type of memory access was the source of error.

I am created in #handleExecutionPrimitiveFailureIn:minimumAddress:readOnlyBelow: and associated methods. 
In the IA32 case, the type is managed by the OpcodeExecutionMap, using the first byte of the last instruction as index. 
In the ARM case, we need rely on a case statement, since no byte (sequence) is able to directly tell which type I am of.!

!ProcessorSimulationTrap class methodsFor: 'instance creation' stamp: 'eem 5/7/2010 11:08'!
pc: pc nextpc: nextpc address: address type: type
	^self new
		pc: pc;
		nextpc: nextpc;
		address: address;
		type: type! !

!ProcessorSimulationTrap class methodsFor: 'instance creation' stamp: 'eem 5/7/2010 11:08'!
pc: pc nextpc: nextpc address: address type: type accessor: regAccessorMessage
	^self new
		pc: pc;
		nextpc: nextpc;
		address: address;
		type: type;
		registerAccessor: regAccessorMessage! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/18/2008 15:26'!
address
	"Answer the value of address"

	^ address! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/18/2008 15:26'!
address: anObject
	"Set the value of address"

	address := anObject! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/28/2008 17:56'!
nextpc
	"Answer the value of nextpc"

	^ nextpc! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/28/2008 17:56'!
nextpc: anObject
	"Set the value of nextpc"

	nextpc := anObject! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/18/2008 15:58'!
pc
	"Answer the value of pc"

	^ pc! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/18/2008 15:58'!
pc: anObject
	"Set the value of pc"

	pc := anObject! !

!ProcessorSimulationTrap methodsFor: 'printing' stamp: 'eem 5/23/2010 15:37'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' (pc: '; print: pc.
	aStream nextPutAll: ' nextpc: '; print: nextpc.
	aStream nextPutAll: ' address: '; print: address.
	aStream nextPutAll: ' type: '; print: type.
	aStream nextPutAll: ' accessor: '; print: registerAccessor; nextPut: $)! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 1/1/2009 22:33'!
registerAccessor
	"Answer the value of registerAccessor"

	^ registerAccessor! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 1/1/2009 22:33'!
registerAccessor: anObject
	"Set the value of registerAccessor"

	registerAccessor := anObject! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/18/2008 15:32'!
type
	"Answer the value of type"

	^ type! !

!ProcessorSimulationTrap methodsFor: 'accessing' stamp: 'eem 12/18/2008 15:32'!
type: anObject
	"Set the value of type"

	type := anObject! !

InstructionClient subclass: #ETC
	instanceVariableNames: 'scanner blockEnd joinOffsets sends contextStack currentContext blockEndStack topIsVector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Scripts'!

!ETC commentStamp: '<historical>' prior: 0!
I am a quick-hack type-checker (Eliot's Type Checker) that answers a dictionary of receiver to selector sets for a given method.!

!ETC class methodsFor: 'utilities' stamp: 'eem 10/12/2009 09:22'!
sendsForMethod: aMethod
	^(self new setMethod: aMethod) sends! !

!ETC class methodsFor: 'utilities' stamp: 'eem 8/16/2010 16:07'!
sendsToInstVar: instVarName in: class
	"Answer all selectors sent to instVarName in class's methods"
	| idx ref selectors |
	idx := class instVarIndexFor: instVarName ifAbsent: [^Set new].
	ref := {#inst. idx}.
	selectors := Set new.
	class methodsDo:
		[:m| selectors addAll: ((self sendsForMethod: m) at: ref ifAbsent: [#()])].
	^selectors! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:32'!
blockReturnTop
	"Return Top Of Stack bytecode."
	currentContext pop.
	scanner pc < blockEnd
		ifTrue: [self doJoin]
		ifFalse: [currentContext := contextStack removeLast.
				blockEnd := blockEndStack removeLast]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 13:51'!
doDup
	currentContext doDup! !

!ETC methodsFor: 'private' stamp: 'eem 9/11/2009 14:23'!
doJoin
	scanner pc < blockEnd ifTrue:
		[currentContext instVarNamed: 'stackp' put: (joinOffsets at: scanner pc)]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 13:51'!
doPop
	currentContext doPop! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 13:58'!
jump: offset
	"Unconditional Jump bytecode."
	offset > 0 ifTrue:
		[joinOffsets at: scanner pc + offset put: currentContext stackPtr.
		 self doJoin]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:32'!
jump: offset if: condition 
	"Conditional Jump bytecode."
	currentContext pop.
	offset > 0 ifTrue:
		[joinOffsets at: scanner pc + offset put: currentContext stackPtr]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:54'!
methodReturnConstant: value
	currentContext isExecutingBlock
		ifTrue: [currentContext push: value.
				self blockReturnTop]
		ifFalse: [self doJoin]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:56'!
methodReturnReceiver
	currentContext isExecutingBlock
		ifTrue: [self pushReceiver.
				self blockReturnTop]
		ifFalse: [self doJoin]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:57'!
methodReturnTop
	currentContext isExecutingBlock
		ifTrue: [self blockReturnTop]
		ifFalse: [currentContext pop.
				self doJoin]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:32'!
popIntoLiteralVariable: anAssociation
	currentContext pop! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:32'!
popIntoReceiverVariable: offset
	currentContext pop! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:33'!
popIntoRemoteTemp: remoteTempIndex inVectorAt: tempVectorIndex
	currentContext pop! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:33'!
popIntoTemporaryVariable: offset
	| vector |
	topIsVector
		ifTrue:
			[vector := currentContext top.
			 1 to: vector size do:
				[:i|
				vector
					at: i
					put: 'remote ', i printString,
						' @ ', (offset + 1) printString,
						' in ', currentContext startpc printString].
			 currentContext popIntoTemporaryVariable: offset]
		ifFalse:
			[currentContext pop]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 14:00'!
pushActiveContext
	currentContext pushActiveContext! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:34'!
pushClosureCopyNumCopiedValues: numCopied numArgs: numArgs blockSize: blockSize
	"Push Closure bytecode.  Either compute the end of the block if this is
	 the block we're analysing, or skip it, adjusting the stack as appropriate."
	| blockClosure j |
	blockEndStack addLast: blockEnd.
	blockEnd := scanner pc + blockSize.
	contextStack addLast: currentContext.
	currentContext
		pc: scanner pc;
		pushClosureCopyNumCopiedValues: numCopied numArgs: numArgs blockSize: blockSize.
	blockClosure := currentContext top.
	currentContext := blockClosure asContext.
	1 to: numArgs do:
		[:i| currentContext push: 'blkarg', i printString].
	1 to: numCopied do:
		[:i| currentContext push: (blockClosure at: i)].
	j := numArgs + numCopied.
	[scanner nextByte = ParseNode pushNilCode] whileTrue:
		[scanner interpretNextInstructionFor: self.
		 currentContext at: currentContext stackPtr put: 'blktmp', (j := j + 1) printString]

	"ETC sendsForMethod: (ETC compiledMethodAt: #pushClosureCopyNumCopiedValues:numArgs:blockSize:)"! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 15:38'!
pushConsArrayWithElements: numElements
	currentContext pushConsArrayWithElements: numElements! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 14:07'!
pushConstant: value
	currentContext pushConstant: value! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 14:08'!
pushLiteralVariable: anAssociation
	currentContext push: anAssociation! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 15:38'!
pushNewArrayOfSize: numElements
	currentContext pushNewArrayOfSize: numElements.
	scanner willStorePop ifTrue:
		[topIsVector := true.
		 scanner interpretNextInstructionFor: self.
		 topIsVector := false]! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 14:08'!
pushReceiver
	currentContext push: #self! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:29'!
pushReceiverVariable: offset
	currentContext push: {#inst. offset + 1}! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 14:09'!
pushRemoteTemp: remoteTempIndex inVectorAt: tempVectorIndex
	currentContext pushRemoteTemp: remoteTempIndex inVectorAt: tempVectorIndex! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 9/11/2009 14:10'!
pushTemporaryVariable: offset
	currentContext pushTemporaryVariable: offset! !

!ETC methodsFor: 'instruction decoding' stamp: 'eem 10/12/2009 09:36'!
send: selector super: supered numArgs: numArgs
	numArgs timesRepeat: [currentContext pop].
	currentContext pop ifNotNil:
		[:top|
		((top isArray and: [top size = 2 and: [top first == #inst]])
		 or: [top == #self]) ifTrue:
			[(sends at: top ifAbsentPut: [Set new]) add: selector]].
	currentContext push: 'send ', selector! !

!ETC methodsFor: 'results' stamp: 'eem 10/12/2009 09:22'!
sends
	[scanner atEnd] whileFalse:
		[scanner interpretNextInstructionFor: self].
	^sends! !

!ETC methodsFor: 'initialize-release' stamp: 'eem 10/12/2009 09:35'!
setMethod: aCompiledMethod
	scanner := InstructionStream new method: aCompiledMethod pc: aCompiledMethod initialPC.
	contextStack := OrderedCollection new.
	currentContext := MethodContext
						sender: nil
						receiver: nil
						method: aCompiledMethod
						arguments: ((1 to: aCompiledMethod numArgs) collect:
										[:i| 'arg', i printString]).
	aCompiledMethod numArgs + 1 to: aCompiledMethod numTemps do:
		[:i| currentContext at: i put: 'tmp', (i - aCompiledMethod numArgs) printString].
	sends := Dictionary new.
	joinOffsets := Dictionary new.
	blockEndStack := OrderedCollection new.
	blockEnd := aCompiledMethod endPC.
	topIsVector := false! !

!Stream methodsFor: '*Cog-Benchmarks-platform' stamp: ''!
nl
   self nextPut: Character lf! !

!Stream methodsFor: '*Cog-Benchmarks-platform' stamp: ''!
print: number digits: decimalPlaces
   | precision rounded |
   decimalPlaces <= 0 ifTrue: [^ number rounded printString].
   precision := Utilities floatPrecisionForDecimalPlaces: decimalPlaces.
   rounded := number roundTo: precision.
   self nextPutAll: 
      ((rounded asScaledDecimal: decimalPlaces) printString copyUpTo: $s)! !

!Stream methodsFor: '*Cog-Benchmarks-platform' stamp: ''!
print: number paddedTo: width
   self nextPutAll: (number printStringLength: width padded: false)! !

TestCase subclass: #BochsIA32AlienTests
	instanceVariableNames: 'processor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Processors-Tests'!

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 5/7/2010 10:43'!
callTrapPerformance: n
	"Call a function that is out-of-range.  Ensure the call is trapped."
	"self new testCallTrap"
	| memory |
	 "The address is out of range of memory every which way (whether relative or absolute and whether big-endian or little."
	memory := ByteArray new: 1024.
	memory replaceFrom: 1 to: 5 with: { self processor callOpcode. 0. 16r80. 16r80. 0. } asByteArray.
	self processor
			eip: 0;
			esp: (memory size - 4). "Room for return address"
	1 to: n do:
		[:ign|
		[self processor singleStepIn: memory]
			on: ProcessorSimulationTrap
			do: [:ex|]].

	"QSystemProfiler spyOn: [BochsIA32AlienTests new callTrapPerformance: 1024*128]"
	"Time millisecondsToRun: [BochsIA32AlienTests new callTrapPerformance: 1024*128] 2463"
	"Time millisecondsToRun: [1 to: 1024*1024*64 do: [:ign| nil yourself]] 636"
	"Time millisecondsToRun: [1 to: 1024*1024*64 do: [:ign| nil perform: #ifNotNilDo: with: nil]] 3639"
	"Time millisecondsToRun: [1 to: 1024*1024*64 do: [:ign| nil perform: #ifNotNilDo:ifNil: with: nil with: nil]] 12401"! !

!BochsIA32AlienTests methodsFor: 'accessing' stamp: 'eem 12/12/2008 16:23'!
nfib
	"long fib(long n) { return n <= 1 ? 1 : fib(n-1) + fib(n-2) + 1; }
	 as compiled by Microsoft Visual C++ V6 (12.00.8804) cl /O2 /Fc"
	"| bat nfib ip |
	bat := BochsIA32AlienTests new.
	nfib := bat nfib asByteArray.
	ip := 0.
	20 timesRepeat:
		[bat processor disassembleInstructionAt: ip In: nfib into:
			[:da :len|
			Transcript nextPutAll: da; cr; flush.
			ip := ip + len]]"
	^#("00000" 16r56							"push    esi"
		"00001" 16r8B 16r74 16r24 16r08		"mov     esi, DWORD PTR _n$[esp]"
		"00005" 16r83 16rFE 16r01				"cmp     esi, 1"
		"00008" 16r7F 16r07						"jg      SHORT $L528"
		"0000a" 16rB8 16r01 16r00 16r00 16r00	"mov     eax, 1"
		"0000f" 16r5E							"pop     esi"
		"00010" 16rC3							"ret     0"
"
$L528:"
		"00011" 16r8D 16r46 16rFE				"lea     eax, DWORD PTR [esi-2]"
		"00014" 16r57							"push    edi"
		"00015" 16r50							"push    eax"
		"00016" 16rE8 16rE5 16rFF 16rFF 16rFF   "call    _fib"
		"0001b" 16r4E							"dec     esi"
		"0001c" 16r8B 16rF8						"mov     edi, eax"
		"0001e" 16r56							"push    esi"
		"0001f" 16rE8 16rDC 16rFF 16rFF 16rFF   "call    _fib"
		"00024" 16r83 16rC4 16r08				"add     esp, 8"
		"00027" 16r8D 16r44 16r07 16r01		"lea     eax, DWORD PTR [edi+eax+1]"
		"0002b" 16r5F							"pop     edi"
		"0002c" 16r5E							"pop     esi"
		"0002d" 16rC3							"ret     0")! !

!BochsIA32AlienTests methodsFor: 'accessing' stamp: 'eem 12/11/2008 13:53'!
processor
	processor ifNil:
		[processor := BochsIA32Alien new].
	^processor! !

!BochsIA32AlienTests methodsFor: 'accessing' stamp: 'eem 1/20/2009 12:57'!
registerGetters
	^#(eax ecx edx ebx esp ebp esi edi eip)! !

!BochsIA32AlienTests methodsFor: 'accessing' stamp: 'eem 1/20/2009 12:57'!
registerSetters
	^#(eax: ecx: edx: ebx: esp: ebp: esi: edi: eip:)! !

!BochsIA32AlienTests methodsFor: 'execution' stamp: 'eem 10/21/2010 11:02'!
runNFib: n disassemble: disassemble printRegisters: printRegisters
	"Run nfib wth the argument. Answer the result."
	"self new runNFib: 5 disassemble: true printRegisters: true"
	| memory |
	memory := ByteArray new: 4096 * 2 withAll: self processor nopOpcode.
	memory
		replaceFrom: 1 to: self nfib size with: self nfib asByteArray startingAt: 1;
		longAt: memory size - 3 put: n bigEndian: false; "argument n"
		longAt: memory size - 7 put: self nfib size bigEndian: false.  "return address"
	self processor
		eip: 0;
		esp: (memory size - 8). "Room for return address and argument n"
	printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	"run until something goes wrong."
	self processor runInMemory: memory readExecuteOnlyBelow: memory size / 2.
	printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	^self processor eax! !

!BochsIA32AlienTests methodsFor: 'execution' stamp: 'eem 10/21/2010 11:06'!
singleStepNFib: n disassemble: disassemble printRegisters: printRegisters
	"Run nfib wth the argument. Answer the result."
	"self new runNFib: 5 disassemble: true printRegisters: true"
	| memory finalSP |
	memory := ByteArray new: 4096 * 2 withAll: self processor nopOpcode.
	finalSP := memory size - 4. "Stop when we return to the nop following nfib"
	memory
		replaceFrom: 1 to: self nfib size with: self nfib asByteArray startingAt: 1;
		longAt: memory size - 3 put: n bigEndian: false; "argument n"
		longAt: memory size - 7 put: self nfib size bigEndian: false.  "return address"
	self processor
			eip: 0;
			esp: (memory size - 8). "Room for return address and argument n"
	printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	[disassemble ifTrue:
		[Transcript nextPutAll: (self processor disassembleNextInstructionIn: memory); cr; flush].
	 self processor singleStepIn: memory readExecuteOnlyBelow: memory size / 2.
	 printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	 self processor esp ~= finalSP] whileTrue.
	^self processor eax! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 4/15/2009 17:13'!
testCPUHasSSE2
	"Use the CPUID instruction to check if SSE2 is supported.  Cog
	 uses SSE2 instructions for machine-code floating-point primitives."
	self processor eax: 0. "get vendor identfication string"
	self processor eip: 0; singleStepIn: (ByteArray with: 16r0F with: 16rA2 with: 16r90) "cpuid;nop".
	self assert: self processor eip = 2.
	self assert: self processor eax >= 1.
	self processor eax: 1.
	self processor eip: 0; singleStepIn: (ByteArray with: 16r0F with: 16rA2 with: 16r90) "cpuid;nop".
	self assert: self processor eip = 2.
	self assert: (self processor edx bitAnd: 1 << 26) ~= 0

	"self new testCPUHasSSE2"! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 12/11/2008 18:09'!
testCPUID
	| vendorString |
	self processor
			eip: 0;
			eax: 0. "get vendor identfication string"
	self processor singleStepIn: (ByteArray with: 16r0F with: 16rA2 with: 16r90) "cpuid;nop".
	self assert: self processor eip = 2.
	self assert: self processor eax ~= 0.
	vendorString := (ByteArray new: 12)
						longAt: 1 put: self processor ebx bigEndian: false;
						longAt: 5 put: self processor edx bigEndian: false;
						longAt: 9 put: self processor ecx bigEndian: false;
						asString.
	self assert: (vendorString = 'GenuineIntel'
				or: [vendorString = 'AuthenticAMD'])! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 5/7/2010 11:05'!
testCallTrap
	"Call a function that is out-of-range.  Ensure the call is trapped."
	"self new testCallTrap"
	| memory |
	 "The address is out of range of memory every which way (whether relative or absolute and whether big-endian or little."
	memory := ByteArray new: 1024.
	memory replaceFrom: 1 to: 5 with: { self processor callOpcode. 0. 16r80. 16r80. 0. } asByteArray.
	self processor
			eip: 0;
			esp: (memory size - 4). "Room for return address"
	self should: [self processor singleStepIn: memory]
		raise: ProcessorSimulationTrap
		withExceptionDo:
			[:pst|
			self assert: pst address = ((memory longAt: 2 bigEndian: false) + 5 "length of call instr").
			self assert: pst pc = 0.
			self assert: pst type = #call].

	"| memory |
	memory := ByteArray new: 1024.
	memory replaceFrom: 1 to: 5 with: { BochsIA32Alien new callOpcode. 0. 16r80. 16r80. 0. } asByteArray.
	BochsIA32AlienTests new processor
		eip: 0;
		esp: (memory size - 4);
		singleStepIn: memory;
		printRegistersOn: Transcript.
	Transcript flush"! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 10/20/2010 15:58'!
testExecutionTrap
	"Execute a run of nops.  test executing beyond the executable limit is trapped."
	"self new testExecutionTrap"
	| memory |
	 "The address is out of range of memory every which way (whether relative or absolute and whether big-endian or little."
	memory := ByteArray new: 4096 * 2 withAll: self processor nopOpcode.
	self processor
			eip: 0;
			esp: (memory size - 4). "Room for return address"
	self should: [self processor runInMemory: memory minimumAddress: 0 readOnlyBelow: memory size / 2]
		raise: Error
		withExceptionDo:
			[:err|
			self assert: self processor pc = (memory size / 2).
			self assert: ('*EIP*> CS.limit*' match: err messageText)].
	self processor eip: 0.
	self should: [[self processor singleStepIn: memory minimumAddress: 0 readOnlyBelow: memory size / 2] repeat]
		raise: Error
		withExceptionDo:
			[:err|
			self assert: self processor pc = (memory size / 2).
			self assert: ('*EIP*> CS.limit*' match: err messageText)]! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 4/16/2009 17:57'!
testMOVSD
	"Test MOVSD indirecting through edx."
	"self new testMOVSD"
	self processor
		edx: 0;
		eip: 0;
		singleStepIn: {16rF2. 16r0F. 16r10. 16r42. 16r04. 16r90. 16r01. 16r02. 16r03. 16r04. 16r05. 16r06} asByteArray "movsd %ds:0x4(%edx), %xmm0;nop;garbage".
	self assert: self processor eip = 5.
	self assert: self processor xmm0low = 16r0605040302019004! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/14/2009 15:50'!
testNfib1
	"self new testNfib1"
	self should: [self runNFib: 1 disassemble: false printRegisters: false]
		raise: Error.
	self deny: (self processor eip between: 1 and: self nfib size).
	self assert: self processor eax = 1 benchFib! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/14/2009 15:51'!
testNfib16
	"self new testNfib16"
	self should: [self runNFib: 16 disassemble: false printRegisters: false]
		raise: Error.
	self deny: (self processor eip between: 1 and: self nfib size).
	self assert: self processor eax = 16 benchFib! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/14/2009 15:50'!
testNfib2
	"self new testNfib2"
	self should: [self runNFib: 2 disassemble: false printRegisters: false]
		raise: Error.
	self deny: (self processor eip between: 1 and: self nfib size).
	self assert: self processor eax = 2 benchFib! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/14/2009 15:49'!
testNfib4
	"self new testNfib4"
	self should: [self runNFib: 4 disassemble: false printRegisters: false]
		raise: Error.
	self deny: (self processor eip between: 1 and: self nfib size).
	self assert: self processor eax = 4 benchFib! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/20/2009 12:58'!
testResetCPU
	"self new testResetCPU"
	self registerSetters do:
		[:setter|
		self processor perform: setter with: 16r55555555].
	self registerGetters do:
		[:getter|
		self assert: 16r55555555 = (self processor perform: getter)].
	self processor reset.
	self registerGetters do:
		[:getter|
		self assert: 0 = (self processor perform: getter)]! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/14/2009 15:10'!
testStepNfib1
	"self new testNfib1"
	self singleStepNFib: 1 disassemble: false printRegisters: false.
	self assert: self processor eip = self nfib size.
	self assert: self processor eax = 1 benchFib! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/14/2009 15:10'!
testStepNfib2
	"self new testNfib2"
	self singleStepNFib: 2 disassemble: false printRegisters: false.
	self assert: self processor eip = self nfib size.
	self assert: self processor eax = 2 benchFib! !

!BochsIA32AlienTests methodsFor: 'tests' stamp: 'eem 1/14/2009 15:11'!
testStepNfib4
	"self new testNfib4"
	self singleStepNFib: 4 disassemble: false printRegisters: false.
	self assert: self processor eip = self nfib size.
	self assert: self processor eax = 4 benchFib! !

TestCase subclass: #CogVMTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Tests'!

!CogVMTests methodsFor: 'support' stamp: 'eem 6/17/2009 23:18'!
recurse
	self recurse! !

!CogVMTests methodsFor: 'support' stamp: 'eem 6/17/2009 08:45'!
runningOnCogit
	"CogVMTests new runningOnCogit"
	^(1 to: 3) anySatisfy: [:each| thisContext xrayIsExecutingMachineCode]! !

!CogVMTests methodsFor: 'tests' stamp: 'eem 9/14/2009 06:18'!
testMultiplicativeOperators
	"CogVMTests new setUp testMultiplicativeOperators"
	self runningOnCogit ifTrue:
		[thisContext xrayIsExecutingMachineCode ifFalse:
			[^self testMultiplicativeOperators]].
	1 to: 3 do: "Running 3 times should ensure all operators are compiled to machine code"
		[:ignored| | values |
		self assert: (1 / 2 literalEqual: (Fraction numerator: 1 denominator: 2)).
		self assert: 1 // 2 == 0.
		self assert: (1 quo: 2) == 0.
		self assert: 3 * 4 == 12.

		self assert: 65536 / 2 == 32768.
		self assert: 65536 // 2 == 32768.
		self assert: (65536 quo: 2) == 32768.
		self assert: 65536 * 2 == 131072.
		#(1 2 4 8 16 32 64 128 256 512 1024 2048 4096 8192 16384 32768 65536
		   131072 262144 524288 1048576 2097152 4194304 8388608 16777216 33554432
		   67108864 134217728 268435456 536870912 1073741824 2147483648 4294967296) withIndexDo:
			[:value :index| | power | "test that integer overflow is detected in multiplication"
			power := index - 1.
			self assert: value * value = (2 raisedTo: power + power)].

		self assert: SmallInteger minVal / -1 = (SmallInteger maxVal + 1).
		self assert: SmallInteger minVal // -1 = (SmallInteger maxVal + 1).
		self assert: (SmallInteger minVal quo: -1) = (SmallInteger maxVal + 1).
		self assert: (SmallInteger minVal \\ -1) = 0.

		values := Integer primesUpTo: 1000.
		values := (values reverse collect: [:ea| ea negated]), values.
		values do:
			[:dividend|
			values do:
				[:divisor|
				self assert: (dividend quo: divisor) * divisor + (dividend rem: divisor) = dividend.
				self assert: (dividend // divisor) * divisor + (dividend \\ divisor) = dividend]]]! !

!CogVMTests methodsFor: 'tests' stamp: 'eem 6/17/2009 23:18'!
testPCAccessForActivation
	"self new testPCAccessForActivation"
	| runningOnCogit theProcess block run pc executingMachineCode scanner blockCount |
	runningOnCogit := self runningOnCogit.
	run := true.
	theProcess := Processor activeProcess.
	"Fork a process that will sample the pc of the following recursive block."
	[(Delay forMilliseconds: 1) wait.
	 pc := theProcess suspendedContext pc.
	 executingMachineCode := theProcess suspendedContext xrayIsExecutingMachineCode.
	 run := false] forkAt: Processor userInterruptPriority.
	"The only suspension point in this block is on activation (assuming
	 ifTrue: is inlined) so its pc must be the first bytecode in the block."
	block := [run ifTrue: [block value]].
	block value.
	"Find the first bytecode of the second block in this method."
	blockCount := 0.
	(scanner := InstructionStream on: thisContext method)
		scanFor:
			[:b| blockCount = 2 or: [b == 143 and: [blockCount := blockCount + 1. false]]].
	self assert: pc = scanner pc.
	self assert: executingMachineCode = runningOnCogit.
	"Fork a process that will use recurse: to infinitely recurse.
	 The only suspension point in this call is on activation (assuming
	 ifTrue: is inlined) so its pc must be the first bytecode in the method."
	theProcess := [self recurse] forkAt: Processor activePriority - 1.
	(Delay forMilliseconds: 1) wait.
	pc := theProcess suspendedContext pc.
	executingMachineCode := theProcess suspendedContext xrayIsExecutingMachineCode.
	theProcess terminate.
	self assert: pc = (self class compiledMethodAt: #recurse) initialPC.
	self assert: executingMachineCode = runningOnCogit! !

!CogVMTests methodsFor: 'tests' stamp: 'eem 6/17/2009 22:31'!
testPCAccessForSends
	"self new testPCAccessForSends"
	| runningOnCogit |
	runningOnCogit := self runningOnCogit.
	#("Method-level pc maping"
		( '{ thisContext pc.
			thisContext xrayIsExecutingMachineCode.
			thisContext method }'
			pc)
		( '{ thisContext perform: #perform:withArguments: with: #pc with: Array new.
			thisContext xrayIsExecutingMachineCode.
			thisContext method }'
			perform:with:with: )
		( '{ [:ctxt :msg :ign1| ctxt perform: msg] value: thisContext value: #pc value: nil.
			thisContext xrayIsExecutingMachineCode.
			thisContext method }'
			value:value:value: )
	   "Block-level pc maping"
		( '[:method|
			{ thisContext pc.
			  thisContext xrayIsExecutingMachineCode.
			  method } ] value: thisContext method'
			pc)
		( '[:method|
			{ (thisContext perform: #perform:withArguments: with: #pc with: Array new).
			  thisContext xrayIsExecutingMachineCode.
			  method } ] value: thisContext method'
			perform:with:with:)
		( '[:method|
			{ [:ctxt :msg :ign1| ctxt perform: msg] value: thisContext value: #pc value: nil.
			  thisContext xrayIsExecutingMachineCode.
			  method } ] value: thisContext method'
			value:value:value: ) ) do:
		[:case|
		 [:actualPc :executingMachineCode :method| | flag scanner expectedPc |
		  flag := false.
		  (scanner := InstructionStream on: method)
			scanFor:
				[:b| flag or: [flag := scanner selectorToSendOrSelf == case second. false]].
		  expectedPc := scanner pc.
		  self assert: actualPc = expectedPc.
		  self assert: executingMachineCode = runningOnCogit]
			valueWithArguments: (Compiler evaluate: case first logged: false)]! !

TestCase subclass: #GdbARMAlienTests
	instanceVariableNames: 'processor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Cog-Processors-Tests'!

!GdbARMAlienTests methodsFor: 'accessing' stamp: 'lw 7/11/2012 14:16'!
nfib
	"long fib(long n) { return n <= 1 ? 1 : fib(n-1) + fib(n-2) + 1; }
	 as compiled by arm-elf-gnuabi-gcc fib.c -c -marm
	also, the jumps are changed by hand."
	"| bat nfib ip |
	bat := GdbARMAlienTests new.
	nfib := bat nfib asWordArray.
	ip := 0.
	23 timesRepeat:
		[bat processor disassembleInstructionAt: ip In: nfib into:
			[:da :len|
			Transcript nextPutAll: da; cr; flush.
			ip := ip + len]]"
	^#("00000000 <fib>:
	   0:"		16re92d4810 		"push	{r4, fp, lr}	fp = r11, sp is changed in this command
	   4:"		16re28db008 		"add	fp, sp, #8	now, the frame pointer is changed
	   8:"		16re24dd00c 		"sub	sp, sp, #12
	   c:"		16re50b0010 		"str	r0, [fp, #-16]
	  10:"		16re51b3010 		"ldr	r3, [fp, #-16]	r3 <- [fp-16] <- r0
	  14:"		16re3530001 		"cmp	r3, #1
	  18:"		16rda00000c 		"ble	50 <fib+0x50>
	  1c:"		16re51b3010 		"ldr	r3, [fp, #-16]
	  20:"		16re2433001 		"sub	r3, r3, #1
	  24:"		16re1a00003 		"mov	r0, r3
	  28:"		16rebfffff4 		"bl	0 <fib>
	  2c:"		16re1a04000 		"mov	r4, r0
	  30:"		16re51b3010 		"ldr	r3, [fp, #-16]
	  34:"		16re2433002	 	"sub	r3, r3, #2
	  38:"		16re1a00003 		"mov	r0, r3
	  3c:"		16rebffffef 		"bl	0 <fib>
	  40:"		16re1a03000 		"mov	r3, r0
	  44:"		16re0843003 		"add	r3, r4, r3
	  48:"		16re2833001 		"add	r3, r3, #1
	  4c:"		16rea000000 		"b	54 <fib+0x54>
	  50:"		16re3a03001 		"mov	r3, #1
	  54:"		16re1a00003 		"mov	r0, r3
	  58:"		16re24bd008 		"sub	sp, fp, #8
	  5c:"		16re8bd8810 		"pop	{r4, fp, pc}")! !

!GdbARMAlienTests methodsFor: 'accessing' stamp: 'eem 12/11/2008 13:53'!
processor
	processor ifNil:
		[processor := GdbARMAlien new].
	^processor! !

!GdbARMAlienTests methodsFor: 'accessing' stamp: 'lw 7/2/2012 11:54'!
registerGetters
	^#(r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 r10 r11 r12 lr sp pc)! !

!GdbARMAlienTests methodsFor: 'accessing' stamp: 'lw 7/2/2012 11:55'!
registerSetters
	^#(r0: r1: r2: r3: r4: r5: r6: r7: r8: r9: r10: r11: r12: lr: sp: pc:)! !

!GdbARMAlienTests methodsFor: 'execution' stamp: 'lw 8/9/2012 20:57'!
runNFib: n disassemble: disassemble printRegisters: printRegisters
	"Run nfib wth the argument. Answer the result."
	"self new runNFib: 5 disassemble: true printRegisters: true"
	| memory |
	memory := Bitmap new: 1024 * 2 withAll: self processor nopOpcode.
	memory replaceFrom: 1 to: self nfib size with: self nfib asWordArray startingAt: 1.
	self processor
		r0: n;"argument n"
		lr: memory size * 2; "return address"
		pc: 0;
		sp: (memory size * 4) - 16.
	printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	"run until something goes wrong."
	self processor runInMemory: memory readExecuteOnlyBelow: memory size / 2.
	printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	^self processor r0! !

!GdbARMAlienTests methodsFor: 'execution' stamp: 'lw 7/11/2012 13:45'!
singleStepNFib: n disassemble: disassemble printRegisters: printRegisters
	"Run nfib wth the argument. Answer the result."
	"self new runNFib: 5 disassemble: true printRegisters: true"
	| memory |
	memory := WordArray new: 1024 * 2 withAll: self processor nopOpcode.
	memory replaceFrom: 1 to: self nfib size with: self nfib asWordArray startingAt: 1.
	self processor
		r0: n; "argument n"
		lr: self nfib size * 4;  "return address"
		pc: 0;
		sp: (memory size * 4 - 12). "Room for return address, frame pointer and r4"
	printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	[disassemble ifTrue:
		[Transcript nextPutAll: (self processor disassembleNextInstructionIn: memory); cr; flush].
	 self processor singleStepIn: memory readExecuteOnlyBelow: memory size * 4 / 2.
	 printRegisters ifTrue:
		[self processor printRegistersOn: Transcript.
		 Transcript cr; flush].
	"stop, once we leave the nfib code and step through the nops after that."
	 self processor pc < (self nfib size * 4)] whileTrue.
	^self processor r0! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 8/9/2012 21:01'!
testCallTrap
	"Call a function that is out-of-range.  Ensure the call is trapped."
	"self new testCallTrap"
	| memory |
	memory := Bitmap new: 256 withAll: self processor nopOpcode.
	memory longAt: 1 put: (self processor branchAndLinkOpcodeWithOffset: 1024) bigEndian: false.
	memory := memory asByteArray.
	self processor
			pc: 0;
			sp: (memory size - 4); "Room for return address"
			singleStepIn: memory.
			"We have to step twice, because the first step only changes the pc, but does not fetch anything from the address it points to."
	self should: [self processor singleStepIn: memory]
		raise: Error
		withExceptionDo:
			[:pst|
			self assert: self processor pc = 1024.
			self assert: self processor lr = 4.
			self assert: pst pc = 0.
			self assert: pst nextpc = 4.
			self assert: pst address = 1024.
			self assert: pst type = #call].! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/10/2012 12:12'!
testDisassembling

	| memory result |
	memory := WordArray new: 2.
	memory at: 1 put: 16rEF200000.
	result := self processor
		disassembleInstructionAt: 0 
		In: memory 
		into: [:str :len | 
			self 
				assert: len = 4;
				assert: str = 'svc	0x00200000'].! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 8/18/2012 14:25'!
testExecutionTrap
	"Execute a run of nops.  test executing beyond the executable limit is trapped."
	"self new testExecutionTrap"
	| memory |
	 "The address is out of range of memory every which way (whether relative or absolute and whether big-endian or little."
	memory := (Bitmap new: 1024 * 2 withAll: self processor nopOpcode) asByteArray.
	self processor
			pc: 0;
			sp: (memory size - 4). "Room for return address"
	self should: [self processor runInMemory: memory minimumAddress: 0 readOnlyBelow: memory size / 2]
		raise: Error
		withExceptionDo:
			[:err|
			self assert: self processor pc = (memory size / 2).
			self assert: ('Error 0: Illegal Instruction fetch address (0x1000).' match: err messageText)].
	self processor pc: 0.
	self should: [[self processor singleStepIn: memory minimumAddress: 0 readOnlyBelow: memory size / 2] repeat]
		raise: Error
		withExceptionDo:
			[:err|
			self assert: self processor pc = (memory size / 2).
			self assert: ('Error 0: Illegal Instruction fetch address (0x1000).' match: err messageText)]! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/11/2012 13:04'!
testFlags
	"self new testFlags"
	| memory |
	memory := Bitmap new: 3.
	memory longAt: 1 put: 16rE3A03001 bigEndian: false. "MOV r3, #1"
	memory longAt: 5 put: 16rE3530001 bigEndian: false. "CMP r3, #1"
	memory := memory asByteArray.
	self processor
		disassembleInstructionAt: 0 
		In: memory 
		into: [:str :len | 
			self 
				assert: len = 4;
				assert: str = 'mov	r3, #1'].
	self processor
		disassembleInstructionAt: 4 
		In: memory 
		into: [:str :len | 
			self 
				assert: len = 4;
				assert: str = 'cmp	r3, #1'].
	self processor
		pc: 0;
		singleStepIn: memory;
		singleStepIn: memory.
	self 
		assert: self processor pc = 16r8;
		assert: self processor r3 = 1;
		assert: self processor zflag = 1;
		assert: self processor cflag = 1;
		assert: self processor vflag = 0;
		assert: self processor nflag = 0.
	self processor reset.
	self assert: self processor eflags = 3. "IFFlags are both set."! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 8/18/2012 14:24'!
testNfib1
	"self new testNfib1"
	self should: [self runNFib: 1 disassemble: false printRegisters: false]
		raise: Error
		withExceptionDo: 
			[:err| self assert: err messageText = 'Error 0: Illegal Instruction fetch address (0x1000).'].
	self deny: (self processor pc between: 0 and: self nfib size).
	self assert: self processor r0 = 1 benchFib! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/11/2012 14:21'!
testNfib16
	"self new testNfib16"
	self should: [self runNFib: 16 disassemble: false printRegisters: false]
		raise: Error.
	self deny: (self processor pc between: 0 and: self nfib size).
	self assert: self processor r0 = 16 benchFib! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/10/2012 14:29'!
testNfib2
	"self new testNfib2"
	self should: [self runNFib: 2 disassemble: false printRegisters: false]
		raise: Error.
	self deny: (self processor pc between: 0 and: self nfib size).
	self assert: self processor r0 = 2 benchFib! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/11/2012 14:20'!
testNfib4
	"self new testNfib4"
	self should: [self runNFib: 4 disassemble: false printRegisters: false]
		raise: Error.
	self deny: (self processor pc between: 0 and: self nfib size).
	self assert: self processor r0 = 4 benchFib! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'eem 1/20/2009 12:58'!
testResetCPU
	"self new testResetCPU"
	self registerSetters do:
		[:setter|
		self processor perform: setter with: 16r55555555].
	self registerGetters do:
		[:getter|
		self assert: 16r55555555 = (self processor perform: getter)].
	self processor reset.
	self registerGetters do:
		[:getter|
		self assert: 0 = (self processor perform: getter)]! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/11/2012 13:27'!
testStepNfib1
	"self new testStepNfib1"
	self singleStepNFib: 1 disassemble: false printRegisters: false.
	self assert: self processor pc = (self nfib asWordArray size * 4).
	self assert: self processor r0 = 1 benchFib! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/11/2012 14:19'!
testStepNfib2
	"self new testStepNfib2"
	self singleStepNFib: 2 disassemble: false printRegisters: false.
	self assert: self processor pc = (self nfib size * 4).
	self assert: self processor r0 = 2 benchFib! !

!GdbARMAlienTests methodsFor: 'tests' stamp: 'lw 7/11/2012 14:20'!
testStepNfib4
	"self new testStepNfib4"
	self singleStepNFib: 4 disassemble: false printRegisters: false.
	self assert: self processor pc = (self nfib size * 4).
	self assert: self processor r0 = 4 benchFib! !

SmartSyntaxInterpreterPlugin subclass: #BochsIA32Plugin
	instanceVariableNames: 'prevInterruptCheckChain'
	classVariableNames: ''
	poolDictionaries: 'VMBasicConstants'
	category: 'Cog-ProcessorPlugins'!

!BochsIA32Plugin commentStamp: '<historical>' prior: 0!
I provide access to the Bochs C++ IA32 processor emulator.!

!BochsIA32Plugin class methodsFor: 'translation' stamp: 'eem 1/15/2009 15:23'!
declareCVarsIn: aCCodeGenerator
	"prevInterruptCheckChain lives in sqBochsIA32Plugin.c"
	super declareCVarsIn: aCCodeGenerator.
	aCCodeGenerator removeVariable: 'prevInterruptCheckChain'! !

!BochsIA32Plugin class methodsFor: 'translation' stamp: 'eem 12/3/2008 10:25'!
hasHeaderFile
	"We need a header to declare newcpu and pull in bochs.h & cpu.h"
	^true! !

!BochsIA32Plugin methodsFor: 'interruption' stamp: 'eem 10/20/2010 12:16'!
forceStopOnInterrupt
	interpreterProxy getInterruptPending ifTrue:
		[self forceStopRunning]! !

!BochsIA32Plugin methodsFor: 'primitives' stamp: 'eem 10/5/2009 14:00'!
"cpuAlien <BochsIA32Alien>" primitiveDisassembleAt: address "<Integer>" InMemory: memory "<Bitmap|ByteArray|WordArray>"
	"Return an Array of the instruction length and its decompilation as a string for the instruction at address in memory."
	| cpuAlien cpu instrLenOrErr resultObj log logLen logObj logObjData |
	<var: #cpu type: #'void *'>
	cpuAlien := self primitive: #primitiveDisassembleAtInMemory
					parameters: #(Unsigned WordsOrBytes)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	instrLenOrErr := self disassembleFor: cpu
						At: address
						In: memory
						Size: (interpreterProxy byteSizeOf: memory cPtrAsOop).
	instrLenOrErr < 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	log := self getlog: (self cCode: [self addressOf: logLen] inSmalltalk: [logLen := 0]).
	resultObj := interpreterProxy instantiateClass: interpreterProxy classArray indexableSize: 2.
	resultObj = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrNoMemory].

	"Easier keeping the damn thing on the stack than using pushRemappableOop:/popRemappableOop.
	 Where is topRemappableOop when you need it?"
	interpreterProxy pushRemappableOop: resultObj.
	logObj := interpreterProxy
				instantiateClass: interpreterProxy classString
				indexableSize: logLen.
	interpreterProxy failed ifTrue:
		[interpreterProxy popRemappableOop.
		 ^interpreterProxy primitiveFailFor: PrimErrNoMemory].
	logObjData := interpreterProxy arrayValueOf: logObj.
	self cCode: 'memcpy(logObjData, log, logLen)' inSmalltalk: [logObjData. log].
	resultObj := interpreterProxy popRemappableOop.
	interpreterProxy
		storePointer: 0
		ofObject: resultObj
		withValue: (interpreterProxy integerObjectOf: instrLenOrErr).
	interpreterProxy storePointer: 1 ofObject: resultObj withValue: logObj.

	^resultObj! !

!BochsIA32Plugin methodsFor: 'primitives' stamp: 'eem 10/5/2009 14:00'!
primitiveErrorAndLog
	| log logLen resultObj logObj logObjData |
	<var: #log type: #'char *'>
	<var: #logObjData type: #'char *'>
	self primitive: #primitiveErrorAndLog parameters: #().

	log := self getlog: (self cCode: [self addressOf: logLen] inSmalltalk: [logLen := 0]).
	resultObj := interpreterProxy instantiateClass: interpreterProxy classArray indexableSize: 2.
	resultObj = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrNoMemory].

	interpreterProxy
		storePointer: 0
		ofObject: resultObj
		withValue: (interpreterProxy integerObjectOf: self errorAcorn).

	logLen > 0 ifTrue:
		[interpreterProxy pushRemappableOop: resultObj.
		logObj := interpreterProxy
					instantiateClass: interpreterProxy classString
					indexableSize: logLen.
		interpreterProxy failed ifTrue:
			[interpreterProxy popRemappableOop.
			 ^interpreterProxy primitiveFailFor: PrimErrNoMemory].

		resultObj := interpreterProxy popRemappableOop.
		logObjData := interpreterProxy arrayValueOf: logObj.
		self cCode: 'memcpy(logObjData, log, logLen)' inSmalltalk: [logObjData. log].
		interpreterProxy storePointer: 1 ofObject: resultObj withValue: logObj].
	interpreterProxy pop: 1 thenPush: resultObj! !

!BochsIA32Plugin methodsFor: 'primitives' stamp: 'eem 1/15/2009 09:57'!
"cpuAlien <BochsIA32Alien>" primitiveFlushICacheFrom: startAddress "<Integer>" To: endAddress "<Integer>"
	"Flush the icache in the requested range"
	| cpuAlien cpu |
	<var: #cpu type: 'void *'>
	cpuAlien := self primitive: #primitiveFlushICacheFromTo
					parameters: #(Unsigned Unsigned)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	self flushICache: cpu From: startAddress To: endAddress! !

!BochsIA32Plugin methodsFor: 'primitives' stamp: 'eem 1/14/2009 15:28'!
primitiveNewCPU
	| cpu |
	<var: #cpu type: 'void *'>
	self primitive: #primitiveNewCPU parameters: #().

	cpu := self cCode: 'newCPU()' inSmalltalk: [0].
	cpu = 0 ifTrue:
		[^interpreterProxy primitiveFail].
	interpreterProxy
		pop: 1
		thenPush: (interpreterProxy positive32BitIntegerFor:
										(self cCoerceSimple: cpu
											to: 'unsigned long'))! !

!BochsIA32Plugin methodsFor: 'primitives' stamp: 'eem 1/14/2009 15:28'!
primitiveResetCPU
	| cpuAlien cpu maybeErr |
	<var: #cpu type: 'void *'>
	cpuAlien := self primitive: #primitiveResetCPU parameters: #() receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	maybeErr := self resetCPU: cpu.
	maybeErr ~= 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	^cpuAlien! !

!BochsIA32Plugin methodsFor: 'primitives' stamp: 'eem 10/20/2010 16:08'!
"cpuAlien <BochsIA32Alien>" primitiveRunInMemory: memory "<Bitmap|ByteArray|WordArray>" minimumAddress: minAddress "<Integer>" readOnlyBelow: minWriteMaxExecAddress "<Integer>"
	"Run the cpu using the first argument as the memory and the following arguments defining valid addresses, running until it halts or hits an exception."
	| cpuAlien cpu maybeErr |
	<var: #cpu type: #'void *'>
	cpuAlien := self primitive: #primitiveRunInMemoryMinimumAddressReadWrite
					parameters: #(WordsOrBytes Unsigned Unsigned)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	prevInterruptCheckChain := interpreterProxy setInterruptCheckChain: #forceStopOnInterrupt asSymbol.
	prevInterruptCheckChain = #forceStopOnInterrupt asSymbol ifTrue:
		[prevInterruptCheckChain = 0].
	maybeErr := self runCPU: cpu
					In: memory
					Size: (interpreterProxy byteSizeOf: memory cPtrAsOop)
					MinAddressRead: minAddress
					Write: minWriteMaxExecAddress.
	interpreterProxy setInterruptCheckChain: prevInterruptCheckChain.
	maybeErr ~= 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	^cpuAlien! !

!BochsIA32Plugin methodsFor: 'primitives' stamp: 'eem 10/20/2010 16:08'!
"cpuAlien <BochsIA32Alien>" primitiveSingleStepInMemory: memory "<Bitmap|ByteArray|WordArray>" minimumAddress: minAddress "<Integer>"  readOnlyBelow: minWriteMaxExecAddress "<Integer>"
	"Single-step the cpu using the first argument as the memory and the following arguments defining valid addresses."
	| cpuAlien cpu maybeErr |
	<var: #cpu type: #'void *'>
	cpuAlien := self primitive: #primitiveSingleStepInMemoryMinimumAddressReadWrite
					parameters: #(WordsOrBytes Unsigned Unsigned)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	maybeErr := self singleStepCPU: cpu
					In: memory
					Size: (interpreterProxy byteSizeOf: memory cPtrAsOop)
					MinAddressRead: minAddress
					Write: minWriteMaxExecAddress.
	maybeErr ~= 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	^cpuAlien! !

!BochsIA32Plugin methodsFor: 'alien support' stamp: 'eem (auto pragmas 12/08) 12/11/2008 09:57'!
sizeField: rcvr
	"Answer the first field of rcvr which is assumed to be an Alien of at least 8 bytes"
	<inline: true>
	^self longAt: rcvr + BaseHeaderSize! !

!BochsIA32Plugin methodsFor: 'alien support' stamp: 'eem (auto pragmas 12/08) 12/11/2008 09:55'!
startOfData: rcvr "<Alien oop> ^<Integer>"
	"Answer the start of rcvr's data.  For direct aliens this is the address of
	 the second field.  For indirect and pointer aliens it is what the second field points to."
	<inline: true>
	^(self sizeField: rcvr) > 0
	 	ifTrue: [rcvr + BaseHeaderSize + BytesPerOop]
		ifFalse: [self longAt: rcvr + BaseHeaderSize + BytesPerOop]! !

SmartSyntaxInterpreterPlugin subclass: #GdbARMPlugin
	instanceVariableNames: 'prevInterruptCheckChain'
	classVariableNames: ''
	poolDictionaries: 'VMBasicConstants'
	category: 'Cog-ProcessorPlugins'!

!GdbARMPlugin commentStamp: '<historical>' prior: 0!
I provide access to the ARMulator ARM emulator and the libopcodes ARM disassembler.!

!GdbARMPlugin class methodsFor: 'translation' stamp: 'eem 1/15/2009 15:23'!
declareCVarsIn: aCCodeGenerator
	"prevInterruptCheckChain lives in sqGdbARMPlugin.c"
	super declareCVarsIn: aCCodeGenerator.
	aCCodeGenerator removeVariable: 'prevInterruptCheckChain'! !

!GdbARMPlugin class methodsFor: 'translation' stamp: 'eem 12/3/2008 10:25'!
hasHeaderFile
	"We need a header to declare newcpu and pull in bochs.h & cpu.h"
	^true! !

!GdbARMPlugin methodsFor: 'interruption' stamp: 'eem 10/20/2010 12:16'!
forceStopOnInterrupt
	interpreterProxy getInterruptPending ifTrue:
		[self forceStopRunning]! !

!GdbARMPlugin methodsFor: 'primitives' stamp: 'eem 10/5/2009 14:00'!
"cpuAlien <GdbARMAlien>" primitiveDisassembleAt: address "<Integer>" InMemory: memory "<Bitmap|ByteArray|WordArray>"
	"Return an Array of the instruction length and its decompilation as a string for the instruction at address in memory."
	| cpuAlien cpu instrLenOrErr resultObj log logLen logObj logObjData |
	<var: #cpu type: #'void *'>
	cpuAlien := self primitive: #primitiveDisassembleAtInMemory
					parameters: #(Unsigned WordsOrBytes)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	instrLenOrErr := self disassembleFor: cpu
						At: address
						In: memory
						Size: (interpreterProxy byteSizeOf: memory cPtrAsOop).
	instrLenOrErr < 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	log := self getlog: (self cCode: [self addressOf: logLen] inSmalltalk: [logLen := 0]).
	resultObj := interpreterProxy instantiateClass: interpreterProxy classArray indexableSize: 2.
	resultObj = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrNoMemory].

	"Easier keeping the damn thing on the stack than using pushRemappableOop:/popRemappableOop.
	 Where is topRemappableOop when you need it?"
	interpreterProxy pushRemappableOop: resultObj.
	logObj := interpreterProxy
				instantiateClass: interpreterProxy classString
				indexableSize: logLen.
	interpreterProxy failed ifTrue:
		[interpreterProxy popRemappableOop.
		 ^interpreterProxy primitiveFailFor: PrimErrNoMemory].
	logObjData := interpreterProxy arrayValueOf: logObj.
	self cCode: 'memcpy(logObjData, log, logLen)' inSmalltalk: [logObjData. log].
	resultObj := interpreterProxy popRemappableOop.
	interpreterProxy
		storePointer: 0
		ofObject: resultObj
		withValue: (interpreterProxy integerObjectOf: instrLenOrErr).
	interpreterProxy storePointer: 1 ofObject: resultObj withValue: logObj.

	^resultObj! !

!GdbARMPlugin methodsFor: 'primitives' stamp: 'eem 10/5/2009 14:00'!
primitiveErrorAndLog
	| log logLen resultObj logObj logObjData |
	<var: #log type: #'char *'>
	<var: #logObjData type: #'char *'>
	self primitive: #primitiveErrorAndLog parameters: #().

	log := self getlog: (self cCode: [self addressOf: logLen] inSmalltalk: [logLen := 0]).
	resultObj := interpreterProxy instantiateClass: interpreterProxy classArray indexableSize: 2.
	resultObj = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrNoMemory].

	interpreterProxy
		storePointer: 0
		ofObject: resultObj
		withValue: (interpreterProxy integerObjectOf: self errorAcorn).

	logLen > 0 ifTrue:
		[interpreterProxy pushRemappableOop: resultObj.
		logObj := interpreterProxy
					instantiateClass: interpreterProxy classString
					indexableSize: logLen.
		interpreterProxy failed ifTrue:
			[interpreterProxy popRemappableOop.
			 ^interpreterProxy primitiveFailFor: PrimErrNoMemory].

		resultObj := interpreterProxy popRemappableOop.
		logObjData := interpreterProxy arrayValueOf: logObj.
		self cCode: 'memcpy(logObjData, log, logLen)' inSmalltalk: [logObjData. log].
		interpreterProxy storePointer: 1 ofObject: resultObj withValue: logObj].
	interpreterProxy pop: 1 thenPush: resultObj! !

!GdbARMPlugin methodsFor: 'primitives' stamp: 'eem 1/15/2009 09:57'!
"cpuAlien <GdbARMAlien>" primitiveFlushICacheFrom: startAddress "<Integer>" To: endAddress "<Integer>"
	"Flush the icache in the requested range"
	| cpuAlien cpu |
	<var: #cpu type: 'void *'>
	cpuAlien := self primitive: #primitiveFlushICacheFromTo
					parameters: #(Unsigned Unsigned)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	self flushICache: cpu From: startAddress To: endAddress! !

!GdbARMPlugin methodsFor: 'primitives' stamp: 'eem 1/14/2009 15:28'!
primitiveNewCPU
	| cpu |
	<var: #cpu type: 'void *'>
	self primitive: #primitiveNewCPU parameters: #().

	cpu := self cCode: 'newCPU()' inSmalltalk: [0].
	cpu = 0 ifTrue:
		[^interpreterProxy primitiveFail].
	interpreterProxy
		pop: 1
		thenPush: (interpreterProxy positive32BitIntegerFor:
										(self cCoerceSimple: cpu
											to: 'unsigned long'))! !

!GdbARMPlugin methodsFor: 'primitives' stamp: 'eem 1/14/2009 15:28'!
primitiveResetCPU
	| cpuAlien cpu maybeErr |
	<var: #cpu type: 'void *'>
	cpuAlien := self primitive: #primitiveResetCPU parameters: #() receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	maybeErr := self resetCPU: cpu.
	maybeErr ~= 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	^cpuAlien! !

!GdbARMPlugin methodsFor: 'primitives' stamp: 'eem 10/20/2010 16:08'!
"cpuAlien <GdbARMAlien>" primitiveRunInMemory: memory "<Bitmap|ByteArray|WordArray>" minimumAddress: minAddress "<Integer>" readOnlyBelow: minWriteMaxExecAddress "<Integer>"
	"Run the cpu using the first argument as the memory and the following arguments defining valid addresses, running until it halts or hits an exception."
	| cpuAlien cpu maybeErr |
	<var: #cpu type: #'void *'>
	cpuAlien := self primitive: #primitiveRunInMemoryMinimumAddressReadWrite
					parameters: #(WordsOrBytes Unsigned Unsigned)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	prevInterruptCheckChain := interpreterProxy setInterruptCheckChain: #forceStopOnInterrupt asSymbol.
	prevInterruptCheckChain = #forceStopOnInterrupt asSymbol ifTrue:
		[prevInterruptCheckChain = 0].
	maybeErr := self runCPU: cpu
					In: memory
					Size: (interpreterProxy byteSizeOf: memory cPtrAsOop)
					MinAddressRead: minAddress
					Write: minWriteMaxExecAddress.
	interpreterProxy setInterruptCheckChain: prevInterruptCheckChain.
	maybeErr ~= 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	^cpuAlien! !

!GdbARMPlugin methodsFor: 'primitives' stamp: 'eem 10/20/2010 16:08'!
"cpuAlien <GdbARMAlien>" primitiveSingleStepInMemory: memory "<Bitmap|ByteArray|WordArray>" minimumAddress: minAddress "<Integer>"  readOnlyBelow: minWriteMaxExecAddress "<Integer>"
	"Single-step the cpu using the first argument as the memory and the following arguments defining valid addresses."
	| cpuAlien cpu maybeErr |
	<var: #cpu type: #'void *'>
	cpuAlien := self primitive: #primitiveSingleStepInMemoryMinimumAddressReadWrite
					parameters: #(WordsOrBytes Unsigned Unsigned)
					receiver: #Oop.
	(cpu := self startOfData: cpuAlien) = 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrBadReceiver].
	maybeErr := self singleStepCPU: cpu
					In: memory
					Size: (interpreterProxy byteSizeOf: memory cPtrAsOop)
					MinAddressRead: minAddress
					Write: minWriteMaxExecAddress.
	maybeErr ~= 0 ifTrue:
		[^interpreterProxy primitiveFailFor: PrimErrInappropriate].
	^cpuAlien! !

!GdbARMPlugin methodsFor: 'alien support' stamp: 'eem (auto pragmas 12/08) 12/11/2008 09:57'!
sizeField: rcvr
	"Answer the first field of rcvr which is assumed to be an Alien of at least 8 bytes"
	<inline: true>
	^self longAt: rcvr + BaseHeaderSize! !

!GdbARMPlugin methodsFor: 'alien support' stamp: 'eem (auto pragmas 12/08) 12/11/2008 09:55'!
startOfData: rcvr "<Alien oop> ^<Integer>"
	"Answer the start of rcvr's data.  For direct aliens this is the address of
	 the second field.  For indirect and pointer aliens it is what the second field points to."
	<inline: true>
	^(self sizeField: rcvr) > 0
	 	ifTrue: [rcvr + BaseHeaderSize + BytesPerOop]
		ifFalse: [self longAt: rcvr + BaseHeaderSize + BytesPerOop]! !
CommandLineLauncher initialize!
BochsIA32Alien initialize!
GdbARMAlien initialize!
ShootoutChameneosColour initialize!
ShootoutMall initialize!
